@@ -37,37 +37,51 @@ extern char **environ;
 //
 
 #  if CUPS_VERSION_MAJOR < 3
+#    define CUPS_ENCODING_ISO8859_1	CUPS_ISO8859_1
+#    define CUPS_ENCODING_JIS_X0213	CUPS_JIS_X0213
 #    define cups_len_t int
 #    define cups_page_header_t cups_page_header2_t
 #    define cupsArrayNew cupsArrayNew3
-#    define cupsArrayGetCount (size_t)cupsArrayCount
+#    define cupsArrayGetCount cupsArrayCount
 #    define cupsArrayGetElement(a,n) cupsArrayIndex(a,(int)n)
 #    define cupsArrayGetFirst cupsArrayFirst
 #    define cupsArrayGetLast cupsArrayLast
 #    define cupsArrayGetNext cupsArrayNext
 #    define cupsArrayGetPrev cupsArrayPrev
 #    define cupsGetUser cupsUser
+#    define cupsLangGetName(lang)	lang->language
 #    define cupsRasterReadHeader cupsRasterReadHeader2
 #    define cupsRasterWriteHeader cupsRasterWriteHeader2
+#    define cupsTempFd(prefix,suffix,buffer,bufsize) cupsTempFd(buffer,bufsize)
 #    define httpAddrConnect httpAddrConnect2
+#    define httpAddrGetFamily httpAddrFamily
+#    define httpAddrGetLength httpAddrLength
+#    define httpAddrGetString httpAddrString
+#    define httpAddrIsLocalhost httpAddrLocalhost
 #    define httpConnect httpConnect2
-#    define httpDecode64 httpDecode64_2
-#    define httpEncode64 httpEncode64_2
+#    define httpDecode64(out,outlen,in,end) httpDecode64_2(out,outlen,in)
+#    define httpEncode64(out,outlen,in,inlen,url) httpEncode64_2(out,outlen,in,inlen)
 #    define httpGetDateString httpGetDateString2
 #    define httpRead httpRead2
 #    define httpReconnect httpReconnect2
+#    define httpSetEncryption(http,e) (httpEncryption(http,e)>=0)
 #    define httpStatusString httpStatus
 #    define httpWrite httpWrite2
+#    define httpWriteResponse(http,code) (httpWriteResponse(http,code) == 0)
+#    define ippGetFirstAttribute ippFirstAttribute
+#    define ippGetNextAttribute ippNextAttribute
 #    define IPP_NUM_CAST (int)
 typedef cups_array_func_t cups_array_cb_t;
 typedef cups_acopy_func_t cups_acopy_cb_t;
 typedef cups_afree_func_t cups_afree_cb_t;
 typedef cups_raster_iocb_t cups_raster_cb_t;
+typedef ipp_copycb_t ipp_copy_cb_t;
 #    if CUPS_VERSION_MINOR < 3
 #      define HTTP_STATUS_FOUND (http_status_t)302
 #    endif // CUPS_VERSION_MINOR < 3
 #  else
 #    define cups_len_t size_t
+#    define cups_utf8_t char
 #    define IPP_NUM_CAST (size_t)
 #  endif // CUPS_VERSION_MAJOR < 3
 
@@ -78,8 +92,14 @@ typedef cups_raster_iocb_t cups_raster_cb_t;
 
 #  ifdef DEBUG
 #    define _PAPPL_DEBUG(...) fprintf(stderr, __VA_ARGS__)
+#    define _papplRWLockRead(obj) fprintf(stderr, "%p/%s: rdlock %p(%s)\n", (void *)pthread_self(), __func__, (void *)obj, obj->name), pthread_rwlock_rdlock(&obj->rwlock)
+#    define _papplRWLockWrite(obj) fprintf(stderr, "%p/%s: wrlock %p(%s)\n", (void *)pthread_self(), __func__, (void *)obj, obj->name), pthread_rwlock_wrlock(&obj->rwlock)
+#    define _papplRWUnlock(obj) fprintf(stderr, "%p/%s: unlock %p(%s)\n", (void *)pthread_self(), __func__, (void *)obj, obj->name), pthread_rwlock_unlock(&obj->rwlock)
 #  else
 #    define _PAPPL_DEBUG(...)
+#    define _papplRWLockRead(obj) pthread_rwlock_rdlock(&obj->rwlock)
+#    define _papplRWLockWrite(obj) pthread_rwlock_wrlock(&obj->rwlock)
+#    define _papplRWUnlock(obj) pthread_rwlock_unlock(&obj->rwlock)
 #  endif // DEBUG
 
 #  define _PAPPL_LOC(s) s
@@ -87,6 +107,15 @@ typedef cups_raster_iocb_t cups_raster_cb_t;
 #  define _PAPPL_LOOKUP_VALUE(keyword,strings) _papplLookupValue(keyword, sizeof(strings) / sizeof(strings[0]), strings)
 
 
+//
+// Macros to implement a simple Fibonacci sequence for variable back-off...
+//
+
+#  define _PAPPL_FIB_NEXT(v) (((((v >> 8) + (v & 255) - 1) % 60) + 1) | ((v & 255) << 8))
+#  define _PAPPL_FIB_VALUE(v) (v & 255)
+
+
+
 //
 // Types and structures...
 //
