@@ -286,7 +286,7 @@ _papplClientGetAuthWebScheme(
     // Use HTTPS if non-TLS is disabled...
     return ("https");
   }
-  else if (httpAddrLocalhost(httpGetAddress(client->http)))
+  else if (httpAddrIsLocalhost(httpGetAddress(client->http)))
   {
     // Use HTTP over loopback interface...
     return ("http");
@@ -304,6 +304,21 @@ _papplClientGetAuthWebScheme(
 }
 
 
+//
+// 'papplClientIsEncrypted()' - Return whether a Client connection is encrypted.
+//
+// This function returns a boolean value indicating whether a Client connection
+// is encrypted with TLS.
+//
+
+bool					// O - `true` if encrypted, `false` otherwise
+papplClientIsEncrypted(
+    pappl_client_t *client)		// I - Client
+{
+  return (client ? httpIsEncrypted(client->http) : false);
+}
+
+
 //
 // 'papplClientSetUsername()' - Set the authenticated username, if any.
 //
