@@ -91,7 +91,7 @@ _papplClientProcessIPP(
   {
     papplClientRespondIPP(client, IPP_STATUS_ERROR_BAD_REQUEST, "Bad request-id %d.", ippGetRequestId(client->request));
   }
-  else if (!ippFirstAttribute(client->request))
+  else if (!ippGetFirstAttribute(client->request))
   {
     papplClientRespondIPP(client, IPP_STATUS_ERROR_BAD_REQUEST, "No attributes in request.");
   }
@@ -99,7 +99,7 @@ _papplClientProcessIPP(
   {
     // Make sure that the attributes are provided in the correct order and
     // don't repeat groups...
-    for (attr = ippFirstAttribute(client->request), group = ippGetGroupTag(attr); attr; attr = ippNextAttribute(client->request))
+    for (attr = ippGetFirstAttribute(client->request), group = ippGetGroupTag(attr); attr; attr = ippGetNextAttribute(client->request))
     {
       if (ippGetGroupTag(attr) < group && ippGetGroupTag(attr) != IPP_TAG_ZERO)
       {
@@ -118,14 +118,14 @@ _papplClientProcessIPP(
       //   attributes-charset
       //   attributes-natural-language
       //   system-uri/printer-uri/job-uri
-      attr = ippFirstAttribute(client->request);
+      attr = ippGetFirstAttribute(client->request);
       name = ippGetName(attr);
       if (attr && name && !strcmp(name, "attributes-charset") && ippGetValueTag(attr) == IPP_TAG_CHARSET)
 	charset = attr;
       else
 	charset = NULL;
 
-      attr = ippNextAttribute(client->request);
+      attr = ippGetNextAttribute(client->request);
       name = ippGetName(attr);
 
       if (attr && name && !strcmp(name, "attributes-natural-language") && ippGetValueTag(attr) == IPP_TAG_LANGUAGE)
@@ -240,7 +240,10 @@ _papplClientProcessIPP(
 
   // Send the HTTP header and return...
   if (httpGetState(client->http) != HTTP_STATE_POST_SEND)
-    _papplClientFlushDocumentData(client);	// Flush trailing (junk) data
+  {
+    // Flush trailing (junk) data
+    _papplClientFlushDocumentData(client);
+  }
 
   if (httpGetState(client->http) != HTTP_STATE_WAITING)
     return (papplClientRespond(client, HTTP_STATUS_OK, NULL, "application/ipp", 0, ippLength(client->response)));
@@ -271,7 +274,8 @@ papplClientRespondIPP(
   const char	*formatted = NULL;	// Formatted message
 
 
-  ippSetStatusCode(client->response, status);
+  if (status > ippGetStatusCode(client->response))
+    ippSetStatusCode(client->response, status);
 
   if (message)
   {
@@ -297,6 +301,29 @@ papplClientRespondIPP(
 }
 
 
+//
+// '_papplClientRespondIPPIgnored()' - Respond with an ignored IPP attribute.
+//
+// This function returns a 'successful-ok-ignored-or-substituted-attributes'
+// status code and adds the specified attribute to the unsupported attributes
+// group in the response.
+//
+
+void
+_papplClientRespondIPPIgnored(
+    pappl_client_t  *client,		// I - Client
+    ipp_attribute_t *attr)		// I - Atribute
+{
+  ipp_attribute_t	*temp;		// Copy of attribute
+
+
+  papplClientRespondIPP(client, IPP_STATUS_OK_IGNORED_OR_SUBSTITUTED, "Ignoring unsupported %s %s%s value.", ippGetName(attr), ippGetCount(attr) > 1 ? "1setOf " : "", ippTagString(ippGetValueTag(attr)));
+
+  temp = ippCopyAttribute(client->response, attr, 0);
+  ippSetGroupTag(client->response, &temp, IPP_TAG_UNSUPPORTED_GROUP);
+}
+
+
 //
 // 'papplClientRespondIPPUnsupported()' - Respond with an unsupported IPP attribute.
 //
