@@ -70,9 +70,9 @@ _papplClientCreate(
 
   client->system = system;
 
-  pthread_rwlock_wrlock(&system->rwlock);
+  _papplRWLockWrite(system);
   client->number = system->next_client ++;
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 
   // Accept the client and get the remote address...
   if ((client->http = httpAcceptConnection(sock, 1)) == NULL)
@@ -112,7 +112,7 @@ _papplClientCreateTempFile(
   }
 
   // Write the data to a temporary file...
-  if ((fd = cupsTempFd(tempfile, sizeof(tempfile))) < 0)
+  if ((fd = cupsTempFd(NULL, NULL, tempfile, sizeof(tempfile))) < 0)
   {
     papplLogClient(client, PAPPL_LOGLEVEL_ERROR, "Unable to create temporary file: %s", strerror(errno));
     return (NULL);
@@ -171,9 +171,9 @@ _papplClientDelete(
   free(client);
 
   // Update the number of active clients...
-  pthread_rwlock_wrlock(&system->rwlock);
+  _papplRWLockWrite(system);
   system->num_clients --;
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 }
 
 
@@ -197,6 +197,8 @@ _papplClientProcessHTTP(
   int			port;		// Port number
   char			*ptr;		// Pointer into string
   _pappl_resource_t	*resource;	// Current resource
+  char			system_host[HTTP_MAX_HOST];
+					// System hostname
 
 
   // Clear state variables...
@@ -284,17 +286,17 @@ _papplClientProcessHTTP(
     client->host_port = (int)strtol(ptr, &end, 10);
 
     if (errno == ERANGE || *end)
-      client->host_port = client->system->port;
+      client->host_port = papplSystemGetHostPort(client->system);
   }
   else
   {
     // Use the default port number...
-    client->host_port = client->system->port;
+    client->host_port = papplSystemGetHostPort(client->system);
   }
 
   ptr = strstr(client->host_field, ".local");
 
-  if (!isdigit(client->host_field[0] & 255) && client->host_field[0] != '[' && strcmp(client->host_field, client->system->hostname) && strcmp(client->host_field, "localhost") && (!ptr || (strcmp(ptr, ".local") && strcmp(ptr, ".local."))))
+  if (!isdigit(client->host_field[0] & 255) && client->host_field[0] != '[' && strcmp(client->host_field, papplSystemGetHostName(client->system, system_host, sizeof(system_host))) && strcmp(client->host_field, "localhost") && (!ptr || (strcmp(ptr, ".local") && strcmp(ptr, ".local."))))
   {
     papplLogClient(client, PAPPL_LOGLEVEL_ERROR, "Bad Host: header '%s'.", client->host_field);
     papplClientRespond(client, HTTP_STATUS_BAD_REQUEST, NULL, NULL, 0, 0);
@@ -311,7 +313,7 @@ _papplClientProcessHTTP(
 
       papplLogClient(client, PAPPL_LOGLEVEL_INFO, "Upgrading to encrypted connection.");
 
-      if (httpEncryption(client->http, HTTP_ENCRYPTION_REQUIRED))
+      if (!httpSetEncryption(client->http, HTTP_ENCRYPTION_REQUIRED))
       {
 	papplLogClient(client, PAPPL_LOGLEVEL_ERROR, "Unable to encrypt connection: %s", cupsLastErrorString());
 	return (false);
@@ -501,7 +503,7 @@ papplClientRespond(
   if (code == HTTP_STATUS_CONTINUE)
   {
     // 100-continue doesn't send any headers...
-    return (httpWriteResponse(client->http, HTTP_STATUS_CONTINUE) == 0);
+    return (httpWriteResponse(client->http, HTTP_STATUS_CONTINUE));
   }
 
   // Format an error message...
@@ -555,7 +557,7 @@ papplClientRespond(
     httpSetField(client->http, HTTP_FIELD_LOCATION, redirect);
   }
 
-  if (httpWriteResponse(client->http, code) < 0)
+  if (!httpWriteResponse(client->http, code))
     return (false);
 
   // Send the response data...
@@ -609,9 +611,9 @@ papplClientRespondRedirect(
     char	url[1024];		// Absolute URL
 
     if (*path == '/')
-      httpAssembleURI(HTTP_URI_CODING_ALL, url, sizeof(url), "https", NULL, client->host_field, client->host_port, path);
+      httpAssembleURI(HTTP_URI_CODING_ALL, url, sizeof(url), httpIsEncrypted(client->http) ? "https" : "http", NULL, client->host_field, client->host_port, path);
     else
-      httpAssembleURIf(HTTP_URI_CODING_ALL, url, sizeof(url), "https", NULL, client->host_field, client->host_port, "/%s", path);
+      httpAssembleURIf(HTTP_URI_CODING_ALL, url, sizeof(url), httpIsEncrypted(client->http) ? "https" : "http", NULL, client->host_field, client->host_port, "/%s", path);
 
     httpSetField(client->http, HTTP_FIELD_LOCATION, url);
   }
@@ -621,7 +623,7 @@ papplClientRespondRedirect(
     httpSetField(client->http, HTTP_FIELD_LOCATION, path);
   }
 
-  if (httpWriteResponse(client->http, code) < 0)
+  if (!httpWriteResponse(client->http, code))
     return (false);
 
   return (httpWrite(client->http, "", 0) >= 0);
@@ -651,7 +653,7 @@ _papplClientRun(
       {
         papplLogClient(client, PAPPL_LOGLEVEL_INFO, "Starting HTTPS session.");
 
-	if (httpEncryption(client->http, HTTP_ENCRYPTION_ALWAYS))
+	if (!httpSetEncryption(client->http, HTTP_ENCRYPTION_ALWAYS))
 	{
           papplLogClient(client, PAPPL_LOGLEVEL_ERROR, "Unable to encrypt connection: %s", cupsLastErrorString());
 	  break;
