@@ -108,6 +108,7 @@ typedef struct _pappl_dns_sd_dev_t	// DNS-SD browse data
 #ifdef HAVE_AVAHI
   AvahiRecordBrowser	*ref;			// Browser for query
 #endif // HAVE_AVAHI
+  pthread_mutex_t	mutex;			// Update lock
   char			*name,			// Service name
 			*domain,		// Domain name
 			*fullName,		// Full name
@@ -211,7 +212,7 @@ static ssize_t		pappl_socket_read(pappl_device_t *device, void *buffer, size_t b
 static pappl_preason_t	pappl_socket_status(pappl_device_t *device);
 static int		pappl_socket_supplies(pappl_device_t *device, int max_supplies, pappl_supply_t *supplies);
 static ssize_t		pappl_socket_write(pappl_device_t *device, const void *buffer, size_t bytes);
-static void		utf16_to_utf8(cups_utf8_t *dst, const unsigned char *src, size_t srcsize, size_t dstsize, bool le);
+static void		utf16_to_utf8(char *dst, const unsigned char *src, size_t srcsize, size_t dstsize, bool le);
 
 
 //
@@ -313,6 +314,7 @@ pappl_dnssd_free(_pappl_dns_sd_dev_t *d)// I - Device
   free(d->make_and_model);
   free(d->device_id);
   free(d->uuid);
+  pthread_mutex_destroy(&d->mutex);
   free(d);
 }
 
@@ -368,6 +370,8 @@ pappl_dnssd_get_device(
   if ((device = calloc(sizeof(_pappl_dns_sd_dev_t), 1)) == NULL)
     return (NULL);
 
+  pthread_mutex_init(&device->mutex, NULL);
+
   if ((device->name = strdup(serviceName)) == NULL)
   {
     free(device);
@@ -422,7 +426,7 @@ pappl_dnssd_list(
 					// Network device name
 			device_uri[1024];
 					// Network device URI
-  size_t		last_count;	// Last number of devices
+  cups_len_t		last_count;	// Last number of devices
   int			timeout;	// Timeout counter
 #  ifdef HAVE_MDNSRESPONDER
   int			error;		// Error code, if any
@@ -477,6 +481,17 @@ pappl_dnssd_list(
 
   _PAPPL_DEBUG("pappl_dnssd_find: timeout=%d, last_count=%u\n", timeout, (unsigned)last_count);
 
+  // Stop browsing...
+  _papplDNSSDLock();
+
+#  ifdef HAVE_MDNSRESPONDER
+  DNSServiceRefDeallocate(pdl_ref);
+#  else
+  avahi_service_browser_free(pdl_ref);
+#  endif // HAVE_MDNSRESPONDER
+
+  _papplDNSSDUnlock();
+
   // Do the callback for each of the devices...
   for (device = (_pappl_dns_sd_dev_t *)cupsArrayGetFirst(devices); device; device = (_pappl_dns_sd_dev_t *)cupsArrayGetNext(devices))
   {
@@ -494,17 +509,7 @@ pappl_dnssd_list(
     }
   }
 
-  // Stop browsing and free memory...
-  _papplDNSSDLock();
-
-#  ifdef HAVE_MDNSRESPONDER
-  DNSServiceRefDeallocate(pdl_ref);
-#  else
-  avahi_service_browser_free(pdl_ref);
-#  endif // HAVE_MDNSRESPONDER
-
-  _papplDNSSDUnlock();
-
+  // Free memory and return...
   cupsArrayDelete(devices);
 
   return (ret);
@@ -734,8 +739,10 @@ pappl_dnssd_query_cb(
   snprintf(device_id, sizeof(device_id), "MFG:%s;MDL:%s;CMD:%s;", mfg, mdl, cmd);
 
   // Save the make and model and IEEE-1284 device ID...
+  pthread_mutex_lock(&device->mutex);
   device->device_id      = strdup(device_id);
   device->make_and_model = strdup(ty);
+  pthread_mutex_unlock(&device->mutex);
 }
 
 
@@ -758,6 +765,7 @@ pappl_dnssd_resolve_cb(
     void                *context)	// I - Device
 {
   (void)sdRef;
+  (void)flags;
   (void)interfaceIndex;
   (void)fullname;
   (void)txtLen;
@@ -881,7 +889,7 @@ pappl_snmp_find(
   bool			ret = false;	// Return value
   cups_array_t		*devices = NULL;//  Device array
   int			snmp_sock = -1;	// SNMP socket
-  size_t		last_count;	// Last devices count
+  cups_len_t		last_count;	// Last devices count
   fd_set		input;		// Input set for select()
   struct timeval	timeout;	// Timeout for select()
   time_t		endtime;	// End time for scan
@@ -917,7 +925,7 @@ pappl_snmp_find(
   // Send queries to every broadcast address...
   for (addr = addrs; addr; addr = addr->next)
   {
-    _PAPPL_DEBUG("pappl_snmp_find: Sending SNMP device type get request to '%s'.\n", httpAddrString(&(addr->addr), temp, sizeof(temp)));
+    _PAPPL_DEBUG("pappl_snmp_find: Sending SNMP device type get request to '%s'.\n", httpAddrGetString(&(addr->addr), temp, sizeof(temp)));
 
     _papplSNMPWrite(snmp_sock, &(addr->addr), _PAPPL_SNMP_VERSION_1, _PAPPL_SNMP_COMMUNITY, _PAPPL_ASN1_GET_REQUEST, _PAPPL_SNMP_QUERY_DEVICE_TYPE, DeviceTypeOID);
   }
@@ -994,7 +1002,7 @@ pappl_snmp_find(
       // Save the address and port...
       char	address_str[256];	// IP address as a string
 
-      sock->host = strdup(httpAddrString(&cur_device->address, address_str, sizeof(address_str)));
+      sock->host = strdup(httpAddrGetString(&cur_device->address, address_str, sizeof(address_str)));
       sock->port = cur_device->port;
       ret        = true;
       break;
@@ -1153,7 +1161,7 @@ pappl_snmp_read_response(
     return;
   }
 
-  httpAddrString(&(packet.address), addrname, sizeof(addrname));
+  httpAddrGetString(&(packet.address), addrname, sizeof(addrname));
 
   // Look for the response status code in the SNMP message header
   if (packet.error)
@@ -1396,25 +1404,25 @@ pappl_snmp_walk_cb(
 
 	    case _PAPPL_TC_csISOLatin1 :
 	    case _PAPPL_TC_csUnicodeLatin1 :
-		cupsCharsetToUTF8((cups_utf8_t *)sock->supplies[i].description, (char *)packet->object_value.string.bytes, sizeof(sock->supplies[i].description), CUPS_ISO8859_1);
+		cupsCharsetToUTF8((cups_utf8_t *)sock->supplies[i].description, (char *)packet->object_value.string.bytes, (cups_len_t)sizeof(sock->supplies[i].description), CUPS_ENCODING_ISO8859_1);
 		break;
 
 	    case _PAPPL_TC_csShiftJIS :
 	    case _PAPPL_TC_csWindows31J : /* Close enough for our purposes */
-		cupsCharsetToUTF8((cups_utf8_t *)sock->supplies[i].description, (char *)packet->object_value.string.bytes, sizeof(sock->supplies[i].description), CUPS_JIS_X0213);
+		cupsCharsetToUTF8((cups_utf8_t *)sock->supplies[i].description, (char *)packet->object_value.string.bytes, (cups_len_t)sizeof(sock->supplies[i].description), CUPS_ENCODING_JIS_X0213);
 		break;
 
 	    case _PAPPL_TC_csUCS4 :
 	    case _PAPPL_TC_csUTF32 :
 	    case _PAPPL_TC_csUTF32BE :
 	    case _PAPPL_TC_csUTF32LE :
-		cupsUTF32ToUTF8((cups_utf8_t *)sock->supplies[i].description, (cups_utf32_t *)packet->object_value.string.bytes, sizeof(sock->supplies[i].description));
+		cupsUTF32ToUTF8((cups_utf8_t *)sock->supplies[i].description, (cups_utf32_t *)packet->object_value.string.bytes, (cups_len_t)sizeof(sock->supplies[i].description));
 		break;
 
 	    case _PAPPL_TC_csUnicode :
 	    case _PAPPL_TC_csUTF16BE :
 	    case _PAPPL_TC_csUTF16LE :
-		utf16_to_utf8((cups_utf8_t *)sock->supplies[i].description, packet->object_value.string.bytes, packet->object_value.string.num_bytes, sizeof(sock->supplies[i].description), sock->charset == _PAPPL_TC_csUTF16LE);
+		utf16_to_utf8(sock->supplies[i].description, packet->object_value.string.bytes, packet->object_value.string.num_bytes, sizeof(sock->supplies[i].description), sock->charset == _PAPPL_TC_csUTF16LE);
 		break;
 
 	    default :
@@ -1685,7 +1693,7 @@ pappl_socket_open(
   }
 
   // Open SNMP socket...
-  if ((sock->snmp_fd = _papplSNMPOpen(httpAddrFamily(&(sock->addr->addr)))) < 0)
+  if ((sock->snmp_fd = _papplSNMPOpen(httpAddrGetFamily(&(sock->addr->addr)))) < 0)
   {
     papplDeviceError(device, "Unable to open SNMP socket.");
     return (false);
@@ -1956,7 +1964,7 @@ pappl_socket_write(
 
 static void
 utf16_to_utf8(
-    cups_utf8_t         *dst,		// I - Destination buffer
+    char                *dst,		// I - Destination buffer
     const unsigned char *src,		// I - Source string
     size_t		srcsize,	// I - Size of source string
     size_t              dstsize,	// I - Size of destination buffer
@@ -2004,5 +2012,5 @@ utf16_to_utf8(
 
   *ptr = '\0';
 
-  cupsUTF32ToUTF8(dst, temp, (cups_len_t)dstsize);
+  cupsUTF32ToUTF8((cups_utf8_t *)dst, temp, (cups_len_t)dstsize);
 }
