@@ -229,7 +229,7 @@ _papplDeviceAddSupportedSchemes(
 
   pthread_rwlock_rdlock(&device_rwlock);
 
-  attr = ippAddStrings(attrs, IPP_TAG_SYSTEM, IPP_TAG_URISCHEME, "smi2699-device-uri-schemes-supported", IPP_NUM_CAST cupsArrayGetCount(device_schemes), NULL, NULL);
+  attr = ippAddStrings(attrs, IPP_TAG_SYSTEM, IPP_TAG_URISCHEME, "smi55357-device-uri-schemes-supported", IPP_NUM_CAST cupsArrayGetCount(device_schemes), NULL, NULL);
 
   for (i = 0, devscheme = (_pappl_devscheme_t *)cupsArrayGetFirst(device_schemes); devscheme; i ++, devscheme = (_pappl_devscheme_t *)cupsArrayGetNext(device_schemes))
     ippSetString(attrs, &attr, i, devscheme->scheme);
@@ -792,13 +792,17 @@ papplDevicePrintf(
 {
   va_list	ap;			// Pointer to additional args
   char		buffer[8192];		// Output buffer
+  int		bytes;			// Bytes to write
 
 
   va_start(ap, format);
-  vsnprintf(buffer, sizeof(buffer), format, ap);
+  bytes = vsnprintf(buffer, sizeof(buffer), format, ap);
   va_end(ap);
 
-  return (papplDeviceWrite(device, buffer, strlen(buffer)));
+  if (bytes > 0)
+    return (papplDeviceWrite(device, buffer, (size_t)bytes));
+  else
+    return ((ssize_t)bytes);
 }
 
 
