@@ -34,15 +34,17 @@ typedef struct _pappl_attr_s		// Input attribute structure
 static void		ipp_cancel_job(pappl_client_t *client);
 static void		ipp_close_job(pappl_client_t *client);
 static void		ipp_get_job_attributes(pappl_client_t *client);
+static void		ipp_hold_job(pappl_client_t *client);
+static void		ipp_release_job(pappl_client_t *client);
 static void		ipp_send_document(pappl_client_t *client);
 
 
 //
-// '_papplJobCopyAttributes()' - Copy job attributes to the response.
+// '_papplJobCopyAttributesNoLock()' - Copy job attributes to the response.
 //
 
 void
-_papplJobCopyAttributes(
+_papplJobCopyAttributesNoLock(
     pappl_job_t    *job,		// I - Job
     pappl_client_t *client,		// I - Client
     cups_array_t   *ra)			// I - requested-attributes
@@ -77,7 +79,7 @@ _papplJobCopyAttributes(
   if (!ra || cupsArrayFind(ra, "job-printer-up-time"))
     ippAddInteger(client->response, IPP_TAG_JOB, IPP_TAG_INTEGER, "job-printer-up-time", (int)(time(NULL) - client->printer->start_time));
 
-  _papplJobCopyState(job, IPP_TAG_JOB, client->response, ra);
+  _papplJobCopyStateNoLock(job, IPP_TAG_JOB, client->response, ra);
 
   if (!ra || cupsArrayFind(ra, "time-at-creation"))
     ippAddInteger(client->response, IPP_TAG_JOB, IPP_TAG_INTEGER, "time-at-creation", (int)(job->created - client->printer->start_time));
@@ -111,15 +113,28 @@ _papplJobCopyDocumentData(
   // server-error-busy...
   if (!strcmp(job->format, "image/pwg-raster") || !strcmp(job->format, "image/urf"))
   {
+    _papplRWLockRead(job->printer);
+
     if (job->printer->processing_job)
     {
       papplClientRespondIPP(client, IPP_STATUS_ERROR_BUSY, "Currently printing another job.");
-      _papplClientFlushDocumentData(client);
-      return;
+      _papplRWUnlock(job->printer);
+      goto abort_job;
+    }
+    else if (job->printer->hold_new_jobs)
+    {
+      papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_ACCEPTING_JOBS, "Currently holding new jobs.");
+      _papplRWUnlock(job->printer);
+      goto abort_job;
     }
 
+    _papplRWLockWrite(job);
+
     job->state = IPP_JSTATE_PENDING;
 
+    _papplRWUnlock(job);
+    _papplRWUnlock(job->printer);
+
     _papplJobProcessRaster(job, client);
 
     goto complete_job;
@@ -201,27 +216,36 @@ _papplJobCopyDocumentData(
   cupsArrayAdd(ra, "job-state-reasons");
   cupsArrayAdd(ra, "job-uri");
 
-  _papplJobCopyAttributes(job, client, ra);
+  _papplRWLockRead(job);
+  _papplJobCopyAttributesNoLock(job, client, ra);
+  _papplRWUnlock(job);
+
   cupsArrayDelete(ra);
   return;
 
   // If we get here we had to abort the job...
   abort_job:
 
+  papplLogJob(job, PAPPL_LOGLEVEL_INFO, "Aborting job.");
+
   _papplClientFlushDocumentData(client);
 
+  _papplRWLockWrite(client->printer);
+  _papplRWLockWrite(job);
+
   job->state     = IPP_JSTATE_ABORTED;
   job->completed = time(NULL);
 
-  pthread_rwlock_wrlock(&client->printer->rwlock);
-
   cupsArrayRemove(client->printer->active_jobs, job);
   cupsArrayAdd(client->printer->completed_jobs, job);
 
+  _papplRWLockWrite(client->system);
   if (!client->system->clean_time)
     client->system->clean_time = time(NULL) + 60;
+  _papplRWUnlock(client->system);
 
-  pthread_rwlock_unlock(&client->printer->rwlock);
+  _papplRWUnlock(job);
+  _papplRWUnlock(client->printer);
 
   ra = cupsArrayNew((cups_array_cb_t)strcmp, NULL, NULL, 0, NULL, NULL);
   cupsArrayAdd(ra, "job-id");
@@ -229,17 +253,17 @@ _papplJobCopyDocumentData(
   cupsArrayAdd(ra, "job-state-reasons");
   cupsArrayAdd(ra, "job-uri");
 
-  _papplJobCopyAttributes(job, client, ra);
+  _papplJobCopyAttributesNoLock(job, client, ra);
   cupsArrayDelete(ra);
 }
 
 
 //
-// '_papplJobCopyState()' - Copy the job-state-xxx sttributes.
+// '_papplJobCopyStateNoLock()' - Copy the job-state-xxx sttributes.
 //
 
 void
-_papplJobCopyState(
+_papplJobCopyStateNoLock(
     pappl_job_t    *job,	// I - Job
     ipp_tag_t      group_tag,	// I - Group tag
     ipp_t          *ipp,	// I - IPP message
@@ -366,20 +390,28 @@ _papplJobProcessIPP(
 {
   switch (ippGetOperation(client->request))
   {
-    case IPP_OP_SEND_DOCUMENT :
-	ipp_send_document(client);
-	break;
-
     case IPP_OP_CANCEL_JOB :
 	ipp_cancel_job(client);
 	break;
 
+    case IPP_OP_CLOSE_JOB :
+	ipp_close_job(client);
+	break;
+
     case IPP_OP_GET_JOB_ATTRIBUTES :
 	ipp_get_job_attributes(client);
 	break;
 
-    case IPP_OP_CLOSE_JOB :
-	ipp_close_job(client);
+    case IPP_OP_HOLD_JOB :
+	ipp_hold_job(client);
+	break;
+
+    case IPP_OP_RELEASE_JOB :
+	ipp_release_job(client);
+	break;
+
+    case IPP_OP_SEND_DOCUMENT :
+	ipp_send_document(client);
 	break;
 
     default :
@@ -506,7 +538,7 @@ _papplJobValidateDocumentAttributes(
     }
   }
 
-  pthread_rwlock_rdlock(&client->printer->rwlock);
+  _papplRWLockRead(client->printer);
 
   if (op != IPP_OP_CREATE_JOB && (supported = ippFindAttribute(client->printer->attrs, "document-format-supported", IPP_TAG_MIMETYPE)) != NULL && !ippContainsString(supported, format))
   {
@@ -514,7 +546,7 @@ _papplJobValidateDocumentAttributes(
     valid = false;
   }
 
-  pthread_rwlock_unlock(&client->printer->rwlock);
+  _papplRWUnlock(client->printer);
 
   return (valid);
 }
@@ -646,11 +678,92 @@ ipp_get_job_attributes(
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
 
   ra = ippCreateRequestedArray(client->request);
-  _papplJobCopyAttributes(job, client, ra);
+  _papplJobCopyAttributesNoLock(job, client, ra);
   cupsArrayDelete(ra);
 }
 
 
+//
+// 'ipp_hold_job()' - Hold a job.
+//
+
+static void
+ipp_hold_job(pappl_client_t *client)	// I - Client
+{
+  pappl_job_t	*job = client->job;	// Job information
+
+
+  // Authorize access...
+  if (!_papplPrinterIsAuthorized(client))
+    return;
+
+  // Get the job...
+  if (job)
+  {
+    const char	*hold_until;		// "job-hold-until" keyword
+    time_t	hold_until_time;	// "job-hold-until-time" value
+
+    hold_until      = ippGetString(ippFindAttribute(client->request, "job-hold-until", IPP_TAG_KEYWORD), 0, NULL);
+    hold_until_time = ippDateToTime(ippGetDate(ippFindAttribute(client->request, "job-hold-until-time", IPP_TAG_DATE), 0));
+
+    if (!hold_until && !hold_until_time)
+      hold_until = "indefinite";
+
+    // "job-hold-until" = 'no-hold' means release the job...
+    if (hold_until && !strcmp(hold_until, "no-hold"))
+    {
+      if (papplJobRelease(job, client->username))
+	papplClientRespondIPP(client, IPP_STATUS_OK, "Job released.");
+      else
+	papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_POSSIBLE, "Job already released.");
+    }
+    else
+    {
+      // Otherwise hold with the specified values...
+      if (papplJobHold(job, client->username, hold_until, hold_until_time))
+	papplClientRespondIPP(client, IPP_STATUS_OK, "Job held.");
+      else
+	papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_POSSIBLE, "Job not pending/held.");
+    }
+  }
+  else
+  {
+    // Not found...
+    papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_FOUND, "Job does not exist.");
+  }
+}
+
+
+//
+// 'ipp_release_job()' - Release a job.
+//
+
+static void
+ipp_release_job(pappl_client_t *client)	// I - Client
+{
+  pappl_job_t	*job = client->job;	// Job information
+
+
+  // Authorize access...
+  if (!_papplPrinterIsAuthorized(client))
+    return;
+
+  // Get the job...
+  if (job)
+  {
+    if (papplJobRelease(job, client->username))
+      papplClientRespondIPP(client, IPP_STATUS_OK, "Job released.");
+    else
+      papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_POSSIBLE, "Job not held.");
+  }
+  else
+  {
+    // Not found...
+    papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_FOUND, "Job does not exist.");
+  }
+}
+
+
 //
 // 'ipp_send_document()' - Add an attached document to a job object created with
 //                         Create-Job.
@@ -728,7 +841,7 @@ ipp_send_document(
     job->state = IPP_JSTATE_ABORTED;
 
   // Then finish getting the document data and process things...
-  pthread_rwlock_wrlock(&(client->printer->rwlock));
+  _papplRWLockWrite(client->printer);
 
   _papplCopyAttributes(job->attrs, client->request, NULL, IPP_TAG_JOB, 0);
 
@@ -739,7 +852,7 @@ ipp_send_document(
   else
     job->format = client->printer->driver_data.format;
 
-  pthread_rwlock_unlock(&(client->printer->rwlock));
+  _papplRWUnlock(client->printer);
 
   if (have_data)
     _papplJobCopyDocumentData(client, job);
