@@ -49,6 +49,7 @@ papplJobCreatePrintOptions(
   pappl_printer_t	*printer = job->printer;
 					// Printer
   const char		*raster_type;	// Raster type for output
+#if CUPS_VERSION_MAJOR < 3
   static const char * const media_positions[] =
   {					// "media-source" to MediaPosition mapping
     "auto",
@@ -110,6 +111,7 @@ papplJobCreatePrintOptions(
     CUPS_ORIENT_180,
     CUPS_ORIENT_0
   };
+#endif // CUPS_VERSION_MAJOR < 3
   static const char * const sheet_back[] =
   {					// "pwg-raster-document-sheet-back values
     "normal",
@@ -127,7 +129,7 @@ papplJobCreatePrintOptions(
 
   options->media = printer->driver_data.media_default;
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
 
   // copies
   if ((attr = ippFindAttribute(job->attrs, "copies", IPP_TAG_INTEGER)) != NULL)
@@ -413,6 +415,7 @@ papplJobCreatePrintOptions(
     memcpy(options->dither, printer->driver_data.gdither, sizeof(options->dither));
 
   // Generate the raster header...
+#if CUPS_VERSION_MAJOR < 3
   cupsRasterInitPWGHeader(&options->header, pwgMediaForPWG(options->media.size_name), raster_type, options->printer_resolution[0], options->printer_resolution[1], _papplSidesString(options->sides), sheet_back[printer->driver_data.duplex]);
   for (i = 0; i < (int)(sizeof(media_positions) / sizeof(media_positions[0])); i ++)
   {
@@ -434,6 +437,25 @@ papplJobCreatePrintOptions(
   options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxBottom] = (unsigned)options->media.bottom_margin * options->header.HWResolution[1] / 2540;
   options->header.cupsInteger[CUPS_RASTER_PWG_PrintQuality]   = options->print_quality;
 
+#else // CUPS 3.x has a new API for this...
+  cups_size_t	media;			// CUPS media value
+
+  memset(&media, 0, sizeof(media));
+
+  papplCopyString(media.media, options->media.size_name, sizeof(media.media));
+  papplCopyString(media.source, options->media.source, sizeof(media.source));
+  papplCopyString(media.type, options->media.type, sizeof(media.type));
+
+  media.width  = options->media.size_width;
+  media.length = options->media.size_length;
+  media.bottom = options->media.bottom_margin;
+  media.left   = options->media.left_margin;
+  media.right  = options->media.right_margin;
+  media.top    = options->media.top_margin;
+
+  cupsRasterInitHeader(&options->header, &media, _papplContentString(options->print_content_optimize), options->print_quality, /*intent*/NULL, options->orientation_requested, _papplSidesString(options->sides), raster_type, options->printer_resolution[0], options->printer_resolution[1], sheet_back[printer->driver_data.duplex]);
+#endif // CUPS_VERSION_MAJOR < 3
+
   // Log options...
   papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "header.cupsWidth=%u", options->header.cupsWidth);
   papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "header.cupsHeight=%u", options->header.cupsHeight);
@@ -477,7 +499,7 @@ papplJobCreatePrintOptions(
   for (i = 0; i < (cups_len_t)options->num_vendor; i ++)
     papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "%s=%s", options->vendor[i].name, options->vendor[i].value);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   return (options);
 }
@@ -945,8 +967,8 @@ finish_job(pappl_job_t  *job)		// I - Job
 					// Printer
 
 
-  pthread_rwlock_wrlock(&printer->rwlock);
-  pthread_rwlock_wrlock(&job->rwlock);
+  _papplRWLockWrite(printer);
+  _papplRWLockWrite(job);
 
   if (job->is_canceled)
     job->state = IPP_JSTATE_CANCELED;
@@ -965,7 +987,7 @@ finish_job(pappl_job_t  *job)		// I - Job
 
   _papplSystemAddEventNoLock(job->system, job->printer, job, PAPPL_EVENT_JOB_COMPLETED, NULL);
 
-  pthread_rwlock_unlock(&job->rwlock);
+  _papplRWUnlock(job);
 
   if (printer->is_stopped)
   {
@@ -994,7 +1016,7 @@ finish_job(pappl_job_t  *job)		// I - Job
   if (printer->max_preserved_jobs > 0)
     _papplPrinterCleanJobsNoLock(printer);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 
@@ -1002,25 +1024,26 @@ finish_job(pappl_job_t  *job)		// I - Job
   {
     papplPrinterDelete(printer);
   }
-  else if (cupsArrayGetCount(printer->active_jobs) > 0)
-  {
-    _papplPrinterCheckJobs(printer);
-  }
-  else
+  else if (!strncmp(printer->device_uri, "file:", 5) || cupsArrayGetCount(printer->active_jobs) == 0)
   {
     pappl_devmetrics_t	metrics;	// Metrics for device IO
 
-    pthread_rwlock_wrlock(&printer->rwlock);
+    _papplRWLockWrite(printer);
 
     papplDeviceGetMetrics(printer->device, &metrics);
     papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "Device read metrics: %lu requests, %lu bytes, %lu msecs", (unsigned long)metrics.read_requests, (unsigned long)metrics.read_bytes, (unsigned long)metrics.read_msecs);
     papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "Device write metrics: %lu requests, %lu bytes, %lu msecs", (unsigned long)metrics.write_requests, (unsigned long)metrics.write_bytes, (unsigned long)metrics.write_msecs);
 
+    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Closing device for job %d.", job->job_id);
+
     papplDeviceClose(printer->device);
     printer->device = NULL;
 
-    pthread_rwlock_unlock(&printer->rwlock);
+    _papplRWUnlock(printer);
   }
+
+  if (cupsArrayGetCount(printer->active_jobs) > 0)
+    _papplPrinterCheckJobs(printer);
 }
 
 
@@ -1037,8 +1060,8 @@ start_job(pappl_job_t *job)		// I - Job
 
 
   // Move the job to the 'processing' state...
-  pthread_rwlock_wrlock(&printer->rwlock);
-  pthread_rwlock_wrlock(&job->rwlock);
+  _papplRWLockWrite(printer);
+  _papplRWLockWrite(job);
 
   papplLogJob(job, PAPPL_LOGLEVEL_INFO, "Starting print job.");
 
@@ -1048,11 +1071,25 @@ start_job(pappl_job_t *job)		// I - Job
 
   _papplSystemAddEventNoLock(printer->system, printer, job, PAPPL_EVENT_JOB_STATE_CHANGED, NULL);
 
-  pthread_rwlock_unlock(&job->rwlock);
+  _papplRWUnlock(job);
 
   // Open the output device...
+  if (printer->device_in_use)
+  {
+    papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "Waiting for device to become available.");
+
+    while (printer->device_in_use && !printer->is_deleted && !job->is_canceled && papplSystemIsRunning(printer->system))
+    {
+      _papplRWUnlock(printer);
+      sleep(1);
+      _papplRWLockWrite(printer);
+    }
+  }
+
   while (!printer->device && !printer->is_deleted && !job->is_canceled && papplSystemIsRunning(printer->system))
   {
+    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Opening device for job %d.", job->job_id);
+
     printer->device = papplDeviceOpen(printer->device_uri, job->name, papplLogDevice, job->system);
 
     if (!printer->device && !printer->is_deleted && !job->is_canceled)
@@ -1066,10 +1103,14 @@ start_job(pappl_job_t *job)		// I - Job
 	printer->state      = IPP_PSTATE_STOPPED;
 	printer->state_time = time(NULL);
       }
+      else
+      {
+        papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Still unable to open device.");
+      }
 
-      pthread_rwlock_unlock(&printer->rwlock);
+      _papplRWUnlock(printer);
       sleep(5);
-      pthread_rwlock_wrlock(&printer->rwlock);
+      _papplRWLockWrite(printer);
     }
   }
 
@@ -1077,9 +1118,9 @@ start_job(pappl_job_t *job)		// I - Job
   {
     job->state = IPP_JSTATE_PENDING;
 
-    pthread_rwlock_rdlock(&job->rwlock);
+    _papplRWLockRead(job);
     _papplSystemAddEventNoLock(job->system, job->printer, job, PAPPL_EVENT_JOB_STATE_CHANGED, NULL);
-    pthread_rwlock_unlock(&job->rwlock);
+    _papplRWUnlock(job);
   }
 
   if (printer->device)
@@ -1091,7 +1132,7 @@ start_job(pappl_job_t *job)		// I - Job
 
   _papplSystemAddEventNoLock(printer->system, printer, NULL, PAPPL_EVENT_PRINTER_STATE_CHANGED, NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   return (printer->device != NULL);
 }
