@@ -31,7 +31,7 @@ struct _pappl_loc_s			// Localization data
 {
   pthread_rwlock_t	rwlock;			// Reader/writer lock
   pappl_system_t	*system;		// Associated system
-  char			*language;		// Language/locale name
+  char			*name;			// Language/locale name
   cups_array_t		*pairs;			// String pairs
 };
 
