@@ -1,8 +1,7 @@
 //
 // Private header file for the Printer Application Framework
 //
-// Copyright © 2019-2020 by Michael R Sweet.
-// Copyright © 2010-2019 by Apple Inc.
+// Copyright © 2019-2022 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
@@ -10,18 +9,13 @@
 
 #ifndef _PAPPL_PAPPL_PRIVATE_H_
 #  define _PAPPL_PAPPL_PRIVATE_H_
-
-//
-// Include necessary headers...
-//
-
+#  include "client-private.h"
 #  include "device.h"
 #  include "dnssd-private.h"
-#  include "system-private.h"
-#  include "client-private.h"
-#  include "printer-private.h"
 #  include "job-private.h"
-#  include "mainloop-private.h"
+#  include "loc-private.h"
 #  include "log-private.h"
-
+#  include "mainloop-private.h"
+#  include "printer-private.h"
+#  include "system-private.h"
 #endif // !_PAPPL_PAPPL_PRIVATE_H_
