@@ -1,7 +1,7 @@
 //
 // Public header file for the Printer Application Framework
 //
-// Copyright © 2019-2020 by Michael R Sweet.
+// Copyright © 2019-2022 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
@@ -9,17 +9,12 @@
 
 #ifndef _PAPPL_PAPPL_H_
 #  define _PAPPL_PAPPL_H_
-
-//
-// Include necessary headers...
-//
-
 #  include "device.h"
 #  include "system.h"
 #  include "client.h"
 #  include "printer.h"
 #  include "job.h"
+#  include "loc.h"
 #  include "log.h"
 #  include "mainloop.h"
-
 #endif // !_PAPPL_PAPPL_H_
