@@ -12,6 +12,7 @@
 //
 
 #include "printer-private.h"
+#include "job-private.h"
 #include "system-private.h"
 
 
@@ -26,17 +27,28 @@ void
 papplPrinterCloseDevice(
     pappl_printer_t *printer)		// I - Printer
 {
-  if (!printer || !printer->device || !printer->device_in_use || printer->processing_job)
+  if (!printer || !printer->device || !printer->device_in_use)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Done using device for status/maintenance.");
 
-  papplDeviceClose(printer->device);
-
-  printer->device        = NULL;
   printer->device_in_use = false;
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  if (cupsArrayGetCount(printer->active_jobs) > 0 && !printer->processing_job)
+    _papplPrinterCheckJobs(printer);
+
+  if (printer->state != IPP_PSTATE_PROCESSING)
+  {
+    _papplRWLockWrite(printer);
+
+    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Closing device.");
+
+    papplDeviceClose(printer->device);
+
+    printer->device = NULL;
+
+    _papplRWUnlock(printer);
+  }
 }
 
 
@@ -96,11 +108,11 @@ papplPrinterGetContact(
     return (contact);
   }
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
 
   *contact = printer->contact;
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   return (contact);
 }
@@ -156,9 +168,9 @@ papplPrinterGetDNSSDName(
     return (NULL);
   }
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
   papplCopyString(buffer, printer->dns_sd_name, bufsize);
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   return (buffer);
 }
@@ -186,9 +198,9 @@ papplPrinterGetGeoLocation(
     return (NULL);
   }
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
   papplCopyString(buffer, printer->geo_location, bufsize);
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   return (buffer);
 }
@@ -245,9 +257,9 @@ papplPrinterGetLocation(
     return (NULL);
   }
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
   papplCopyString(buffer, printer->location, bufsize);
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   return (buffer);
 }
@@ -400,9 +412,9 @@ papplPrinterGetOrganization(
     return (NULL);
   }
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
   papplCopyString(buffer, printer->organization, bufsize);
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   return (buffer);
 }
@@ -429,9 +441,9 @@ papplPrinterGetOrganizationalUnit(
     return (NULL);
   }
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
   papplCopyString(buffer, printer->org_unit, bufsize);
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   return (buffer);
 }
@@ -490,9 +502,9 @@ papplPrinterGetPrintGroup(
     return (NULL);
   }
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
   papplCopyString(buffer, printer->print_group, bufsize);
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   return (buffer);
 }
@@ -571,14 +583,14 @@ papplPrinterGetSupplies(
 
   memset(supplies, 0, (size_t)max_supplies * sizeof(pappl_supply_t));
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
 
   if ((count = printer->num_supply) > max_supplies)
     count = max_supplies;
 
   memcpy(supplies, printer->supply, (size_t)count * sizeof(pappl_supply_t));
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   return (count);
 }
@@ -600,7 +612,49 @@ papplPrinterGetSystem(
 
 
 //
-// '()' - Return whether the printer is accepting jobs.
+// 'papplPrinterHoldNewJobs()' - Hold new jobs for printing.
+//
+// This function holds any new jobs for printing and is typically used prior to
+// performing printer maintenance.  Existing jobs will finish printing but new
+// jobs will be held until you call @link papplPrinterReleaseHeldNewJobs@.
+//
+
+bool					// O - `true` on success, `false` on failure
+papplPrinterHoldNewJobs(
+    pappl_printer_t *printer)		// I - Printer
+{
+  bool	ret = false;			// Return value
+
+
+  // Range check input...
+  if (!printer)
+    return (false);
+
+  // See if we need to change things...
+  _papplRWLockWrite(printer);
+
+  if (!printer->hold_new_jobs)
+  {
+    // Set the 'hold-new-jobs' flag...
+    printer->hold_new_jobs = true;
+    printer->config_time   = time(NULL);
+    ret                    = true;
+
+    // Notify of the change in state...
+    _papplSystemAddEventNoLock(printer->system, printer, NULL, PAPPL_EVENT_PRINTER_CONFIG_CHANGED, "Holding new jobs.");
+  }
+
+  _papplRWUnlock(printer);
+
+  if (ret)
+    _papplSystemConfigChanged(printer->system);
+
+  return (ret);
+}
+
+
+//
+// 'papplPrinterIsAcceptingJobs()' - Return whether the printer is accepting jobs.
 //
 // This function returns a boolean value indicating whether a printer is
 // accepting jobs.
@@ -610,7 +664,73 @@ bool					// O - `true` if the printer is accepting jobs, `false` otherwise
 papplPrinterIsAcceptingJobs(
     pappl_printer_t *printer)		// I - Printer
 {
-  return (printer ? printer->is_accepting : false);
+  bool	is_accepting;			// Return value
+
+
+  // Range check input...
+  if (!printer)
+    return (false);
+
+  // Lock and grab value...
+  _papplRWLockRead(printer);
+  is_accepting = printer->is_accepting;
+  _papplRWUnlock(printer);
+
+  return (is_accepting);
+}
+
+
+//
+// 'papplPrinterIsDeleted()' - Return whether a printer is in the process of being deleted.
+//
+// This function returns a boolean value indicating whether a printer is being
+// deleted.
+//
+
+bool					// O - `true` is printer is being deleted, `false` otherwise
+papplPrinterIsDeleted(
+    pappl_printer_t *printer)		// I - Printer
+{
+  bool	is_deleted;			// Return value
+
+
+  // Range check input...
+  if (!printer)
+    return (false);
+
+  // Lock and grab value...
+  _papplRWLockRead(printer);
+  is_deleted = printer->is_deleted;
+  _papplRWUnlock(printer);
+
+  return (is_deleted);
+}
+
+
+//
+// 'papplPrinterIsHoldingNewJobs()' - Return whether the printer is holding new jobs.
+//
+// This function returns a boolean value indicating whether a printer is
+// holding new jobs.
+//
+
+bool					// O - `true` if the printer is holding new jobs, `false` otherwise
+papplPrinterIsHoldingNewJobs(
+    pappl_printer_t *printer)		// I - Printer
+{
+  bool	hold_new_jobs;			// Return value
+
+
+  // Range check input...
+  if (!printer)
+    return (false);
+
+  // Lock and grab value...
+  _papplRWLockRead(printer);
+  hold_new_jobs = printer->hold_new_jobs;
+  _papplRWUnlock(printer);
+
+  return (hold_new_jobs);
 }
 
 
@@ -635,7 +755,7 @@ papplPrinterIterateActiveJobs(
     int             limit)		// I - Maximum jobs to iterate or `0` for no limit
 {
   pappl_job_t	*job;			// Current job
-  size_t	j,			// Looping var
+  cups_len_t	j,			// Looping var
 		jcount;			// Number of jobs
   int		count;			// Number of jobs iterated
 
@@ -643,7 +763,7 @@ papplPrinterIterateActiveJobs(
   if (!printer || !cb)
     return;
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
 
   // Note: Cannot use cupsArrayGetFirst/Last since other threads might be
   // enumerating the active_jobs array.
@@ -651,14 +771,14 @@ papplPrinterIterateActiveJobs(
   if (limit <= 0)
     limit = INT_MAX;
 
-  for (count = 0, j = (size_t)job_index - 1, jcount = cupsArrayGetCount(printer->active_jobs); j < jcount && count < limit; j ++, count ++)
+  for (count = 0, j = (cups_len_t)job_index - 1, jcount = cupsArrayGetCount(printer->active_jobs); j < jcount && count < limit; j ++, count ++)
   {
     job = (pappl_job_t *)cupsArrayGetElement(printer->active_jobs, j);
 
     (cb)(job, data);
   }
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 }
 
 
@@ -682,7 +802,7 @@ papplPrinterIterateAllJobs(
     int             limit)		// I - Maximum jobs to iterate, `0` for no limit
 {
   pappl_job_t	*job;			// Current job
-  size_t	j,			// Looping var
+  cups_len_t	j,			// Looping var
 		jcount;			// Number of jobs
   int		count;			// Number of jobs iterated
 
@@ -690,7 +810,7 @@ papplPrinterIterateAllJobs(
   if (!printer || !cb)
     return;
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
 
   // Note: Cannot use cupsArrayGetFirst/Last since other threads might be
   // enumerating the all_jobs array.
@@ -698,14 +818,14 @@ papplPrinterIterateAllJobs(
   if (limit <= 0)
     limit = INT_MAX;
 
-  for (count = 0, j = (size_t)job_index - 1, jcount = cupsArrayGetCount(printer->all_jobs); j < jcount && count < limit; j ++, count ++)
+  for (count = 0, j = (cups_len_t)job_index - 1, jcount = cupsArrayGetCount(printer->all_jobs); j < jcount && count < limit; j ++, count ++)
   {
     job = (pappl_job_t *)cupsArrayGetElement(printer->all_jobs, j);
 
     (cb)(job, data);
   }
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 }
 
 
@@ -730,7 +850,7 @@ papplPrinterIterateCompletedJobs(
     int             limit)		// I - Maximum jobs to iterate, `0` for no limit
 {
   pappl_job_t	*job;			// Current job
-  size_t	j,			// Looping var
+  cups_len_t	j,			// Looping var
 		jcount;			// Number of jobs
   int		count;			// Number of jobs iterated
 
@@ -738,7 +858,7 @@ papplPrinterIterateCompletedJobs(
   if (!printer || !cb)
     return;
 
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
 
   // Note: Cannot use cupsArrayGetFirst/Last since other threads might be
   // enumerating the completed_jobs array.
@@ -746,14 +866,14 @@ papplPrinterIterateCompletedJobs(
   if (limit <= 0)
     limit = INT_MAX;
 
-  for (count = 0, j = (size_t)job_index - 1, jcount = cupsArrayGetCount(printer->completed_jobs); j < jcount && count < limit; j ++, count ++)
+  for (count = 0, j = (cups_len_t)job_index - 1, jcount = cupsArrayGetCount(printer->completed_jobs); j < jcount && count < limit; j ++, count ++)
   {
     job = (pappl_job_t *)cupsArrayGetElement(printer->completed_jobs, j);
 
     (cb)(job, data);
   }
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 }
 
 
@@ -777,15 +897,17 @@ papplPrinterOpenDevice(
   if (!printer || printer->device_in_use || printer->processing_job || !printer->device_uri)
     return (NULL);
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   if (!printer->device_in_use && !printer->processing_job)
   {
+    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Opening device for status/maintenance.");
+
     printer->device        = device = papplDeviceOpen(printer->device_uri, "printer", papplLogDevice, printer->system);
     printer->device_in_use = device != NULL;
   }
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   return (device);
 }
@@ -805,7 +927,7 @@ papplPrinterPause(
   if (!printer)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   if (printer->processing_job)
     printer->is_stopped = true;
@@ -814,7 +936,63 @@ papplPrinterPause(
 
   _papplSystemAddEventNoLock(printer->system, printer, NULL, PAPPL_EVENT_PRINTER_STATE_CHANGED | PAPPL_EVENT_PRINTER_STOPPED, NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
+}
+
+
+//
+// 'papplPrinterReleaseHeldNewJobs()' - Release any previously held jobs for printing.
+//
+// This function releases all jobs that were previously held due to a prior
+// call to @link papplPrinterHoldNewJobs@.
+//
+
+bool					// O - `true` on success, `false` on failure
+papplPrinterReleaseHeldNewJobs(
+    pappl_printer_t *printer,		// I - Printer
+    const char      *username)		// I - User that released the held jobs or `NULL` for none/system
+{
+  pappl_job_t	*job;			// Current job
+  bool		ret = false,		// Return value
+		released_jobs = false;	// Have we released any jobs?
+
+
+  // Range check input...
+  if (!printer)
+    return (false);
+
+  // Only release if the printer is holding new jobs...
+  _papplRWLockWrite(printer);
+
+  if (printer->hold_new_jobs)
+  {
+    // Release jobs and clear the 'hold-new-jobs' flag...
+    printer->hold_new_jobs = false;
+    printer->config_time   = time(NULL);
+    ret                    = true;
+
+    _papplSystemAddEventNoLock(printer->system, printer, NULL, PAPPL_EVENT_PRINTER_CONFIG_CHANGED, "Releasing held new jobs.");
+
+    for (job = (pappl_job_t *)cupsArrayGetFirst(printer->active_jobs); job; job = (pappl_job_t *)cupsArrayGetNext(printer->active_jobs))
+    {
+      if (job->state == IPP_JSTATE_HELD && job->hold_until == 0 && !(job->state_reasons & PAPPL_JREASON_JOB_HOLD_UNTIL_SPECIFIED))
+      {
+	// Release this job
+	_papplRWLockWrite(job);
+	_papplJobReleaseNoLock(job, username);
+	_papplRWUnlock(job);
+
+	released_jobs = true;
+      }
+    }
+  }
+
+  _papplRWUnlock(printer);
+
+  if (released_jobs)
+    _papplPrinterCheckJobs(printer);
+
+  return (ret);
 }
 
 
@@ -831,14 +1009,14 @@ papplPrinterResume(
   if (!printer)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   printer->is_stopped = false;
   printer->state      = IPP_PSTATE_IDLE;
 
-  _papplSystemAddEventNoLock(printer->system, printer, NULL, PAPPL_EVENT_PRINTER_STATE_CHANGED, NULL);
+  _papplSystemAddEventNoLock(printer->system, printer, NULL, PAPPL_EVENT_PRINTER_STATE_CHANGED, "Resumed printer.");
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplPrinterCheckJobs(printer);
 }
@@ -858,12 +1036,12 @@ papplPrinterSetContact(
   if (!printer || !contact)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   printer->contact     = *contact;
   printer->config_time = time(NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -884,7 +1062,7 @@ papplPrinterSetDNSSDName(
   if (!printer)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   free(printer->dns_sd_name);
   printer->dns_sd_name      = value ? strdup(value) : NULL;
@@ -897,7 +1075,7 @@ papplPrinterSetDNSSDName(
   else
     _papplPrinterRegisterDNSSDNoLock(printer);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -925,7 +1103,7 @@ papplPrinterSetGeoLocation(
   if (value && sscanf(value, "geo:%f,%f", &lat, &lon) != 2)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   free(printer->geo_location);
   printer->geo_location = value ? strdup(value) : NULL;
@@ -933,7 +1111,7 @@ papplPrinterSetGeoLocation(
 
   _papplPrinterRegisterDNSSDNoLock(printer);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -955,12 +1133,12 @@ papplPrinterSetImpressionsCompleted(
   if (!printer || add <= 0)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   printer->impcompleted += add;
   printer->state_time   = time(NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -981,7 +1159,7 @@ papplPrinterSetLocation(
   if (!printer)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   free(printer->location);
   printer->location    = value ? strdup(value) : NULL;
@@ -989,7 +1167,7 @@ papplPrinterSetLocation(
 
   _papplPrinterRegisterDNSSDNoLock(printer);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -1014,12 +1192,12 @@ papplPrinterSetMaxActiveJobs(
   if (!printer || max_active_jobs < 0)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   printer->max_active_jobs = max_active_jobs;
   printer->config_time     = time(NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -1041,12 +1219,12 @@ papplPrinterSetMaxCompletedJobs(
   if (!printer || max_completed_jobs < 0)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   printer->max_completed_jobs = max_completed_jobs;
   printer->config_time        = time(NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -1068,12 +1246,12 @@ papplPrinterSetMaxPreservedJobs(
   if (!printer || max_preserved_jobs < 0)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   printer->max_preserved_jobs = max_preserved_jobs;
   printer->config_time        = time(NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -1097,12 +1275,12 @@ papplPrinterSetNextJobID(
   if (!printer || next_job_id < 1)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   printer->next_job_id = next_job_id;
   printer->config_time = time(NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -1123,13 +1301,13 @@ papplPrinterSetOrganization(
   if (!printer)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   free(printer->organization);
   printer->organization = value ? strdup(value) : NULL;
   printer->config_time  = time(NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -1150,13 +1328,13 @@ papplPrinterSetOrganizationalUnit(
   if (!printer)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   free(printer->org_unit);
   printer->org_unit    = value ? strdup(value) : NULL;
   printer->config_time = time(NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -1180,7 +1358,7 @@ papplPrinterSetPrintGroup(
   if (!printer)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   free(printer->print_group);
   printer->print_group = value ? strdup(value) : NULL;
@@ -1202,7 +1380,7 @@ papplPrinterSetPrintGroup(
 #endif // !_WIN32
     printer->print_gid = (gid_t)-1;
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   _papplSystemConfigChanged(printer->system);
 }
@@ -1226,13 +1404,13 @@ papplPrinterSetReasons(
   if (!printer)
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   printer->state_reasons &= ~remove;
   printer->state_reasons |= add;
   printer->state_time    = printer->status_time = time(NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 }
 
 
@@ -1251,7 +1429,7 @@ papplPrinterSetSupplies(
   if (!printer || num_supplies < 0 || num_supplies > PAPPL_MAX_SUPPLY || (num_supplies > 0 && !supplies))
     return;
 
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   printer->num_supply = num_supplies;
   memset(printer->supply, 0, sizeof(printer->supply));
@@ -1259,5 +1437,5 @@ papplPrinterSetSupplies(
     memcpy(printer->supply, supplies, (size_t)num_supplies * sizeof(pappl_supply_t));
   printer->state_time = time(NULL);
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 }
