@@ -40,9 +40,11 @@ static void		ipp_disable_printer(pappl_client_t *client);
 static void		ipp_enable_printer(pappl_client_t *client);
 static void		ipp_get_jobs(pappl_client_t *client);
 static void		ipp_get_printer_attributes(pappl_client_t *client);
+static void		ipp_hold_new_jobs(pappl_client_t *client);
 static void		ipp_identify_printer(pappl_client_t *client);
 static void		ipp_pause_printer(pappl_client_t *client);
 static void		ipp_print_job(pappl_client_t *client);
+static void		ipp_release_held_new_jobs(pappl_client_t *client);
 static void		ipp_resume_printer(pappl_client_t *client);
 static void		ipp_set_printer_attributes(pappl_client_t *client);
 static void		ipp_validate_job(pappl_client_t *client);
@@ -51,30 +53,30 @@ static bool		valid_job_attributes(pappl_client_t *client);
 
 
 //
-// '_papplPrinterCopyAttributes()' - Copy printer attributes to a response...
+// '_papplPrinterCopyAttributesNoLock()' - Copy printer attributes to a response...
 //
 
 void
-_papplPrinterCopyAttributes(
+_papplPrinterCopyAttributesNoLock(
     pappl_printer_t *printer,		// I - Printer
     pappl_client_t  *client,		// I - Client
     cups_array_t    *ra,		// I - Requested attributes
     const char      *format)		// I - "document-format" value, if any
 {
-  size_t	i,			// Looping var
+  cups_len_t	i,			// Looping var
 		num_values;		// Number of values
   unsigned	bit;			// Current bit value
   const char	*svalues[100];		// String values
   int		ivalues[100];		// Integer values
   pappl_pr_driver_data_t *data = &printer->driver_data;
 					// Driver data
-  const char	*webscheme = (httpAddrLocalhost(httpGetAddress(client->http)) || !papplSystemGetTLSOnly(client->system)) ? "http" : "https";
+  const char	*webscheme = (httpAddrIsLocalhost(httpGetAddress(client->http)) || !papplSystemGetTLSOnly(client->system)) ? "http" : "https";
 					// URL scheme for resources
 
 
   _papplCopyAttributes(client->response, printer->attrs, ra, IPP_TAG_ZERO, IPP_TAG_CUPS_CONST);
   _papplCopyAttributes(client->response, printer->driver_attrs, ra, IPP_TAG_ZERO, IPP_TAG_CUPS_CONST);
-  _papplPrinterCopyState(printer, IPP_TAG_PRINTER, client->response, client, ra);
+  _papplPrinterCopyStateNoLock(printer, IPP_TAG_PRINTER, client->response, client, ra);
 
   if (!ra || cupsArrayFind(ra, "copies-supported"))
   {
@@ -100,6 +102,9 @@ _papplPrinterCopyAttributes(
       ippAddString(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "identify-actions-default", NULL, "none");
   }
 
+  if (!ra || cupsArrayFind(ra, "job-spooling-supported"))
+    ippAddString(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "job-spooling-supported", NULL, printer->max_active_jobs == 1 ? "stream" : "spool");
+
   if ((!ra || cupsArrayFind(ra, "label-mode-configured")) && data->mode_configured)
     ippAddString(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "label-mode-configured", NULL, _papplLabelModeString(data->mode_configured));
 
@@ -113,7 +118,7 @@ _papplPrinterCopyAttributes(
 
     if (!ra || cupsArrayFind(ra, "marker-colors"))
     {
-      for (i = 0; i < (size_t)printer->num_supply; i ++)
+      for (i = 0; i < (cups_len_t)printer->num_supply; i ++)
         svalues[i] = _papplMarkerColorString(supply[i].color);
 
       ippAddStrings(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_NAME), "marker-colors", IPP_NUM_CAST printer->num_supply, NULL, svalues);
@@ -121,7 +126,7 @@ _papplPrinterCopyAttributes(
 
     if (!ra || cupsArrayFind(ra, "marker-high-levels"))
     {
-      for (i = 0; i < (size_t)printer->num_supply; i ++)
+      for (i = 0; i < (cups_len_t)printer->num_supply; i ++)
         ivalues[i] = supply[i].is_consumed ? 100 : 90;
 
       ippAddIntegers(client->response, IPP_TAG_PRINTER, IPP_TAG_INTEGER, "marker-high-levels", IPP_NUM_CAST printer->num_supply, ivalues);
@@ -129,7 +134,7 @@ _papplPrinterCopyAttributes(
 
     if (!ra || cupsArrayFind(ra, "marker-levels"))
     {
-      for (i = 0; i < (size_t)printer->num_supply; i ++)
+      for (i = 0; i < (cups_len_t)printer->num_supply; i ++)
         ivalues[i] = supply[i].level;
 
       ippAddIntegers(client->response, IPP_TAG_PRINTER, IPP_TAG_INTEGER, "marker-levels", IPP_NUM_CAST printer->num_supply, ivalues);
@@ -137,7 +142,7 @@ _papplPrinterCopyAttributes(
 
     if (!ra || cupsArrayFind(ra, "marker-low-levels"))
     {
-      for (i = 0; i < (size_t)printer->num_supply; i ++)
+      for (i = 0; i < (cups_len_t)printer->num_supply; i ++)
         ivalues[i] = supply[i].is_consumed ? 10 : 0;
 
       ippAddIntegers(client->response, IPP_TAG_PRINTER, IPP_TAG_INTEGER, "marker-low-levels", IPP_NUM_CAST printer->num_supply, ivalues);
@@ -145,7 +150,7 @@ _papplPrinterCopyAttributes(
 
     if (!ra || cupsArrayFind(ra, "marker-names"))
     {
-      for (i = 0; i < (size_t)printer->num_supply; i ++)
+      for (i = 0; i < (cups_len_t)printer->num_supply; i ++)
         svalues[i] = supply[i].description;
 
       ippAddStrings(client->response, IPP_TAG_PRINTER, IPP_TAG_NAME, "marker-names", IPP_NUM_CAST printer->num_supply, NULL, svalues);
@@ -153,7 +158,7 @@ _papplPrinterCopyAttributes(
 
     if (!ra || cupsArrayFind(ra, "marker-types"))
     {
-      for (i = 0; i < (size_t)printer->num_supply; i ++)
+      for (i = 0; i < (cups_len_t)printer->num_supply; i ++)
         svalues[i] = _papplMarkerTypeString(supply[i].type);
 
       ippAddStrings(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "marker-types", IPP_NUM_CAST printer->num_supply, NULL, svalues);
@@ -171,13 +176,13 @@ _papplPrinterCopyAttributes(
 
   if (!ra || cupsArrayFind(ra, "media-col-ready"))
   {
-    size_t		j,		// Looping var
+    cups_len_t		j,		// Looping var
 			count;		// Number of values
     ipp_t		*col;		// Collection value
     ipp_attribute_t	*attr;		// media-col-ready attribute
     pappl_media_col_t	media;		// Current media...
 
-    for (i = 0, count = 0; i < (size_t)printer->num_ready; i ++)
+    for (i = 0, count = 0; i < (cups_len_t)printer->num_ready; i ++)
     {
       if (data->media_ready[i].size_name[0])
         count ++;
@@ -190,7 +195,7 @@ _papplPrinterCopyAttributes(
     {
       attr = ippAddCollections(client->response, IPP_TAG_PRINTER, "media-col-ready", IPP_NUM_CAST count, NULL);
 
-      for (i = 0, j = 0; i < (size_t)printer->num_ready && j < count; i ++)
+      for (i = 0, j = 0; i < (cups_len_t)printer->num_ready && j < count; i ++)
       {
 	if (data->media_ready[i].size_name[0])
 	{
@@ -228,11 +233,11 @@ _papplPrinterCopyAttributes(
 
   if (!ra || cupsArrayFind(ra, "media-ready"))
   {
-    size_t		j,		// Looping vars
+    cups_len_t		j,		// Looping vars
 			count;		// Number of values
     ipp_attribute_t	*attr;		// media-col-ready attribute
 
-    for (i = 0, count = 0; i < (size_t)printer->num_ready; i ++)
+    for (i = 0, count = 0; i < (cups_len_t)printer->num_ready; i ++)
     {
       if (data->media_ready[i].size_name[0])
         count ++;
@@ -242,7 +247,7 @@ _papplPrinterCopyAttributes(
     {
       attr = ippAddStrings(client->response, IPP_TAG_PRINTER, IPP_TAG_KEYWORD, "media-ready", IPP_NUM_CAST count, NULL, NULL);
 
-      for (i = 0, j = 0; i < (size_t)printer->num_ready && j < count; i ++)
+      for (i = 0, j = 0; i < (cups_len_t)printer->num_ready && j < count; i ++)
       {
 	if (data->media_ready[i].size_name[0])
 	  ippSetString(client->response, &attr, IPP_NUM_CAST j ++, data->media_ready[i].size_name);
@@ -297,7 +302,12 @@ _papplPrinterCopyAttributes(
     ippAddDate(client->response, IPP_TAG_PRINTER, "printer-config-change-date-time", ippTimeToDate(printer->config_time));
 
   if (!ra || cupsArrayFind(ra, "printer-config-change-time"))
-    ippAddInteger(client->response, IPP_TAG_PRINTER, IPP_TAG_INTEGER, "printer-config-change-time", (int)(printer->config_time - printer->start_time));
+  {
+    if (printer->config_time > printer->start_time)
+      ippAddInteger(client->response, IPP_TAG_PRINTER, IPP_TAG_INTEGER, "printer-config-change-time", (int)(printer->config_time - printer->start_time));
+    else
+      ippAddInteger(client->response, IPP_TAG_PRINTER, IPP_TAG_INTEGER, "printer-config-change-time", 1);
+  }
 
   if (!ra || cupsArrayFind(ra, "printer-contact-col"))
   {
@@ -315,9 +325,9 @@ _papplPrinterCopyAttributes(
   if (!ra || cupsArrayFind(ra, "printer-dns-sd-name"))
     ippAddString(client->response, IPP_TAG_PRINTER, IPP_TAG_NAME, "printer-dns-sd-name", NULL, printer->dns_sd_name ? printer->dns_sd_name : "");
 
-  pthread_rwlock_rdlock(&client->system->rwlock);
+//  _papplRWLockRead(client->system);
   _papplSystemExportVersions(client->system, client->response, IPP_TAG_PRINTER, ra);
-  pthread_rwlock_unlock(&client->system->rwlock);
+//  _papplRWUnlock(client->system);
 
   if (!ra || cupsArrayFind(ra, "printer-geo-location"))
   {
@@ -352,7 +362,7 @@ _papplPrinterCopyAttributes(
     char		value[256];	// Value for current tray
     pappl_media_col_t	*media;		// Media in the tray
 
-    for (i = 0, media = data->media_ready; i < (size_t)data->num_source; i ++, media ++)
+    for (i = 0, media = data->media_ready; i < (cups_len_t)data->num_source; i ++, media ++)
     {
       const char	*type;		// Tray type
 
@@ -408,20 +418,20 @@ _papplPrinterCopyAttributes(
   if (!ra || cupsArrayFind(ra, "printer-strings-languages-supported"))
   {
     _pappl_resource_t	*r;		// Current resource
-    size_t		rcount;		// Number of resources
+    cups_len_t		rcount;		// Number of resources
 
-    pthread_rwlock_rdlock(&printer->system->rwlock);
+//    _papplRWLockRead(printer->system);
 
     // Cannot use cupsArrayGetFirst/Last since other threads might be iterating
     // this array...
-    for (i = 0, num_values = 0, rcount = cupsArrayGetCount(printer->system->resources); i < rcount && num_values < (sizeof(svalues) / sizeof(svalues[0])); i ++)
+    for (i = 0, num_values = 0, rcount = cupsArrayGetCount(printer->system->resources); i < rcount && num_values < (cups_len_t)(sizeof(svalues) / sizeof(svalues[0])); i ++)
     {
       r = (_pappl_resource_t *)cupsArrayGetElement(printer->system->resources, (size_t)i);
 
       if (r->language)
         svalues[num_values ++] = r->language;
     }
-    pthread_rwlock_unlock(&printer->system->rwlock);
+//    _papplRWUnlock(printer->system);
 
     if (num_values > 0)
       ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_TAG_LANGUAGE, "printer-strings-languages-supported", IPP_NUM_CAST num_values, NULL, svalues);
@@ -434,11 +444,11 @@ _papplPrinterCopyAttributes(
     char	baselang[3],		// Base language
 		uri[1024];		// Strings file URI
     _pappl_resource_t	*r;		// Current resource
-    size_t	rcount;			// Number of resources
+    cups_len_t	rcount;			// Number of resources
 
     papplCopyString(baselang, lang, sizeof(baselang));
 
-    pthread_rwlock_rdlock(&printer->system->rwlock);
+//    _papplRWLockRead(printer->system);
 
     // Cannot use cupsArrayGetFirst/Last since other threads might be iterating
     // this array...
@@ -454,7 +464,7 @@ _papplPrinterCopyAttributes(
       }
     }
 
-    pthread_rwlock_unlock(&printer->system->rwlock);
+//    _papplRWUnlock(printer->system);
   }
 
   if (printer->num_supply > 0)
@@ -467,7 +477,7 @@ _papplPrinterCopyAttributes(
       char		value[256];	// "printer-supply" value
       ipp_attribute_t	*attr = NULL;	// "printer-supply" attribute
 
-      for (i = 0; i < (size_t)printer->num_supply; i ++)
+      for (i = 0; i < (cups_len_t)printer->num_supply; i ++)
       {
 	snprintf(value, sizeof(value), "index=%u;type=%s;maxcapacity=100;level=%d;colorantname=%s;", (unsigned)i, _papplSupplyTypeString(supply[i].type), supply[i].level, _papplSupplyColorString(supply[i].color));
 
@@ -480,7 +490,7 @@ _papplPrinterCopyAttributes(
 
     if (!ra || cupsArrayFind(ra, "printer-supply-description"))
     {
-      for (i = 0; i < (size_t)printer->num_supply; i ++)
+      for (i = 0; i < (cups_len_t)printer->num_supply; i ++)
         svalues[i] = supply[i].description;
 
       ippAddStrings(client->response, IPP_TAG_PRINTER, IPP_TAG_TEXT, "printer-supply-description", IPP_NUM_CAST printer->num_supply, NULL, svalues);
@@ -505,14 +515,14 @@ _papplPrinterCopyAttributes(
 
     num_values = 0;
 
-    if (httpAddrLocalhost(httpGetAddress(client->http)) || !papplSystemGetTLSOnly(client->system))
+    if (httpAddrIsLocalhost(httpGetAddress(client->http)) || !papplSystemGetTLSOnly(client->system))
     {
       httpAssembleURI(HTTP_URI_CODING_ALL, uris[num_values], sizeof(uris[0]), "ipp", NULL, client->host_field, client->host_port, printer->resource);
       values[num_values] = uris[num_values];
       num_values ++;
     }
 
-    if (!httpAddrLocalhost(httpGetAddress(client->http)) && !(client->system->options & PAPPL_SOPTIONS_NO_TLS))
+    if (!httpAddrIsLocalhost(httpGetAddress(client->http)) && !(client->system->options & PAPPL_SOPTIONS_NO_TLS))
     {
       httpAssembleURI(HTTP_URI_CODING_ALL, uris[num_values], sizeof(uris[0]), "ipps", NULL, client->host_field, client->host_port, printer->resource);
       values[num_values] = uris[num_values];
@@ -523,7 +533,7 @@ _papplPrinterCopyAttributes(
       ippAddStrings(client->response, IPP_TAG_PRINTER, IPP_TAG_URI, "printer-uri-supported", IPP_NUM_CAST num_values, NULL, values);
   }
 
-  if (client->system->wifi_status_cb && httpAddrLocalhost(httpGetAddress(client->http)) && (!ra || cupsArrayFind(ra, "printer-wifi-ssid") || cupsArrayFind(ra, "printer-wifi-state")))
+  if (client->system->wifi_status_cb && httpAddrIsLocalhost(httpGetAddress(client->http)) && (!ra || cupsArrayFind(ra, "printer-wifi-ssid") || cupsArrayFind(ra, "printer-wifi-state")))
   {
     // Get Wi-Fi status...
     pappl_wifi_t	wifi;		// Wi-Fi status
@@ -539,7 +549,7 @@ _papplPrinterCopyAttributes(
   }
 
   if (!ra || cupsArrayFind(ra, "printer-xri-supported"))
-    _papplPrinterCopyXRI(printer, client->response, client);
+    _papplPrinterCopyXRINoLock(printer, client->response, client);
 
   if (!ra || cupsArrayFind(ra, "queued-job-count"))
     ippAddInteger(client->response, IPP_TAG_PRINTER, IPP_TAG_INTEGER, "queued-job-count", (int)cupsArrayGetCount(printer->active_jobs));
@@ -558,7 +568,7 @@ _papplPrinterCopyAttributes(
     // supported.  Since we only support authentication over a secure (TLS)
     // channel, the value is always 'none' for the "ipp" URI and either 'none'
     // or 'basic' for the "ipps" URI...
-    if (httpAddrLocalhost(httpGetAddress(client->http)) || (client->system->options & PAPPL_SOPTIONS_NO_TLS))
+    if (httpAddrIsLocalhost(httpGetAddress(client->http)) || (client->system->options & PAPPL_SOPTIONS_NO_TLS))
     {
       ippAddString(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "uri-authentication-supported", NULL, "none");
     }
@@ -594,11 +604,11 @@ _papplPrinterCopyAttributes(
 
 
 //
-// '_papplPrinterCopyState()' - Copy the printer-state-xxx attributes.
+// '_papplPrinterCopyStateNoLock()' - Copy the printer-state-xxx attributes.
 //
 
 void
-_papplPrinterCopyState(
+_papplPrinterCopyStateNoLock(
     pappl_printer_t *printer,		// I - Printer
     ipp_tag_t       group_tag,		// I - Group tag
     ipp_t           *ipp,		// I - IPP message
@@ -624,7 +634,7 @@ _papplPrinterCopyState(
     bool		wifi_not_configured = false;
 					// Need the 'wifi-not-configured' reason?
 
-    if (client && client->system->wifi_status_cb && httpAddrLocalhost(httpGetAddress(client->http)))
+    if (client && client->system->wifi_status_cb && httpAddrIsLocalhost(httpGetAddress(client->http)))
     {
       pappl_wifi_t	wifi;		// Wi-Fi status
 
@@ -642,6 +652,14 @@ _papplPrinterCopyState(
       else if (printer->state == IPP_PSTATE_STOPPED)
 	attr = ippAddString(ipp, group_tag, IPP_CONST_TAG(IPP_TAG_KEYWORD), "printer-state-reasons", NULL, "paused");
 
+      if (printer->hold_new_jobs)
+      {
+        if (attr)
+          ippSetString(ipp, &attr, ippGetCount(attr), "hold-new-jobs");
+	else
+          attr = ippAddString(ipp, group_tag, IPP_CONST_TAG(IPP_TAG_KEYWORD), "printer-state-reasons", NULL, "hold-new-jobs");
+      }
+
       if (wifi_not_configured)
       {
         if (attr)
@@ -671,7 +689,11 @@ _papplPrinterCopyState(
 	ippSetString(ipp, &attr, ippGetCount(attr), "moving-to-paused");
       else if (printer->state == IPP_PSTATE_STOPPED)
 	ippSetString(ipp, &attr, ippGetCount(attr), "paused");
-      else if (wifi_not_configured)
+
+      if (printer->hold_new_jobs)
+	ippSetString(ipp, &attr, ippGetCount(attr), "hold-new-jobs");
+
+      if (wifi_not_configured)
 	ippSetString(ipp, &attr, ippGetCount(attr), "wifi-not-configured-report");
     }
   }
@@ -679,11 +701,11 @@ _papplPrinterCopyState(
 
 
 //
-// '_papplPrinterCopyXRI()' - Copy the "printer-xri-supported" attribute.
+// '_papplPrinterCopyXRINoLock()' - Copy the "printer-xri-supported" attribute.
 //
 
 void
-_papplPrinterCopyXRI(
+_papplPrinterCopyXRINoLock(
     pappl_printer_t *printer,		// I - Printer
     ipp_t           *ipp,		// I - IPP message
     pappl_client_t  *client)		// I - Client
@@ -695,7 +717,7 @@ _papplPrinterCopyXRI(
 	*values[2];			// Values for attribute
 
 
-  if (httpAddrLocalhost(httpGetAddress(client->http)) || !papplSystemGetTLSOnly(client->system))
+  if (httpAddrIsLocalhost(httpGetAddress(client->http)) || !papplSystemGetTLSOnly(client->system))
   {
     // Add ipp: URI...
     httpAssembleURI(HTTP_URI_CODING_ALL, uri, sizeof(uri), "ipp", NULL, client->host_field, client->host_port, printer->resource);
@@ -708,7 +730,7 @@ _papplPrinterCopyXRI(
     values[num_values ++] = col;
   }
 
-  if (!httpAddrLocalhost(httpGetAddress(client->http)) && !(client->system->options & PAPPL_SOPTIONS_NO_TLS))
+  if (!httpAddrIsLocalhost(httpGetAddress(client->http)) && !(client->system->options & PAPPL_SOPTIONS_NO_TLS))
   {
     // Add ipps: URI...
     httpAssembleURI(HTTP_URI_CODING_ALL, uri, sizeof(uri), "ipps", NULL, client->host_field, client->host_port, printer->resource);
@@ -825,6 +847,14 @@ _papplPrinterProcessIPP(
         ipp_disable_printer(client);
         break;
 
+    case IPP_OP_HOLD_NEW_JOBS :
+        ipp_hold_new_jobs(client);
+        break;
+
+    case IPP_OP_RELEASE_HELD_NEW_JOBS :
+        ipp_release_held_new_jobs(client);
+        break;
+
     case IPP_OP_CREATE_PRINTER_SUBSCRIPTIONS :
     case IPP_OP_CREATE_JOB_SUBSCRIPTIONS :
         _papplSubscriptionIPPCreate(client);
@@ -932,7 +962,7 @@ _papplPrinterSetAttributes(
 
   papplPrinterGetDriverData(printer, &driver_data);
 
-  for (rattr = ippFirstAttribute(client->request); rattr; rattr = ippNextAttribute(client->request))
+  for (rattr = ippGetFirstAttribute(client->request); rattr; rattr = ippGetNextAttribute(client->request))
   {
     papplLogClient(client, PAPPL_LOGLEVEL_DEBUG, "%s %s %s%s ...", ippTagString(ippGetGroupTag(rattr)), ippGetName(rattr), ippGetCount(rattr) > 1 ? "1setOf " : "", ippTagString(ippGetValueTag(rattr)));
 
@@ -946,10 +976,10 @@ _papplPrinterSetAttributes(
       continue;
     }
 
-    if (create_printer && (!strcmp(name, "printer-device-id") || !strcmp(name, "printer-name") || !strcmp(name, "smi2699-device-uri") || !strcmp(name, "smi2699-device-command")))
+    if (create_printer && (!strcmp(name, "printer-device-id") || !strcmp(name, "printer-name") || !strcmp(name, "smi55357-device-uri") || !strcmp(name, "smi55357-driver")))
       continue;
 
-    if ((create_printer || !httpAddrLocalhost(httpGetAddress(client->http)) || !client->system->wifi_join_cb) && (!strcmp(name, "printer-wifi-password") || !strcmp(name, "printer-wifi-ssid")))
+    if ((create_printer || !httpAddrIsLocalhost(httpGetAddress(client->http)) || !client->system->wifi_join_cb) && (!strcmp(name, "printer-wifi-password") || !strcmp(name, "printer-wifi-ssid")))
     {
       // Wi-Fi configuration can only be done over localhost...
       papplClientRespondIPPUnsupported(client, rattr);
@@ -1356,7 +1386,7 @@ ipp_create_job(pappl_client_t *client)	// I - Client
   cupsArrayAdd(ra, "job-state-reasons");
   cupsArrayAdd(ra, "job-uri");
 
-  _papplJobCopyAttributes(job, client, ra);
+  _papplJobCopyAttributesNoLock(job, client, ra);
   cupsArrayDelete(ra);
 }
 
@@ -1407,7 +1437,7 @@ ipp_get_jobs(pappl_client_t *client)	// I - Client
 					// which-jobs values
   int			job_comparison;	// Job comparison
   ipp_jstate_t		job_state;	// job-state value
-  size_t		i,		// Looping var
+  cups_len_t		i,		// Looping var
 			limit,		// Maximum number of jobs to return
 			count;		// Number of jobs that match
   const char		*username;	// Username
@@ -1461,7 +1491,7 @@ ipp_get_jobs(pappl_client_t *client)	// I - Client
     if (temp < 0)
       limit = 0;
     else
-      limit = (size_t)temp;
+      limit = (cups_len_t)temp;
   }
   else
     limit = 0;
@@ -1494,7 +1524,7 @@ ipp_get_jobs(pappl_client_t *client)	// I - Client
 
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
 
-  pthread_rwlock_rdlock(&(client->printer->rwlock));
+  _papplRWLockRead(client->printer);
 
   count = cupsArrayGetCount(list);
   if (limit == 0 || limit > count)
@@ -1512,12 +1542,12 @@ ipp_get_jobs(pappl_client_t *client)	// I - Client
       ippAddSeparator(client->response);
 
     count ++;
-    _papplJobCopyAttributes(job, client, ra);
+    _papplJobCopyAttributesNoLock(job, client, ra);
   }
 
   cupsArrayDelete(ra);
 
-  pthread_rwlock_unlock(&(client->printer->rwlock));
+  _papplRWUnlock(client->printer);
 }
 
 
@@ -1546,16 +1576,42 @@ ipp_get_printer_attributes(
 
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
 
-  pthread_rwlock_rdlock(&(printer->rwlock));
-
-  _papplPrinterCopyAttributes(printer, client, ra, ippGetString(ippFindAttribute(client->request, "document-format", IPP_TAG_MIMETYPE), 0, NULL));
-
-  pthread_rwlock_unlock(&(printer->rwlock));
+  _papplRWLockRead(printer->system);
+  _papplRWLockRead(printer);
+  _papplPrinterCopyAttributesNoLock(printer, client, ra, ippGetString(ippFindAttribute(client->request, "document-format", IPP_TAG_MIMETYPE), 0, NULL));
+  _papplRWUnlock(printer);
+  _papplRWUnlock(printer->system);
 
   cupsArrayDelete(ra);
 }
 
 
+//
+// 'ipp_hold_new_jobs()' - Hold new jobs.
+//
+
+static void
+ipp_hold_new_jobs(
+    pappl_client_t *client)		// I - Client
+{
+  http_status_t	auth_status;		// Authorization status
+
+
+  // Verify the connection is authorized...
+  if ((auth_status = papplClientIsAuthorized(client)) != HTTP_STATUS_CONTINUE)
+  {
+    papplClientRespond(client, auth_status, NULL, NULL, 0, 0);
+    return;
+  }
+
+  // Hold new jobs...
+  if (papplPrinterHoldNewJobs(client->printer))
+    papplClientRespondIPP(client, IPP_STATUS_OK, "New jobs being held.");
+  else
+    papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_POSSIBLE, "Jobs already being held.");
+}
+
+
 //
 // 'ipp_identify_printer()' - Beep or display a message.
 //
@@ -1639,8 +1695,9 @@ ipp_print_job(pappl_client_t *client)	// I - Client
   }
 
   // Are we accepting jobs?
-  if (!client->printer->is_accepting)
+  if (!papplPrinterIsAcceptingJobs(client->printer))
   {
+    _papplClientFlushDocumentData(client);
     papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_ACCEPTING_JOBS, "Not accepting new jobs.");
     return;
   }
@@ -1655,6 +1712,7 @@ ipp_print_job(pappl_client_t *client)	// I - Client
   // Create the job...
   if ((job = create_job(client)) == NULL)
   {
+    _papplClientFlushDocumentData(client);
     papplClientRespondIPP(client, IPP_STATUS_ERROR_BUSY, "Currently printing another job.");
     return;
   }
@@ -1664,6 +1722,32 @@ ipp_print_job(pappl_client_t *client)	// I - Client
 }
 
 
+//
+// 'ipp_release_held_new_jobs()' - Release held (new) jobs.
+//
+
+static void
+ipp_release_held_new_jobs(
+    pappl_client_t *client)		// I - Client
+{
+  http_status_t	auth_status;		// Authorization status
+
+
+  // Verify the connection is authorized...
+  if ((auth_status = papplClientIsAuthorized(client)) != HTTP_STATUS_CONTINUE)
+  {
+    papplClientRespond(client, auth_status, NULL, NULL, 0, 0);
+    return;
+  }
+
+  // Hold new jobs...
+  if (papplPrinterReleaseHeldNewJobs(client->printer, client->username))
+    papplClientRespondIPP(client, IPP_STATUS_OK, "Released all held jobs.");
+  else
+    papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_POSSIBLE, "Jobs not being held.");
+}
+
+
 //
 // 'ipp_resume_printer()' - Start a printer.
 //
@@ -1742,7 +1826,8 @@ valid_job_attributes(
 {
   cups_len_t		i,		// Looping var
 			count;		// Number of values
-  bool			valid = true;	// Valid attributes?
+  bool			valid = true,	// Valid attributes?
+			exact;		// Need attribute fidelity?
   ipp_attribute_t	*attr,		// Current attribute
 			*supported;	// xxx-supported attribute
 
@@ -1757,9 +1842,23 @@ valid_job_attributes(
   // Check operation attributes...
   valid = _papplJobValidateDocumentAttributes(client);
 
-  pthread_rwlock_rdlock(&client->printer->rwlock);
+  _papplRWLockRead(client->printer);
 
   // Check the various job template attributes...
+  exact = ippGetOperation(client->request) == IPP_OP_VALIDATE_JOB;
+
+  if ((attr = ippFindAttribute(client->request, "ipp-attribute-fidelity", IPP_TAG_ZERO)) != NULL)
+  {
+    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_BOOLEAN)
+    {
+      papplClientRespondIPPUnsupported(client, attr);
+      valid = false;
+    }
+
+    if (ippGetBoolean(attr, 0))
+      valid = true;
+  }
+
   if ((attr = ippFindAttribute(client->request, "copies", IPP_TAG_ZERO)) != NULL)
   {
     if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_INTEGER || ippGetInteger(attr, 0) < 1 || ippGetInteger(attr, 0) > 999)
@@ -1769,21 +1868,26 @@ valid_job_attributes(
     }
   }
 
-  if ((attr = ippFindAttribute(client->request, "ipp-attribute-fidelity", IPP_TAG_ZERO)) != NULL)
+  if ((attr = ippFindAttribute(client->request, "job-hold-until", IPP_TAG_ZERO)) != NULL)
   {
-    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_BOOLEAN)
+    if (ippGetCount(attr) != 1 || (ippGetValueTag(attr) != IPP_TAG_NAME && ippGetValueTag(attr) != IPP_TAG_NAMELANG && ippGetValueTag(attr) != IPP_TAG_KEYWORD))
     {
       papplClientRespondIPPUnsupported(client, attr);
       valid = false;
     }
-  }
 
-  if ((attr = ippFindAttribute(client->request, "job-hold-until", IPP_TAG_ZERO)) != NULL)
-  {
-    if (ippGetCount(attr) != 1 || (ippGetValueTag(attr) != IPP_TAG_NAME && ippGetValueTag(attr) != IPP_TAG_NAMELANG && ippGetValueTag(attr) != IPP_TAG_KEYWORD) || strcmp(ippGetString(attr, 0, NULL), "no-hold"))
+    if ((supported = ippFindAttribute(client->printer->attrs, "job-hold-until", IPP_TAG_KEYWORD)) != NULL && !ippContainsString(supported, ippGetString(attr, 0, NULL)))
     {
-      papplClientRespondIPPUnsupported(client, attr);
-      valid = false;
+      if (exact)
+      {
+        papplClientRespondIPPUnsupported(client, attr);
+        valid = false;
+      }
+      else
+      {
+        _papplClientRespondIPPIgnored(client, attr);
+	ippDeleteAttribute(client->request, attr);
+      }
     }
   }
 
@@ -1807,7 +1911,9 @@ valid_job_attributes(
     ippSetGroupTag(client->request, &attr, IPP_TAG_JOB);
   }
   else
+  {
     ippAddString(client->request, IPP_TAG_JOB, IPP_TAG_NAME, "job-name", NULL, "Untitled");
+  }
 
   if ((attr = ippFindAttribute(client->request, "job-priority", IPP_TAG_ZERO)) != NULL)
   {
@@ -1820,11 +1926,16 @@ valid_job_attributes(
 
   if ((attr = ippFindAttribute(client->request, "job-sheets", IPP_TAG_ZERO)) != NULL)
   {
-    if (ippGetCount(attr) != 1 || (ippGetValueTag(attr) != IPP_TAG_NAME && ippGetValueTag(attr) != IPP_TAG_NAMELANG && ippGetValueTag(attr) != IPP_TAG_KEYWORD) || strcmp(ippGetString(attr, 0, NULL), "none"))
+    if (ippGetCount(attr) != 1 || (ippGetValueTag(attr) != IPP_TAG_NAME && ippGetValueTag(attr) != IPP_TAG_NAMELANG && ippGetValueTag(attr) != IPP_TAG_KEYWORD) || (exact && strcmp(ippGetString(attr, 0, NULL), "none")))
     {
       papplClientRespondIPPUnsupported(client, attr);
       valid = false;
     }
+    else if (strcmp(ippGetString(attr, 0, NULL), "none"))
+    {
+      _papplClientRespondIPPIgnored(client, attr);
+      ippDeleteAttribute(client->request, attr);
+    }
   }
 
   if ((attr = ippFindAttribute(client->request, "media", IPP_TAG_ZERO)) != NULL)
@@ -1840,8 +1951,16 @@ valid_job_attributes(
 
       if (!ippContainsString(supported, ippGetString(attr, 0, NULL)))
       {
-	papplClientRespondIPPUnsupported(client, attr);
-	valid = false;
+        if (exact)
+        {
+	  papplClientRespondIPPUnsupported(client, attr);
+	  valid = false;
+	}
+	else
+	{
+	  _papplClientRespondIPPIgnored(client, attr);
+	  ippDeleteAttribute(client->request, attr);
+	}
       }
     }
   }
@@ -1877,8 +1996,16 @@ valid_job_attributes(
 
 	if (!ippContainsString(supported, ippGetString(member, 0, NULL)))
 	{
-	  papplClientRespondIPPUnsupported(client, attr);
-	  valid = false;
+	  if (exact)
+	  {
+	    papplClientRespondIPPUnsupported(client, attr);
+	    valid = false;
+	  }
+	  else
+	  {
+	    _papplClientRespondIPPIgnored(client, attr);
+	    ippDeleteAttribute(client->request, attr);
+	  }
 	}
       }
     }
@@ -1917,8 +2044,16 @@ valid_job_attributes(
 
 	  if (i >= count)
 	  {
-	    papplClientRespondIPPUnsupported(client, attr);
-	    valid = false;
+	    if (exact)
+	    {
+	      papplClientRespondIPPUnsupported(client, attr);
+	      valid = false;
+	    }
+	    else
+	    {
+	      _papplClientRespondIPPIgnored(client, attr);
+	      ippDeleteAttribute(client->request, attr);
+	    }
 	  }
 	}
       }
@@ -1927,20 +2062,46 @@ valid_job_attributes(
 
   if ((attr = ippFindAttribute(client->request, "multiple-document-handling", IPP_TAG_ZERO)) != NULL)
   {
-    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_KEYWORD || (strcmp(ippGetString(attr, 0, NULL), "separate-documents-uncollated-copies") && strcmp(ippGetString(attr, 0, NULL), "separate-documents-collated-copies")))
+    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_KEYWORD)
     {
       papplClientRespondIPPUnsupported(client, attr);
       valid = false;
     }
+    else if (strcmp(ippGetString(attr, 0, NULL), "separate-documents-uncollated-copies") && strcmp(ippGetString(attr, 0, NULL), "separate-documents-collated-copies"))
+    {
+      if (exact)
+      {
+	papplClientRespondIPPUnsupported(client, attr);
+	valid = false;
+      }
+      else
+      {
+	_papplClientRespondIPPIgnored(client, attr);
+	ippDeleteAttribute(client->request, attr);
+      }
+    }
   }
 
   if ((attr = ippFindAttribute(client->request, "orientation-requested", IPP_TAG_ZERO)) != NULL)
   {
-    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_ENUM || ippGetInteger(attr, 0) < IPP_ORIENT_PORTRAIT || ippGetInteger(attr, 0) > IPP_ORIENT_NONE)
+    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_ENUM)
     {
       papplClientRespondIPPUnsupported(client, attr);
       valid = false;
     }
+    else if (ippGetInteger(attr, 0) < IPP_ORIENT_PORTRAIT || ippGetInteger(attr, 0) > IPP_ORIENT_NONE)
+    {
+      if (exact)
+      {
+	papplClientRespondIPPUnsupported(client, attr);
+	valid = false;
+      }
+      else
+      {
+	_papplClientRespondIPPIgnored(client, attr);
+	ippDeleteAttribute(client->request, attr);
+      }
+    }
   }
 
   if ((attr = ippFindAttribute(client->request, "page-ranges", IPP_TAG_ZERO)) != NULL)
@@ -1960,31 +2121,70 @@ valid_job_attributes(
     pappl_color_mode_t value = _papplColorModeValue(ippGetString(attr, 0, NULL));
 					// "print-color-mode" value
 
-    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_KEYWORD || !(value & client->printer->driver_data.color_supported))
+    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_KEYWORD)
     {
       papplClientRespondIPPUnsupported(client, attr);
       valid = false;
     }
+    else if (!(value & client->printer->driver_data.color_supported))
+    {
+      if (exact)
+      {
+	papplClientRespondIPPUnsupported(client, attr);
+	valid = false;
+      }
+      else
+      {
+	_papplClientRespondIPPIgnored(client, attr);
+	ippDeleteAttribute(client->request, attr);
+      }
+    }
   }
 
   if ((attr = ippFindAttribute(client->request, "print-content-optimize", IPP_TAG_ZERO)) != NULL)
   {
-    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_KEYWORD || !_papplContentValue(ippGetString(attr, 0, NULL)))
+    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_KEYWORD)
     {
       papplClientRespondIPPUnsupported(client, attr);
       valid = false;
     }
+    else if (!_papplContentValue(ippGetString(attr, 0, NULL)))
+    {
+      if (exact)
+      {
+	papplClientRespondIPPUnsupported(client, attr);
+	valid = false;
+      }
+      else
+      {
+	_papplClientRespondIPPIgnored(client, attr);
+	ippDeleteAttribute(client->request, attr);
+      }
+    }
   }
 
   if ((attr = ippFindAttribute(client->request, "print-darkness", IPP_TAG_ZERO)) != NULL)
   {
     int value = ippGetInteger(attr, 0);	// "print-darkness" value
 
-    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_INTEGER || value < -100 || value > 100 || client->printer->driver_data.darkness_supported == 0)
+    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_INTEGER || value < -100 || value > 100)
     {
       papplClientRespondIPPUnsupported(client, attr);
       valid = false;
     }
+    else if (client->printer->driver_data.darkness_supported == 0)
+    {
+      if (exact)
+      {
+	papplClientRespondIPPUnsupported(client, attr);
+	valid = false;
+      }
+      else
+      {
+	_papplClientRespondIPPIgnored(client, attr);
+	ippDeleteAttribute(client->request, attr);
+      }
+    }
   }
 
   if ((attr = ippFindAttribute(client->request, "print-quality", IPP_TAG_ZERO)) != NULL)
@@ -2009,11 +2209,24 @@ valid_job_attributes(
   {
     int value = ippGetInteger(attr, 0);	// "print-speed" value
 
-    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_INTEGER || value < client->printer->driver_data.speed_supported[0] || value > client->printer->driver_data.speed_supported[1] || client->printer->driver_data.speed_supported[1] == 0)
+    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_INTEGER)
     {
       papplClientRespondIPPUnsupported(client, attr);
       valid = false;
     }
+    else if (value < client->printer->driver_data.speed_supported[0] || value > client->printer->driver_data.speed_supported[1] || client->printer->driver_data.speed_supported[1] == 0)
+    {
+      if (exact)
+      {
+	papplClientRespondIPPUnsupported(client, attr);
+	valid = false;
+      }
+      else
+      {
+	_papplClientRespondIPPIgnored(client, attr);
+	ippDeleteAttribute(client->request, attr);
+      }
+    }
   }
 
   if ((attr = ippFindAttribute(client->request, "printer-resolution", IPP_TAG_ZERO)) != NULL)
@@ -2039,8 +2252,16 @@ valid_job_attributes(
 
       if (i >= (cups_len_t)client->printer->driver_data.num_resolution)
       {
-	papplClientRespondIPPUnsupported(client, attr);
-	valid = false;
+        if (exact)
+        {
+	  papplClientRespondIPPUnsupported(client, attr);
+	  valid = false;
+	}
+	else
+	{
+	  _papplClientRespondIPPIgnored(client, attr);
+	  ippDeleteAttribute(client->request, attr);
+	}
       }
     }
   }
@@ -2050,14 +2271,27 @@ valid_job_attributes(
     pappl_sides_t value = _papplSidesValue(ippGetString(attr, 0, NULL));
 					// "sides" value
 
-    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_KEYWORD || !(value & client->printer->driver_data.sides_supported))
+    if (ippGetCount(attr) != 1 || ippGetValueTag(attr) != IPP_TAG_KEYWORD)
     {
       papplClientRespondIPPUnsupported(client, attr);
       valid = false;
     }
+    else if (!(value & client->printer->driver_data.sides_supported))
+    {
+      if (exact)
+      {
+	papplClientRespondIPPUnsupported(client, attr);
+	valid = false;
+      }
+      else
+      {
+	_papplClientRespondIPPIgnored(client, attr);
+	ippDeleteAttribute(client->request, attr);
+      }
+    }
   }
 
-  pthread_rwlock_unlock(&client->printer->rwlock);
+  _papplRWUnlock(client->printer);
 
   return (valid);
 }
