@@ -99,105 +99,6 @@ _papplPrinterWebCancelAllJobs(
 }
 
 
-//
-// '_papplPrinterWebCancelJob()' - Cancel a job.
-//
-
-void
-_papplPrinterWebCancelJob(
-    pappl_client_t  *client,		// I - Client
-    pappl_printer_t *printer)		// I - Printer
-{
-  int		job_id = 0;             // Job ID to cancel
-  pappl_job_t	*job;			// Job to cancel
-  const char	*status = NULL;		// Status message, if any
-  cups_len_t	num_form;		// Number of form variables
-  cups_option_t	*form;			// Form variables
-  const char	*value;			// Value of form variable
-
-
-  if (!papplClientHTMLAuthorize(client))
-    return;
-
-  if (client->operation == HTTP_STATE_GET)
-  {
-    if ((num_form = (cups_len_t)papplClientGetForm(client, &form)) == 0)
-    {
-      status = _PAPPL_LOC("Invalid GET data.");
-    }
-    else if ((value = cupsGetOption("job-id", num_form, form)) != NULL)
-    {
-      char *end;			// End of value
-
-      job_id = (int)strtol(value, &end, 10);
-
-      if (errno == ERANGE || *end)
-      {
-        job_id = 0;
-        status = _PAPPL_LOC("Invalid job ID.");
-      }
-    }
-
-    cupsFreeOptions(num_form, form);
-  }
-  else if (client->operation == HTTP_STATE_POST)
-  {
-    if ((num_form = (cups_len_t)papplClientGetForm(client, &form)) == 0)
-    {
-      status = _PAPPL_LOC("Invalid form data.");
-    }
-    else if (!papplClientIsValidForm(client, (int)num_form, form))
-    {
-      status = _PAPPL_LOC("Invalid form submission.");
-    }
-    else if ((value = cupsGetOption("job-id", num_form, form)) != NULL)
-    {
-      // Get the job to cancel
-      char *end;			// End of value
-
-      job_id = (int)strtol(value, &end, 10);
-      if (errno == ERANGE || *end)
-      {
-        status = _PAPPL_LOC("Invalid job ID.");
-      }
-      else if ((job = papplPrinterFindJob(printer, job_id)) != NULL)
-      {
-        char	path[1024];		// Resource path
-
-        papplJobCancel(job);
-        snprintf(path, sizeof(path), "%s/jobs", printer->uriname);
-        papplClientRespondRedirect(client, HTTP_STATUS_FOUND, path);
-        cupsFreeOptions(num_form, form);
-        return;
-      }
-      else
-      {
-        status = _PAPPL_LOC("Invalid Job ID.");
-      }
-
-      cupsFreeOptions(num_form, form);
-    }
-    else
-    {
-      status = _PAPPL_LOC("Invalid form submission.");
-    }
-  }
-
-  papplClientHTMLPrinterHeader(client, printer, _PAPPL_LOC("Cancel Job"), 0, NULL, NULL);
-
-  if (status)
-    papplClientHTMLPrintf(client, "<div class=\"banner\">%s</div>\n", papplClientGetLocString(client, status));
-
-  if (job_id)
-  {
-    papplClientHTMLStartForm(client, client->uri, false);
-    papplClientHTMLPrintf(client, "           <input type=\"hidden\" name=\"job-id\" value=\"%d\"><input type=\"submit\" value=\"%s\"></form>\n", job_id, papplClientGetLocString(client, _PAPPL_LOC("Confirm Cancel Job")));
-  }
-
-  papplClientHTMLFooter(client);
-}
-
-
 //
 // '_papplPrinterWebConfig()' - Show the printer configuration web page.
 //
@@ -658,31 +559,32 @@ _papplPrinterWebDefaults(
   papplClientHTMLPuts(client, "</td></tr>\n");
 
   // Vendor options
-  pthread_rwlock_rdlock(&printer->rwlock);
+  _papplRWLockRead(printer);
 
   for (i = 0; i < data.num_vendor; i ++)
   {
     char	defname[128],		// xxx-default name
 		defvalue[1024],		// xxx-default value
 		supname[128];		// xxx-supported name
-    ipp_attribute_t *attr;		// Attribute
+    ipp_attribute_t *defattr,		// xxx-default attribute
+	      	*supattr;		// xxx-supported attribute
     int		count;			// Number of values
 
     snprintf(defname, sizeof(defname), "%s-default", data.vendor[i]);
     snprintf(supname, sizeof(defname), "%s-supported", data.vendor[i]);
 
-    if ((attr = ippFindAttribute(printer->driver_attrs, defname, IPP_TAG_ZERO)) != NULL)
-      ippAttributeString(attr, defvalue, sizeof(defvalue));
+    if ((defattr = ippFindAttribute(printer->driver_attrs, defname, IPP_TAG_ZERO)) != NULL)
+      ippAttributeString(defattr, defvalue, sizeof(defvalue));
     else
       defvalue[0] = '\0';
 
-    if ((attr = ippFindAttribute(printer->driver_attrs, supname, IPP_TAG_ZERO)) != NULL)
+    papplClientHTMLPrintf(client, "              <tr><th>%s:</th><td>", papplClientGetLocString(client, data.vendor[i]));
+
+    if ((supattr = ippFindAttribute(printer->driver_attrs, supname, IPP_TAG_ZERO)) != NULL)
     {
-      count = (int)ippGetCount(attr);
+      count = (int)ippGetCount(supattr);
 
-      papplClientHTMLPrintf(client, "              <tr><th>%s:</th><td>", papplClientGetLocString(client, data.vendor[i]));
-
-      switch (ippGetValueTag(attr))
+      switch (ippGetValueTag(supattr))
       {
         case IPP_TAG_BOOLEAN :
             papplClientHTMLPrintf(client, "<input type=\"checkbox\" name=\"%s\"%s>", data.vendor[i], !strcmp(defvalue, "true") ? " checked" : "");
@@ -692,7 +594,7 @@ _papplPrinterWebDefaults(
             papplClientHTMLPrintf(client, "<select name=\"%s\">", data.vendor[i]);
             for (j = 0; j < count; j ++)
             {
-              int val = ippGetInteger(attr, (cups_len_t)j);
+              int val = ippGetInteger(supattr, (cups_len_t)j);
 
 	      papplClientHTMLPrintf(client, "<option value=\"%d\"%s>%d</option>", val, val == (int)strtol(defvalue, NULL, 10) ? " selected" : "", val);
             }
@@ -701,7 +603,7 @@ _papplPrinterWebDefaults(
 
         case IPP_TAG_RANGE :
             {
-              int upper, lower = ippGetRange(attr, 0, &upper);
+              int upper, lower = ippGetRange(supattr, 0, &upper);
 					// Range
 
 	      papplClientHTMLPrintf(client, "<input type=\"number\" name=\"%s\" min=\"%d\" max=\"%d\" value=\"%s\">", data.vendor[i], lower, upper, defvalue);
@@ -712,7 +614,7 @@ _papplPrinterWebDefaults(
             papplClientHTMLPrintf(client, "<select name=\"%s\">", data.vendor[i]);
             for (j = 0; j < count; j ++)
             {
-              const char *val = ippGetString(attr, (cups_len_t)j, NULL);
+              const char *val = ippGetString(supattr, (cups_len_t)j, NULL);
 
 	      papplClientHTMLPrintf(client, "<option value=\"%s\"%s>%s</option>", val, !strcmp(val, defvalue) ? " selected" : "", localize_keyword(client, data.vendor[i], val, text, sizeof(text)));
             }
@@ -723,12 +625,17 @@ _papplPrinterWebDefaults(
 	    papplClientHTMLPuts(client, "Unsupported value syntax.");
 	    break;
       }
-
-      papplClientHTMLPuts(client, "</td></tr>\n");
     }
+    else
+    {
+      // Text option
+      papplClientHTMLPrintf(client, "<input type=\"text\" name=\"%s\" value=\"%s\">", data.vendor[i], defvalue);
+    }
+
+    papplClientHTMLPuts(client, "</td></tr>\n");
   }
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   papplClientHTMLPrintf(client,
                         "              <tr><th></th><td><input type=\"submit\" value=\"%s\"></td></tr>\n"
@@ -839,6 +746,12 @@ _papplPrinterWebHome(
     {
       status = _PAPPL_LOC("Missing action.");
     }
+    else if (!strcmp(action, "hold-new-jobs"))
+    {
+      papplPrinterHoldNewJobs(printer);
+
+      status = _PAPPL_LOC("Holding new jobs.");
+    }
     else if (!strcmp(action, "identify-printer"))
     {
       if (printer->driver_data.identify_supported && printer->driver_data.identify_cb)
@@ -905,6 +818,12 @@ _papplPrinterWebHome(
       else
         status = _PAPPL_LOC("Printer pausing.");
     }
+    else if (!strcmp(action, "release-held-new-jobs"))
+    {
+      papplPrinterReleaseHeldNewJobs(printer, client->username);
+
+      status = _PAPPL_LOC("Released held new jobs.");
+    }
     else if (!strcmp(action, "resume-printer"))
     {
       papplPrinterResume(printer);
@@ -1025,7 +944,8 @@ _papplPrinterWebIteratorCallback(
 			"          <p><img class=\"%s\" src=\"%s/icon-md.png\">%s, %s", ippEnumString("printer-state", (int)printer_state), printer->uriname, localize_keyword(client, "printer-state", state_str, text, sizeof(text)), jobs_str);
   if ((printer->system->options & PAPPL_SOPTIONS_MULTI_QUEUE) && printer->printer_id == printer->system->default_printer_id)
     papplClientHTMLPrintf(client, ", %s", papplClientGetLocString(client, _PAPPL_LOC("default printer")));
-
+  if (printer->hold_new_jobs)
+    papplClientHTMLPrintf(client, ", %s", papplClientGetLocString(client, _PAPPL_LOC("holding new jobs")));
   for (i = 0, reason = PAPPL_PREASON_OTHER; reason <= PAPPL_PREASON_TONER_LOW; i ++, reason *= 2)
   {
     if (printer_reasons & reason)
@@ -1040,6 +960,12 @@ _papplPrinterWebIteratorCallback(
   papplClientHTMLPuts(client, "          <div class=\"btn\">");
   _papplClientHTMLPutLinks(client, printer->links, PAPPL_LOPTIONS_STATUS);
 
+  if (!printer->hold_new_jobs && papplPrinterGetMaxActiveJobs(printer) != 1)
+  {
+    papplClientHTMLStartForm(client, uri, false);
+    papplClientHTMLPrintf(client, "<input type=\"hidden\" name=\"action\" value=\"hold-new-jobs\"><input type=\"submit\" value=\"%s\"></form>", papplClientGetLocString(client, _PAPPL_LOC("Hold New Jobs")));
+  }
+
   if (printer->driver_data.identify_supported)
   {
     papplClientHTMLStartForm(client, uri, false);
@@ -1052,6 +978,12 @@ _papplPrinterWebIteratorCallback(
     papplClientHTMLPrintf(client, "<input type=\"hidden\" name=\"action\" value=\"print-test-page\"><input type=\"submit\" value=\"%s\"></form>", papplClientGetLocString(client, _PAPPL_LOC("Print Test Page")));
   }
 
+  if (printer->hold_new_jobs && papplPrinterGetMaxActiveJobs(printer) != 1)
+  {
+    papplClientHTMLStartForm(client, uri, false);
+    papplClientHTMLPrintf(client, "<input type=\"hidden\" name=\"action\" value=\"release-held-new-jobs\"><input type=\"submit\" value=\"%s\"></form>", papplClientGetLocString(client, _PAPPL_LOC("Release Held New Jobs")));
+  }
+
   if (printer->system->options & PAPPL_SOPTIONS_MULTI_QUEUE)
   {
     if (printer->state == IPP_PSTATE_STOPPED)
@@ -1091,12 +1023,14 @@ _papplPrinterWebJobs(
   ipp_pstate_t	printer_state;		// Printer state
   int		job_index = 1,		// Job index
 		limit = 20;		// Jobs per page
-
+  const char	*status = NULL;		// Status message
+  bool		refresh;		// Refresh the window?
 
   if (!papplClientHTMLAuthorize(client))
     return;
 
   printer_state = papplPrinterGetState(printer);
+  refresh       = printer_state == IPP_PSTATE_PROCESSING;
 
   if (client->operation == HTTP_STATE_GET)
   {
@@ -1110,6 +1044,117 @@ _papplPrinterWebJobs(
 
     cupsFreeOptions(num_form, form);
   }
+  else if (client->operation == HTTP_STATE_POST)
+  {
+    cups_len_t		num_form = 0;	// Number of form variable
+    cups_option_t	*form = NULL;	// Form variables
+    const char		*value;		// Value of form variable
+    int			job_id = 0;	// Job ID to cancel
+    pappl_job_t		*job;		// Job to cancel
+    const char		*action;	// Form action
+
+    if ((num_form = (cups_len_t)papplClientGetForm(client, &form)) == 0)
+    {
+      status = _PAPPL_LOC("Invalid form data.");
+    }
+    else if (!papplClientIsValidForm(client, (int)num_form, form))
+    {
+      status = _PAPPL_LOC("Invalid form submission.");
+    }
+    else if ((value = cupsGetOption("job-id", num_form, form)) != NULL)
+    {
+      char *end;			// End of value
+
+      job_id = (int)strtol(value, &end, 10);
+
+      if (errno == ERANGE || *end)
+      {
+        status = _PAPPL_LOC("Invalid job ID.");
+      }
+      else if ((job = papplPrinterFindJob(printer, job_id)) != NULL)
+      {
+        const char	*username;	// Username
+
+        if (client->username[0])
+          username = client->username;
+        else
+          username = "guest";
+
+	if ((action = cupsGetOption("action", num_form, form)) == NULL)
+	{
+	  status = _PAPPL_LOC("Missing action.");
+	}
+	else if (!strcmp(action, "cancel-job"))
+	{
+	  papplJobCancel(job);
+	  status = _PAPPL_LOC("Job canceled.");
+	}
+	else if (!strcmp(action, "hold-job"))
+	{
+	  papplJobHold(job, username, "indefinite", 0);
+	  status = _PAPPL_LOC("Job held.");
+	}
+	else if (!strcmp(action, "release-job"))
+	{
+	  papplJobRelease(job, username);
+	  status  = _PAPPL_LOC("Job released.");
+	  refresh = true;
+	}
+	else if (!strcmp(action, "reprint-job"))
+	{
+	  // Copy the job...
+	  pappl_job_t	*new_job;	// New job
+
+	  if ((new_job = _papplJobCreate(printer, 0, username, job->format, job->name, job->attrs)) != NULL)
+	  {
+	    // Copy the job file...
+	    int		oldfd,		// Old job file
+			newfd;		// New job file
+	    char	filename[1024],	// Job filename
+			buffer[8192];	// Copy buffer
+	    ssize_t	bytes;		// Bytes read...
+
+	    if ((oldfd = open(job->filename, O_RDONLY | O_BINARY)) >= 0)
+	    {
+	      if ((newfd = papplJobOpenFile(new_job, filename, sizeof(filename), printer->system->directory, NULL, "w")) >= 0)
+	      {
+		while ((bytes = read(oldfd, buffer, sizeof(buffer))) > 0)
+		  write(newfd, buffer, (size_t)bytes);
+
+		close(oldfd);
+		close(newfd);
+
+		// Submit the job for processing...
+		_papplJobSubmitFile(new_job, filename);
+		status  = _PAPPL_LOC("Reprinted job.");
+		refresh = true;
+	      }
+
+	      close(oldfd);
+	    }
+	  }
+
+          if (!status)
+	    status = _PAPPL_LOC("Unable to copy print job.");
+	}
+	else
+	{
+	  papplLogClient(client, PAPPL_LOGLEVEL_DEBUG, "action='%s'", action);
+	  status = _PAPPL_LOC("Unknown action.");
+	}
+      }
+      else
+      {
+        status = _PAPPL_LOC("Invalid Job ID.");
+      }
+    }
+    else
+    {
+      status = _PAPPL_LOC("Missing job ID.");
+    }
+
+    cupsFreeOptions(num_form, form);
+  }
 
   if (cupsArrayGetCount(printer->active_jobs) > 0)
   {
@@ -1117,13 +1162,21 @@ _papplPrinterWebJobs(
 
     httpAssembleURIf(HTTP_URI_CODING_ALL, url, sizeof(url), "https", NULL, client->host_field, client->host_port, "%s/cancelall", printer->uriname);
 
-    papplClientHTMLPrinterHeader(client, printer, _PAPPL_LOC("Jobs"), printer_state == IPP_PSTATE_PROCESSING ? 10 : 0, _PAPPL_LOC("Cancel All Jobs"), url);
+    papplClientHTMLPrinterHeader(client, printer, _PAPPL_LOC("Jobs"), refresh ? 10 : 0, _PAPPL_LOC("Cancel All Jobs"), url);
   }
   else
   {
     papplClientHTMLPrinterHeader(client, printer, _PAPPL_LOC("Jobs"), printer_state == IPP_PSTATE_PROCESSING ? 10 : 0, NULL, NULL);
   }
 
+  if (status)
+    papplClientHTMLPrintf(client,
+                          "      <div class=\"row\">\n"
+			  "        <div class=\"col-6\">\n"
+			  "          <div class=\"banner\">%s</div>\n"
+			  "        </div>\n"
+			  "      </div>\n", papplClientGetLocString(client, status));
+
   if (papplPrinterGetNumberOfJobs(printer) > 0)
   {
     job_pager(client, printer, job_index, limit);
@@ -1316,140 +1369,6 @@ _papplPrinterWebMedia(
 }
 
 
-//
-// '_papplPrinterWebReprintJob()' - Reprint a job.
-//
-
-void
-_papplPrinterWebReprintJob(
-    pappl_client_t  *client,		// I - Client
-    pappl_printer_t *printer)		// I - Printer
-{
-  int		job_id = 0;             // Job ID to reprint
-  pappl_job_t	*job,			// Job to reprint
-		*new_job;		// New job
-  const char	*status = NULL;		// Status message, if any
-  cups_len_t	num_form;		// Number of form variables
-  cups_option_t	*form;			// Form variables
-  const char	*value;			// Value of form variable
-
-
-  if (!papplClientHTMLAuthorize(client))
-    return;
-
-  if (client->operation == HTTP_STATE_GET)
-  {
-    if ((num_form = (cups_len_t)papplClientGetForm(client, &form)) == 0)
-    {
-      status = _PAPPL_LOC("Invalid GET data.");
-    }
-    else if ((value = cupsGetOption("job-id", num_form, form)) != NULL)
-    {
-      char *end;			// End of value
-
-      job_id = (int)strtol(value, &end, 10);
-
-      if (errno == ERANGE || *end)
-      {
-        job_id = 0;
-        status = _PAPPL_LOC("Invalid job ID.");
-      }
-    }
-
-    cupsFreeOptions(num_form, form);
-  }
-  else if (client->operation == HTTP_STATE_POST)
-  {
-    if ((num_form = (cups_len_t)papplClientGetForm(client, &form)) == 0)
-    {
-      status = _PAPPL_LOC("Invalid form data.");
-    }
-    else if (!papplClientIsValidForm(client, (int)num_form, form))
-    {
-      status = _PAPPL_LOC("Invalid form submission.");
-    }
-    else if ((value = cupsGetOption("job-id", num_form, form)) != NULL)
-    {
-      // Get the job to cancel
-      char *end;			// End of value
-
-      job_id = (int)strtol(value, &end, 10);
-      if (errno == ERANGE || *end)
-      {
-        status = _PAPPL_LOC("Invalid job ID.");
-      }
-      else if ((job = papplPrinterFindJob(printer, job_id)) != NULL)
-      {
-        char		path[1024];	// Resource path
-        const char	*username;	// Username
-
-        if (client->username[0])
-          username = client->username;
-        else
-          username = "guest";
-
-        // Copy the job...
-        if ((new_job = _papplJobCreate(printer, 0, username, job->format, job->name, job->attrs)) != NULL)
-        {
-          // Copy the job file...
-          int		oldfd,		// Old job file
-			newfd;		// New job file
-	  char		buffer[8192];	// Copy buffer
-	  ssize_t	bytes;		// Bytes read...
-
-          if ((oldfd = open(job->filename, O_RDONLY | O_BINARY)) >= 0)
-          {
-            if ((newfd = papplJobOpenFile(new_job, path, sizeof(path), printer->system->directory, NULL, "w")) >= 0)
-            {
-              while ((bytes = read(oldfd, buffer, sizeof(buffer))) > 0)
-                write(newfd, buffer, (size_t)bytes);
-
-              close(oldfd);
-              close(newfd);
-
-	      // Submit the job for processing...
-	      _papplJobSubmitFile(new_job, path);
-
-	      snprintf(path, sizeof(path), "%s/jobs", printer->uriname);
-	      papplClientRespondRedirect(client, HTTP_STATUS_FOUND, path);
-	      cupsFreeOptions(num_form, form);
-	      return;
-	    }
-
-	    close(oldfd);
-          }
-	}
-
-	status = _PAPPL_LOC("Unable to copy print job.");
-      }
-      else
-      {
-        status = _PAPPL_LOC("Invalid Job ID.");
-      }
-
-      cupsFreeOptions(num_form, form);
-    }
-    else
-    {
-      status = _PAPPL_LOC("Invalid form submission.");
-    }
-  }
-
-  papplClientHTMLPrinterHeader(client, printer, _PAPPL_LOC("Reprint Job"), 0, NULL, NULL);
-
-  if (status)
-    papplClientHTMLPrintf(client, "<div class=\"banner\">%s</div>\n", papplClientGetLocString(client, status));
-
-  if (job_id)
-  {
-    papplClientHTMLStartForm(client, client->uri, false);
-    papplClientHTMLPrintf(client, "           <input type=\"hidden\" name=\"job-id\" value=\"%d\"><input type=\"submit\" value=\"%s\"></form>\n", job_id, papplClientGetLocString(client, _PAPPL_LOC("Confirm Reprint Job")));
-  }
-
-  papplClientHTMLFooter(client);
-}
-
-
 //
 // '_papplPrinterWebSupplies()' - Show the printer supplies web page.
 //
@@ -1521,16 +1440,27 @@ static void
 job_cb(pappl_job_t    *job,		// I - Job
        pappl_client_t *client)		// I - Client
 {
-  bool	show_cancel = false;		// Show the "cancel" button?
-  char	when[256],			// When job queued/started/finished
+  bool	show_cancel = false,		// Show the "cancel" button?
+	show_hold = false,		// Show the "hold" button?
+	show_release = false;		// Show the "release" button?
+  char	uri[256],			// Form URI
+	when[256],			// When job queued/started/finished
       	hhmmss[64];			// Time HH:MM:SS
 
 
+  snprintf(uri, sizeof(uri), "%s/jobs", job->printer->uriname);
+
   switch (papplJobGetState(job))
   {
     case IPP_JSTATE_PENDING :
-    case IPP_JSTATE_HELD :
 	show_cancel = true;
+        show_hold   = papplPrinterGetMaxActiveJobs(papplJobGetPrinter(job)) != 1;
+	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Queued at %s"), time_string(papplJobGetTimeCreated(job), hhmmss, sizeof(hhmmss)));
+	break;
+
+    case IPP_JSTATE_HELD :
+	show_cancel  = true;
+	show_release = true;
 	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Queued at %s"), time_string(papplJobGetTimeCreated(job), hhmmss, sizeof(hhmmss)));
 	break;
 
@@ -1560,14 +1490,33 @@ job_cb(pappl_job_t    *job,		// I - Job
 	break;
   }
 
-  papplClientHTMLPrintf(client, "              <tr><td>%d</td><td>%s</td><td>%s</td><td>%d</td><td>%s</td>", papplJobGetID(job), papplJobGetName(job), papplJobGetUsername(job), papplJobGetImpressionsCompleted(job), when);
+  papplClientHTMLPrintf(client, "              <tr><td>%d</td><td>%s</td><td>%s</td><td>%d</td><td>%s</td><td>", papplJobGetID(job), papplJobGetName(job), papplJobGetUsername(job), papplJobGetImpressionsCompleted(job), when);
 
   if (show_cancel)
-    papplClientHTMLPrintf(client, "          <td><a class=\"btn\" href=\"%s/cancel?job-id=%d\">%s</a></td></tr>\n", job->printer->uriname, papplJobGetID(job), papplClientGetLocString(client, _PAPPL_LOC("Cancel Job")));
-  else if (papplJobGetState(job) >= IPP_JSTATE_ABORTED && job->filename)
-    papplClientHTMLPrintf(client, "          <td><a class=\"btn\" href=\"%s/reprint?job-id=%d\">%s</a></td></tr>\n", job->printer->uriname, papplJobGetID(job), papplClientGetLocString(client, _PAPPL_LOC("Reprint Job")));
-  else
-    papplClientHTMLPuts(client, "<td></td></tr>\n");
+  {
+    papplClientHTMLStartForm(client, uri, false);
+    papplClientHTMLPrintf(client, "<input type=\"hidden\" name=\"action\" value=\"cancel-job\"><input type=\"hidden\" name=\"job-id\" value=\"%d\"><input type=\"submit\" value=\"%s\"></form>", papplJobGetID(job), papplClientGetLocString(client, _PAPPL_LOC("Cancel Job")));
+  }
+
+  if (show_hold)
+  {
+    papplClientHTMLStartForm(client, uri, false);
+    papplClientHTMLPrintf(client, "<input type=\"hidden\" name=\"action\" value=\"hold-job\"><input type=\"hidden\" name=\"job-id\" value=\"%d\"><input type=\"submit\" value=\"%s\"></form>", papplJobGetID(job), papplClientGetLocString(client, _PAPPL_LOC("Hold Job")));
+  }
+
+  if (show_release)
+  {
+    papplClientHTMLStartForm(client, uri, false);
+    papplClientHTMLPrintf(client, "<input type=\"hidden\" name=\"action\" value=\"release-job\"><input type=\"hidden\" name=\"job-id\" value=\"%d\"><input type=\"submit\" value=\"%s\"></form>", papplJobGetID(job), papplClientGetLocString(client, _PAPPL_LOC("Release Job")));
+  }
+
+  if (papplJobGetState(job) >= IPP_JSTATE_ABORTED && job->filename)
+  {
+    papplClientHTMLStartForm(client, uri, false);
+    papplClientHTMLPrintf(client, "<input type=\"hidden\" name=\"action\" value=\"reprint-job\"><input type=\"hidden\" name=\"job-id\" value=\"%d\"><input type=\"submit\" value=\"%s\"></form>", papplJobGetID(job), papplClientGetLocString(client, _PAPPL_LOC("Reprint Job")));
+  }
+
+  papplClientHTMLPuts(client, "</td></tr>\n");
 }
 
 
