@@ -42,7 +42,7 @@ papplPrinterCancelAllJobs(
   // Loop through all jobs and cancel them.
   //
   // Since we have a writer lock, it is safe to use cupsArrayGetFirst/Last...
-  pthread_rwlock_wrlock(&printer->rwlock);
+  _papplRWLockWrite(printer);
 
   for (job = (pappl_job_t *)cupsArrayGetFirst(printer->active_jobs); job; job = (pappl_job_t *)cupsArrayGetNext(printer->active_jobs))
   {
@@ -63,7 +63,7 @@ papplPrinterCancelAllJobs(
     }
   }
 
-  pthread_rwlock_unlock(&printer->rwlock);
+  _papplRWUnlock(printer);
 
   if (!printer->system->clean_time)
     printer->system->clean_time = time(NULL) + 60;
@@ -154,19 +154,41 @@ papplPrinterCreate(
     IPP_OP_CANCEL_JOBS,
     IPP_OP_CANCEL_MY_JOBS,
     IPP_OP_CLOSE_JOB,
-    IPP_OP_IDENTIFY_PRINTER
+    IPP_OP_IDENTIFY_PRINTER,
+    IPP_OP_HOLD_JOB,
+    IPP_OP_RELEASE_JOB,
+    IPP_OP_HOLD_NEW_JOBS,
+    IPP_OP_RELEASE_HELD_NEW_JOBS
   };
   static const char * const charset[] =	// charset-supported values
   {
     "us-ascii",
     "utf-8"
   };
+  static const char * const client_info[] =
+  {					// client-info-supported values
+    "client-name",
+    "client-patches",
+    "client-string-version",
+    "client-version"
+  };
   static const char * const compression[] =
   {					// compression-supported values
     "deflate",
     "gzip",
     "none"
   };
+  static const char * const job_hold_until[] =
+  {					// job-hold-until-supported values
+    "day-time",
+    "evening",
+    "indefinite",
+    "night",
+    "no-hold",
+    "second-shift",
+    "third-shift",
+    "weekend"
+  };
   static const char * const multiple_document_handling[] =
   {					// multiple-document-handling-supported values
     "separate-documents-uncollated-copies",
@@ -188,6 +210,11 @@ papplPrinterCreate(
     "text-and-graphic",
     "text"
   };
+  static const char *print_processing[] =
+  {					// print-processing-attributes-supported
+    "print-color-mode",
+    "printer-resolution"
+  };
   static const int print_quality[] =	// print-quality-supported
   {
     IPP_QUALITY_DRAFT,
@@ -342,7 +369,7 @@ papplPrinterCreate(
   printer->next_job_id        = 1;
   printer->max_active_jobs    = (system->options & PAPPL_SOPTIONS_MULTI_QUEUE) ? 0 : 1;
   printer->max_completed_jobs = 100;
-  printer->usb_vendor_id      = 0x1209;	// See <pid.codes>
+  printer->usb_vendor_id      = 0x1209;	// See <https://pid.codes>
   printer->usb_product_id     = 0x8011;
 
   if (!printer->name || !printer->dns_sd_name || !printer->resource || (device_id && !printer->device_id) || !printer->device_uri || !printer->driver_name || !printer->attrs)
@@ -363,6 +390,7 @@ papplPrinterCreate(
     {
       pappl_device_t	*device;	// Connection to printer
 
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Opening device for auto-setup.");
       if ((device = papplDeviceOpen(device_uri, "auto", papplLogDevice, system)) != NULL)
       {
         char	new_id[1024];		// New 1284 device ID
@@ -370,6 +398,8 @@ papplPrinterCreate(
         if (papplDeviceGetID(device, new_id, sizeof(new_id)))
           printer->device_id = strdup(new_id);
 
+        papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Closing device for auto-setup.");
+
         papplDeviceClose(device);
       }
     }
@@ -428,6 +458,8 @@ papplPrinterCreate(
         format = "PS";
       else if (!strcmp(format, "application/vnd.hp-postscript"))
         format = "PCL";
+      else if (!strcmp(format, "application/vnd.zebra-epl"))
+        format = "EPL";
       else if (!strcmp(format, "application/vnd.zebra-zpl"))
         format = "ZPL";
       else if (!strcmp(format, "image/jpeg"))
@@ -435,7 +467,7 @@ papplPrinterCreate(
       else if (!strcmp(format, "image/png"))
         format = "PNG";
       else if (!strcmp(format, "image/pwg-raster"))
-        format = "PWG";
+        format = "PWGRaster";
       else if (!strcmp(format, "image/urf"))
         format = "URF";
       else if (!strcmp(format, "text/plain"))
@@ -467,12 +499,18 @@ papplPrinterCreate(
   // charset-supported
   ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_CHARSET), "charset-supported", sizeof(charset) / sizeof(charset[0]), NULL, charset);
 
+  // client-info-supported
+  ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "client-info-supported", (int)(sizeof(client_info) / sizeof(client_info[0])), NULL, client_info);
+
   // compression-supported
   ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "compression-supported", (int)(sizeof(compression) / sizeof(compression[0])), NULL, compression);
 
   // copies-default
   ippAddInteger(printer->attrs, IPP_TAG_PRINTER, IPP_TAG_INTEGER, "copies-default", 1);
 
+  // device-uuid
+  ippAddString(printer->attrs, IPP_TAG_PRINTER, IPP_TAG_URI, "device-uuid", NULL, uuid);
+
   // document-format-default
   ippAddString(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_MIMETYPE), "document-format-default", NULL, "application/octet-stream");
 
@@ -480,7 +518,16 @@ papplPrinterCreate(
   ippAddString(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_LANGUAGE), "generated-natural-language-supported", NULL, "en");
 
   // ipp-versions-supported
-  ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "ipp-versions-supported", (int)(sizeof(ipp_versions) / sizeof(ipp_versions[0])), NULL, ipp_versions);
+  ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "ipp-versions-supported", (cups_len_t)(sizeof(ipp_versions) / sizeof(ipp_versions[0])), NULL, ipp_versions);
+
+  // job-hold-until-default
+  ippAddString(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "job-hold-until-default", NULL, "no-hold");
+
+  // job-hold-until-supported
+  ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "job-hold-until-supported", (cups_len_t)(sizeof(job_hold_until) / sizeof(job_hold_until[0])), NULL, job_hold_until);
+
+  // job-hold-until-time-supported
+  ippAddBoolean(printer->attrs, IPP_TAG_PRINTER, "job-hold-until-time-supported", 1);
 
   // job-ids-supported
   ippAddBoolean(printer->attrs, IPP_TAG_PRINTER, "job-ids-supported", 1);
@@ -500,31 +547,6 @@ papplPrinterCreate(
   // job-sheets-supported
   ippAddString(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_NAME), "job-sheets-supported", NULL, "none");
 
-  // job-spooling-supported
-  ippAddString(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "job-spooling-supported", NULL, printer->max_active_jobs > 1 ? "spool" : "stream");
-
-  if (_papplSystemFindMIMEFilter(system, "image/jpeg", "image/pwg-raster"))
-  {
-    static const char * const jpeg_features_supported[] =
-    {					// "jpeg-features-supported" values
-      "arithmetic",
-      "cmyk",
-      "progressive"
-    };
-
-    // jpeg-features-supported
-    ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "jpeg-features-supported", (int)(sizeof(jpeg_features_supported) / sizeof(jpeg_features_supported[0])), NULL, jpeg_features_supported);
-
-    // jpeg-k-octets-supported
-    ippAddRange(printer->attrs, IPP_TAG_PRINTER, "jpeg-k-octets-supported", 0, k_supported);
-
-    // jpeg-x-dimension-supported
-    ippAddRange(printer->attrs, IPP_TAG_PRINTER, "jpeg-x-dimension-supported", 0, 16384);
-
-    // jpeg-y-dimension-supported
-    ippAddRange(printer->attrs, IPP_TAG_PRINTER, "jpeg-y-dimension-supported", 1, 16384);
-  }
-
   // multiple-document-handling-supported
   ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "multiple-document-handling-supported", sizeof(multiple_document_handling) / sizeof(multiple_document_handling[0]), NULL, multiple_document_handling);
 
@@ -570,6 +592,9 @@ papplPrinterCreate(
   // print-content-optimize-supported
   ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "print-content-optimize-supported", (int)(sizeof(print_content_optimize) / sizeof(print_content_optimize[0])), NULL, print_content_optimize);
 
+  // print-processing-attributes-supported
+  ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "print-processing-attributes-supported", (int)(sizeof(print_processing) / sizeof(print_processing[0])), NULL, print_processing);
+
   // print-quality-supported
   ippAddIntegers(printer->attrs, IPP_TAG_PRINTER, IPP_TAG_ENUM, "print-quality-supported", (int)(sizeof(print_quality) / sizeof(print_quality[0])), print_quality);
 
@@ -595,6 +620,9 @@ papplPrinterCreate(
   // printer-uuid
   ippAddString(printer->attrs, IPP_TAG_PRINTER, IPP_TAG_URI, "printer-uuid", NULL, uuid);
 
+  // requesting-user-uri-supported
+  ippAddBoolean(printer->attrs, IPP_TAG_PRINTER, "requesting-user-uri-supported", 1);
+
   // uri-security-supported
   if (system->options & PAPPL_SOPTIONS_NO_TLS)
     ippAddString(printer->attrs, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "uri-security-supported", NULL, "none");
@@ -630,8 +658,14 @@ papplPrinterCreate(
 	// Detach the main thread from the raw thread to prevent hangs...
 	pthread_detach(tid);
 
+        _papplRWLockRead(printer);
 	while (!printer->raw_active)
+	{
+	  _papplRWUnlock(printer);
 	  usleep(1000);			// Wait for raw thread to start
+	  _papplRWLockRead(printer);
+	}
+	_papplRWUnlock(printer);
       }
     }
   }
@@ -645,9 +679,6 @@ papplPrinterCreate(
     snprintf(path, sizeof(path), "%s/", printer->uriname);
     papplSystemAddResourceCallback(system, path, "text/html", (pappl_resource_cb_t)_papplPrinterWebHome, printer);
 
-    snprintf(path, sizeof(path), "%s/cancel", printer->uriname);
-    papplSystemAddResourceCallback(system, path, "text/html", (pappl_resource_cb_t)_papplPrinterWebCancelJob, printer);
-
     snprintf(path, sizeof(path), "%s/cancelall", printer->uriname);
     papplSystemAddResourceCallback(system, path, "text/html", (pappl_resource_cb_t)_papplPrinterWebCancelAllJobs, printer);
 
@@ -671,9 +702,6 @@ papplPrinterCreate(
     papplSystemAddResourceCallback(system, path, "text/html", (pappl_resource_cb_t)_papplPrinterWebDefaults, printer);
     papplPrinterAddLink(printer, _PAPPL_LOC("Printing Defaults"), path, PAPPL_LOPTIONS_NAVIGATION | PAPPL_LOPTIONS_STATUS);
 
-    snprintf(path, sizeof(path), "%s/reprint", printer->uriname);
-    papplSystemAddResourceCallback(system, path, "text/html", (pappl_resource_cb_t)_papplPrinterWebReprintJob, printer);
-
     if (printer->driver_data.has_supplies)
     {
       snprintf(path, sizeof(path), "%s/supplies", printer->uriname);
@@ -704,13 +732,17 @@ _papplPrinterDelete(
 
 
   // Let USB/raw printing threads know to exit
+  _papplRWLockWrite(printer);
   printer->is_deleted = true;
 
   while (printer->raw_active || printer->usb_active)
   {
     // Wait for threads to finish
-    usleep(100000);
+    _papplRWUnlock(printer);
+    usleep(1000);
+    _papplRWLockRead(printer);
   }
+  _papplRWUnlock(printer);
 
   // Close raw listener sockets...
   for (i = 0; i < printer->num_raw_listeners; i ++)
@@ -768,6 +800,8 @@ _papplPrinterDelete(
 
   cupsArrayDelete(printer->links);
 
+  pthread_rwlock_destroy(&printer->rwlock);
+
   free(printer);
 }
 
@@ -791,9 +825,9 @@ papplPrinterDelete(
   papplSystemAddEvent(system, printer, NULL, PAPPL_EVENT_PRINTER_DELETED | PAPPL_EVENT_SYSTEM_CONFIG_CHANGED, NULL);
 
   // Remove the printer from the system object...
-  pthread_rwlock_wrlock(&system->rwlock);
+  _papplRWLockWrite(system);
   cupsArrayRemove(system->printers, printer);
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 
   _papplSystemConfigChanged(system);
 }
@@ -802,15 +836,20 @@ papplPrinterDelete(
 //
 // 'papplPrinterOpenFile()' - Create or open a file for a printer.
 //
-// This function creates or opens a file for a printer.  The "fname" and
-// "fnamesize" arguments specify the location and size of a buffer to store the
-// printer filename, which incorporates the "directory", printer ID, resource
-// name, and "ext" values.  The job name is "sanitized" to only contain
-// alphanumeric characters.
+// This function creates, opens, or removes a file for a printer.  The "fname"
+// and "fnamesize" arguments specify the location and size of a buffer to store
+// the printer filename, which incorporates the "directory", printer ID,
+// resource name, and "ext" values.  The resource name is "sanitized" to only
+// contain alphanumeric characters.
 //
-// The "mode" argument is "r" to read an existing job file or "w" to write a
-// new job file.  New files are created with restricted permissions for
-// security purposes.
+// The "mode" argument is "r" to read an existing printer file, "w" to write a
+// new printer file, or "x" to remove an exitsing printer file.  New files are
+// created with restricted permissions for security purposes.
+//
+// For the "r" and "w" modes, the return value is the file descriptor number on
+// success or `-1` on error.  For the "x" mode, the return value is `0` on
+// success and `-1` on error.  The `errno` variable is set appropriately on
+// error.
 //
 
 int					// O - File descriptor or -1 on error
