@@ -395,9 +395,9 @@ _papplSNMPWrite(
   temp.ipv4.sin_port = htons(_PAPPL_SNMP_PORT);
 
 #if _WIN32
-  return (sendto(fd, buffer, (int)bytes, 0, (void *)&temp, (socklen_t)httpAddrLength(&temp)) == bytes);
+  return (sendto(fd, buffer, (int)bytes, 0, (void *)&temp, (socklen_t)httpAddrGetLength(&temp)) == bytes);
 #else
-  return (sendto(fd, buffer, (size_t)bytes, 0, (void *)&temp, (socklen_t)httpAddrLength(&temp)) == bytes);
+  return (sendto(fd, buffer, (size_t)bytes, 0, (void *)&temp, (socklen_t)httpAddrGetLength(&temp)) == bytes);
 #endif // _WIN32
 }
 
@@ -742,13 +742,15 @@ asn1_get_integer(
 
   if (length > sizeof(int))
   {
-    (*buffer) += length;
+    if (length > (unsigned)(bufend - *buffer))
+      *buffer = bufend;
+    else
+      (*buffer) += length;
+
     return (0);
   }
 
-  for (value = (**buffer & 0x80) ? ~0 : 0;
-       length > 0 && *buffer < bufend;
-       length --, (*buffer) ++)
+  for (value = (**buffer & 0x80) ? ~0 : 0; length > 0 && *buffer < bufend; length --, (*buffer) ++)
     value = ((value & 0xffffff) << 8) | **buffer;
 
   return (value);
@@ -776,16 +778,17 @@ asn1_get_length(unsigned char **buffer,	// IO - Pointer in buffer
   {
     int	count;				// Number of bytes for length
 
-
     if ((count = length & 127) > sizeof(unsigned))
     {
-      (*buffer) += count;
+      if (count > (bufend - *buffer))
+	*buffer = bufend;
+      else
+	(*buffer) += count;
+
       return (0);
     }
 
-    for (length = 0;
-	 count > 0 && *buffer < bufend;
-	 count --, (*buffer) ++)
+    for (length = 0; count > 0 && *buffer < bufend; count --, (*buffer) ++)
       length = (length << 8) | **buffer;
   }
 
@@ -812,7 +815,14 @@ asn1_get_oid(
 
 
   if (*buffer >= bufend)
+  {
     return (0);
+  }
+  else if (length > (unsigned)(bufend - *buffer))
+  {
+    *buffer = bufend;
+    return (0);
+  }
 
   valend = *buffer + length;
   oidptr = oid;
@@ -821,7 +831,7 @@ asn1_get_oid(
   if (valend > bufend)
     valend = bufend;
 
-  number = asn1_get_packed(buffer, bufend);
+  number = asn1_get_packed(buffer, valend);
 
   if (number < 80)
   {
@@ -838,7 +848,7 @@ asn1_get_oid(
 
   while (*buffer < valend)
   {
-    number = asn1_get_packed(buffer, bufend);
+    number = asn1_get_packed(buffer, valend);
 
     if (oidptr < oidend)
       *oidptr++ = number;
