@@ -159,6 +159,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Chad\" = \"Chad\";\n"
 "\"Change Access Password\" = \"Change Access Password\";\n"
 "\"Change Log Level\" = \"Change Log Level\";\n"
+"\"Change Network Settings\" = \"Change Network Settings\";\n"
 "\"Change Wi-Fi Network\" = \"Change Wi-Fi Network\";\n"
 "\"Changes Saved.\" = \"Changes Saved.\";\n"
 "\"Changes saved.\" = \"Changes saved.\";\n"
@@ -172,9 +173,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Configuration\" = \"Configuration\";\n"
 "\"Configure Media\" = \"Configure Media\";\n"
 "\"Confirm Cancel All\" = \"Confirm Cancel All\";\n"
-"\"Confirm Cancel Job\" = \"Confirm Cancel Job\";\n"
 "\"Confirm Delete Printer\" = \"Confirm Delete Printer\";\n"
-"\"Confirm Reprint Job\" = \"Confirm Reprint Job\";\n"
 "\"Congo\" = \"Congo\";\n"
 "\"Congo, Democratic Republic of the\" = \"Congo, Democratic Republic of the\";\n"
 "\"Contact\" = \"Contact\";\n"
@@ -192,6 +191,8 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Cyprus\" = \"Cyprus\";\n"
 "\"Czechia\" = \"Czechia\";\n"
 "\"Côte d'Ivoire\" = \"Côte d'Ivoire\";\n"
+"\"DHCP\" = \"DHCP\";\n"
+"\"DHCP w/Manual Address\" = \"DHCP w/Manual Address\";\n"
 "\"DNS-SD Service Name\" = \"DNS-SD Service Name\";\n"
 "\"Debugging\" = \"Debugging\";\n"
 "\"Default printer set.\" = \"Default printer set.\";\n"
@@ -199,6 +200,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Delete Printer\" = \"Delete Printer\";\n"
 "\"Denmark\" = \"Denmark\";\n"
 "\"Djibouti\" = \"Djibouti\";\n"
+"\"Domain Name\" = \"Domain Name\";\n"
 "\"Dominica\" = \"Dominica\";\n"
 "\"Dominican Republic\" = \"Dominican Republic\";\n"
 "\"Download Certificate Request File\" = \"Download Certificate Request File\";\n"
@@ -210,8 +212,6 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Errors\" = \"Errors\";\n"
 "\"Estonia\" = \"Estonia\";\n"
 "\"Eswatini\" = \"Eswatini\";\n"
-"\"Ethernet %d: %s\" = \"Ethernet %d: %s\";\n"
-"\"Ethernet: %s\" = \"Ethernet: %s\";\n"
 "\"Ethiopia\" = \"Ethiopia\";\n"
 "\"Falkland Islands (Malvinas)\" = \"Falkland Islands (Malvinas)\";\n"
 "\"Faroe Islands\" = \"Faroe Islands\";\n"
@@ -241,26 +241,39 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Guyana\" = \"Guyana\";\n"
 "\"Haiti\" = \"Haiti\";\n"
 "\"Heard Island and McDonald Islands\" = \"Heard Island and McDonald Islands\";\n"
+"\"Hold Job\" = \"Hold Job\";\n"
+"\"Hold New Jobs\" = \"Hold New Jobs\";\n"
+"\"Holding new jobs.\" = \"Holding new jobs.\";\n"
 "\"Holy See\" = \"Holy See\";\n"
 "\"Honduras\" = \"Honduras\";\n"
 "\"Hong Kong\" = \"Hong Kong\";\n"
 "\"Hostname\" = \"Hostname\";\n"
 "\"Human-Readable Location\" = \"Human-Readable Location\";\n"
 "\"Hungary\" = \"Hungary\";\n"
-"\"IPv4 Addresses\" = \"IPv4 Addresses\";\n"
-"\"IPv6 Addresses\" = \"IPv6 Addresses\";\n"
+"\"IPv4 Address\" = \"IPv4 Address\";\n"
+"\"IPv4 Configuration\" = \"IPv4 Configuration\";\n"
+"\"IPv4 Gateway\" = \"IPv4 Gateway\";\n"
+"\"IPv4 Netmask\" = \"IPv4 Netmask\";\n"
+"\"IPv6 Address\" = \"IPv6 Address\";\n"
+"\"IPv6 Configuration\" = \"IPv6 Configuration\";\n"
+"\"IPv6 Link-Local\" = \"IPv6 Link-Local\";\n"
+"\"IPv6 Prefix Length\" = \"IPv6 Prefix Length\";\n"
 "\"Iceland\" = \"Iceland\";\n"
 "\"Identify Printer\" = \"Identify Printer\";\n"
 "\"India\" = \"India\";\n"
 "\"Indonesia\" = \"Indonesia\";\n"
 "\"Informational\" = \"Informational\";\n"
 "\"Install TLS Certificate\" = \"Install TLS Certificate\";\n"
-"\"Invalid GET data.\" = \"Invalid GET data.\";\n"
+"\"Invalid IPv4 address.\" = \"Invalid IPv4 address.\";\n"
+"\"Invalid IPv4 netmask.\" = \"Invalid IPv4 netmask.\";\n"
+"\"Invalid IPv6 address.\" = \"Invalid IPv6 address.\";\n"
+"\"Invalid IPv6 prefix length.\" = \"Invalid IPv6 prefix length.\";\n"
 "\"Invalid Job ID.\" = \"Invalid Job ID.\";\n"
 "\"Invalid certificate or private key.\" = \"Invalid certificate or private key.\";\n"
 "\"Invalid form data.\" = \"Invalid form data.\";\n"
 "\"Invalid form submission.\" = \"Invalid form submission.\";\n"
 "\"Invalid job ID.\" = \"Invalid job ID.\";\n"
+"\"Invalid network configuration.\" = \"Invalid network configuration.\";\n"
 "\"Iran (Islamic Republic of)\" = \"Iran (Islamic Republic of)\";\n"
 "\"Iraq\" = \"Iraq\";\n"
 "\"Ireland\" = \"Ireland\";\n"
@@ -271,8 +284,10 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Japan\" = \"Japan\";\n"
 "\"Jersey\" = \"Jersey\";\n"
 "\"Job #\" = \"Job #\";\n"
+"\"Job canceled.\" = \"Job canceled.\";\n"
+"\"Job held.\" = \"Job held.\";\n"
+"\"Job released.\" = \"Job released.\";\n"
 "\"Jobs\" = \"Jobs\";\n"
-"\"Joining Wi-Fi network.\" = \"Joining Wi-Fi network.\";\n"
 "\"Jordan\" = \"Jordan\";\n"
 "\"Kazakhstan\" = \"Kazakhstan\";\n"
 "\"Kenya\" = \"Kenya\";\n"
@@ -303,6 +318,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Maldives\" = \"Maldives\";\n"
 "\"Mali\" = \"Mali\";\n"
 "\"Malta\" = \"Malta\";\n"
+"\"Manual Configuration\" = \"Manual Configuration\";\n"
 "\"Marshall Islands\" = \"Marshall Islands\";\n"
 "\"Martinique\" = \"Martinique\";\n"
 "\"Mauritania\" = \"Mauritania\";\n"
@@ -312,6 +328,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Mexico\" = \"Mexico\";\n"
 "\"Micronesia (Federated States of)\" = \"Micronesia (Federated States of)\";\n"
 "\"Missing action.\" = \"Missing action.\";\n"
+"\"Missing job ID.\" = \"Missing job ID.\";\n"
 "\"Missing private key.\" = \"Missing private key.\";\n"
 "\"Moldova, Republic of\" = \"Moldova, Republic of\";\n"
 "\"Monaco\" = \"Monaco\";\n"
@@ -342,6 +359,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Northern Mariana Islands\" = \"Northern Mariana Islands\";\n"
 "\"Norway\" = \"Norway\";\n"
 "\"Not set\" = \"Not set\";\n"
+"\"Off\" = \"Off\";\n"
 "\"Offset\" = \"Offset\";\n"
 "\"Oman\" = \"Oman\";\n"
 "\"Options:\" = \"Options:\";\n"
@@ -370,6 +388,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Poland\" = \"Poland\";\n"
 "\"Portrait\" = \"Portrait\";\n"
 "\"Portugal\" = \"Portugal\";\n"
+"\"Primary DNS\" = \"Primary DNS\";\n"
 "\"Print Group\" = \"Print Group\";\n"
 "\"Print Test Page\" = \"Print Test Page\";\n"
 "\"Print job options:\" = \"Print job options:\";\n"
@@ -385,7 +404,11 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Puerto Rico\" = \"Puerto Rico\";\n"
 "\"Qatar\" = \"Qatar\";\n"
 "\"Queued at %s\" = \"Queued at %s\";\n"
+"\"Release Held New Jobs\" = \"Release Held New Jobs\";\n"
+"\"Release Job\" = \"Release Job\";\n"
+"\"Released held new jobs.\" = \"Released held new jobs.\";\n"
 "\"Reprint Job\" = \"Reprint Job\";\n"
+"\"Reprinted job.\" = \"Reprinted job.\";\n"
 "\"Rescan\" = \"Rescan\";\n"
 "\"Resume Printing\" = \"Resume Printing\";\n"
 "\"Reverse Landscape\" = \"Reverse Landscape\";\n"
@@ -408,6 +431,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Sao Tome and Principe\" = \"Sao Tome and Principe\";\n"
 "\"Saudi Arabia\" = \"Saudi Arabia\";\n"
 "\"Save Changes\" = \"Save Changes\";\n"
+"\"Secondary DNS\" = \"Secondary DNS\";\n"
 "\"Security\" = \"Security\";\n"
 "\"Select Driver\" = \"Select Driver\";\n"
 "\"Senegal\" = \"Senegal\";\n"
@@ -467,6 +491,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unable to identify printer.\" = \"Unable to identify printer.\";\n"
 "\"Unable to join Wi-Fi network.\" = \"Unable to join Wi-Fi network.\";\n"
 "\"Unable to lookup address.\" = \"Unable to lookup address.\";\n"
+"\"Unable to save network configuration.\" = \"Unable to save network configuration.\";\n"
 "\"Unable to use that driver.\" = \"Unable to use that driver.\";\n"
 "\"United Arab Emirates\" = \"United Arab Emirates\";\n"
 "\"United Kingdom\" = \"United Kingdom\";\n"
@@ -492,10 +517,8 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Wallis and Futuna\" = \"Wallis and Futuna\";\n"
 "\"Warning\" = \"Warning\";\n"
 "\"Western Sahara\" = \"Western Sahara\";\n"
-"\"Wi-Fi %d: %s\" = \"Wi-Fi %d: %s\";\n"
 "\"Wi-Fi Configuration\" = \"Wi-Fi Configuration\";\n"
 "\"Wi-Fi Network\" = \"Wi-Fi Network\";\n"
-"\"Wi-Fi: %s\" = \"Wi-Fi: %s\";\n"
 "\"Width\" = \"Width\";\n"
 "\"Wrong old password.\" = \"Wrong old password.\";\n"
 "\"Yemen\" = \"Yemen\";\n"
@@ -574,6 +597,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"finishings.97\" = \"Parallel fold\";\n"
 "\"finishings.98\" = \"Poster fold\";\n"
 "\"finishings.99\" = \"Right gate fold\";\n"
+"\"holding new jobs\" = \"holding new jobs\";\n"
 "\"job-state-reasons.aborted-by-system\" = \"Stopping.\";\n"
 "\"job-state-reasons.compression-error\" = \"Decompression error.\";\n"
 "\"job-state-reasons.document-format-error\" = \"Document format error.\";\n"
