@@ -159,6 +159,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Chad\" = \"Chad\";\n"
 "\"Change Access Password\" = \"Cambiar contraseña\";\n"
 "\"Change Log Level\" = \"Nivel de registro de cambios\";\n"
+"\"Change Network Settings\" = \"Change Network Settings\";\n"
 "\"Change Wi-Fi Network\" = \"Cambiar red Wi-Fi\";\n"
 "\"Changes Saved.\" = \"Cambios guardados.\";\n"
 "\"Changes saved.\" = \"Cambios salvados.\";\n"
@@ -172,9 +173,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Configuration\" = \"Configuración\";\n"
 "\"Configure Media\" = \"Configure Media\";\n"
 "\"Confirm Cancel All\" = \"Confirmación Todos\";\n"
-"\"Confirm Cancel Job\" = \"Confirmación de empleo\";\n"
 "\"Confirm Delete Printer\" = \"Confirmar la impresora\";\n"
-"\"Confirm Reprint Job\" = \"Confirmación de trabajo de reimpresión\";\n"
 "\"Congo\" = \"Congo\";\n"
 "\"Congo, Democratic Republic of the\" = \"Congo, República Democrática del Congo\";\n"
 "\"Contact\" = \"Contacto\";\n"
@@ -192,6 +191,8 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Cyprus\" = \"Chipre\";\n"
 "\"Czechia\" = \"Czechia\";\n"
 "\"Côte d'Ivoire\" = \"Cte d'Ivoire\";\n"
+"\"DHCP\" = \"DHCP\";\n"
+"\"DHCP w/Manual Address\" = \"DHCP w/Manual Address\";\n"
 "\"DNS-SD Service Name\" = \"DNS-SD Nombre del servicio\";\n"
 "\"Debugging\" = \"Debugging\";\n"
 "\"Default printer set.\" = \"Juego de impresora predeterminado.\";\n"
@@ -199,6 +200,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Delete Printer\" = \"Eliminar la impresora\";\n"
 "\"Denmark\" = \"Dinamarca\";\n"
 "\"Djibouti\" = \"Djibouti\";\n"
+"\"Domain Name\" = \"Domain Name\";\n"
 "\"Dominica\" = \"Dominica\";\n"
 "\"Dominican Republic\" = \"República Dominicana\";\n"
 "\"Download Certificate Request File\" = \"Descargar certificado Solicitar archivo\";\n"
@@ -210,8 +212,6 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Errors\" = \"Errores\";\n"
 "\"Estonia\" = \"Estonia\";\n"
 "\"Eswatini\" = \"Eswatini\";\n"
-"\"Ethernet %d: %s\" = \"Ethernet %d: %s\";\n"
-"\"Ethernet: %s\" = \"Ethernet: %s\";\n"
 "\"Ethiopia\" = \"Etiopía\";\n"
 "\"Falkland Islands (Malvinas)\" = \"Islas Malvinas (Falkland Islands)\";\n"
 "\"Faroe Islands\" = \"Islas Feroe\";\n"
@@ -241,26 +241,39 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Guyana\" = \"Guyana\";\n"
 "\"Haiti\" = \"Haití\";\n"
 "\"Heard Island and McDonald Islands\" = \"Heard Island y McDonald Islands\";\n"
+"\"Hold Job\" = \"Hold Job\";\n"
+"\"Hold New Jobs\" = \"Hold New Jobs\";\n"
+"\"Holding new jobs.\" = \"Holding new jobs.\";\n"
 "\"Holy See\" = \"Santa Sede\";\n"
 "\"Honduras\" = \"Honduras\";\n"
 "\"Hong Kong\" = \"Hong Kong\";\n"
 "\"Hostname\" = \"Nombre de host\";\n"
 "\"Human-Readable Location\" = \"Ubicación legible por humanos\";\n"
 "\"Hungary\" = \"Hungría\";\n"
-"\"IPv4 Addresses\" = \"Direcciones IPv4\";\n"
-"\"IPv6 Addresses\" = \"Direcciones IPv6\";\n"
+"\"IPv4 Address\" = \"IPv4 Address\";\n"
+"\"IPv4 Configuration\" = \"IPv4 Configuration\";\n"
+"\"IPv4 Gateway\" = \"IPv4 Gateway\";\n"
+"\"IPv4 Netmask\" = \"IPv4 Netmask\";\n"
+"\"IPv6 Address\" = \"IPv6 Address\";\n"
+"\"IPv6 Configuration\" = \"IPv6 Configuration\";\n"
+"\"IPv6 Link-Local\" = \"IPv6 Link-Local\";\n"
+"\"IPv6 Prefix Length\" = \"IPv6 Prefix Length\";\n"
 "\"Iceland\" = \"Islandia\";\n"
 "\"Identify Printer\" = \"Identifique la impresora\";\n"
 "\"India\" = \"India\";\n"
 "\"Indonesia\" = \"Indonesia\";\n"
 "\"Informational\" = \"Información\";\n"
 "\"Install TLS Certificate\" = \"Instala TLS Certificado\";\n"
-"\"Invalid GET data.\" = \"Datos inválidos.\";\n"
+"\"Invalid IPv4 address.\" = \"Invalid IPv4 address.\";\n"
+"\"Invalid IPv4 netmask.\" = \"Invalid IPv4 netmask.\";\n"
+"\"Invalid IPv6 address.\" = \"Invalid IPv6 address.\";\n"
+"\"Invalid IPv6 prefix length.\" = \"Invalid IPv6 prefix length.\";\n"
 "\"Invalid Job ID.\" = \"ID de trabajo inválido.\";\n"
 "\"Invalid certificate or private key.\" = \"Certificado inválido o clave privada.\";\n"
 "\"Invalid form data.\" = \"Datos de formulario inválidos.\";\n"
 "\"Invalid form submission.\" = \"Sumisión de formulario inválido.\";\n"
 "\"Invalid job ID.\" = \"ID de trabajo inválido.\";\n"
+"\"Invalid network configuration.\" = \"Invalid network configuration.\";\n"
 "\"Iran (Islamic Republic of)\" = \"Irán (República Islámica del)\";\n"
 "\"Iraq\" = \"Iraq\";\n"
 "\"Ireland\" = \"Irlanda\";\n"
@@ -271,8 +284,10 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Japan\" = \"Japón\";\n"
 "\"Jersey\" = \"Jersey\";\n"
 "\"Job #\" = \"Job\";\n"
+"\"Job canceled.\" = \"Job canceled.\";\n"
+"\"Job held.\" = \"Job held.\";\n"
+"\"Job released.\" = \"Job released.\";\n"
 "\"Jobs\" = \"Empleo\";\n"
-"\"Joining Wi-Fi network.\" = \"Unirse a la red Wi-Fi.\";\n"
 "\"Jordan\" = \"Jordania\";\n"
 "\"Kazakhstan\" = \"Kazajstán\";\n"
 "\"Kenya\" = \"Kenya\";\n"
@@ -303,6 +318,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Maldives\" = \"Maldivas\";\n"
 "\"Mali\" = \"Malí\";\n"
 "\"Malta\" = \"Malta\";\n"
+"\"Manual Configuration\" = \"Manual Configuration\";\n"
 "\"Marshall Islands\" = \"Islas Marshall\";\n"
 "\"Martinique\" = \"Martinica\";\n"
 "\"Mauritania\" = \"Mauritania\";\n"
@@ -312,6 +328,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Mexico\" = \"México\";\n"
 "\"Micronesia (Federated States of)\" = \"Micronesia (Estados Federados de)\";\n"
 "\"Missing action.\" = \"Falta de acción.\";\n"
+"\"Missing job ID.\" = \"Missing job ID.\";\n"
 "\"Missing private key.\" = \"Falta la llave privada.\";\n"
 "\"Moldova, Republic of\" = \"República de Moldova\";\n"
 "\"Monaco\" = \"Mónaco\";\n"
@@ -342,6 +359,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Northern Mariana Islands\" = \"Islas Marianas del Norte\";\n"
 "\"Norway\" = \"Noruega\";\n"
 "\"Not set\" = \"No está listo.\";\n"
+"\"Off\" = \"Off\";\n"
 "\"Offset\" = \"Offset\";\n"
 "\"Oman\" = \"Omán\";\n"
 "\"Options:\" = \"Opciones:\";\n"
@@ -370,6 +388,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Poland\" = \"Polonia\";\n"
 "\"Portrait\" = \"Retrato\";\n"
 "\"Portugal\" = \"Portugal\";\n"
+"\"Primary DNS\" = \"Primary DNS\";\n"
 "\"Print Group\" = \"Print Group\";\n"
 "\"Print Test Page\" = \"Página de prueba de impresión\";\n"
 "\"Print job options:\" = \"Opciones de trabajo para imprimir:\";\n"
@@ -385,7 +404,11 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Puerto Rico\" = \"Puerto Rico\";\n"
 "\"Qatar\" = \"Qatar\";\n"
 "\"Queued at %s\" = \"Queued at %s\";\n"
+"\"Release Held New Jobs\" = \"Release Held New Jobs\";\n"
+"\"Release Job\" = \"Release Job\";\n"
+"\"Released held new jobs.\" = \"Released held new jobs.\";\n"
 "\"Reprint Job\" = \"Reprint Job\";\n"
+"\"Reprinted job.\" = \"Reprinted job.\";\n"
 "\"Rescan\" = \"Rescan\";\n"
 "\"Resume Printing\" = \"Resume Printing\";\n"
 "\"Reverse Landscape\" = \"Paisaje inverso\";\n"
@@ -408,6 +431,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Sao Tome and Principe\" = \"Santo Tomé y Príncipe\";\n"
 "\"Saudi Arabia\" = \"Arabia Saudita\";\n"
 "\"Save Changes\" = \"Guardar cambios\";\n"
+"\"Secondary DNS\" = \"Secondary DNS\";\n"
 "\"Security\" = \"Seguridad\";\n"
 "\"Select Driver\" = \"Select Driver\";\n"
 "\"Senegal\" = \"Senegal\";\n"
@@ -467,6 +491,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unable to identify printer.\" = \"Incapaz de identificar la impresora.\";\n"
 "\"Unable to join Wi-Fi network.\" = \"Incapaz de unirse a la red Wi-Fi.\";\n"
 "\"Unable to lookup address.\" = \"Incapaz de buscar la dirección.\";\n"
+"\"Unable to save network configuration.\" = \"Unable to save network configuration.\";\n"
 "\"Unable to use that driver.\" = \"Incapaz de usar ese conductor.\";\n"
 "\"United Arab Emirates\" = \"Emiratos Árabes Unidos\";\n"
 "\"United Kingdom\" = \"Reino Unido\";\n"
@@ -492,10 +517,8 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Wallis and Futuna\" = \"Wallis y Futuna\";\n"
 "\"Warning\" = \"Advertencia\";\n"
 "\"Western Sahara\" = \"Sáhara Occidental\";\n"
-"\"Wi-Fi %d: %s\" = \"Wi-Fi %d: %s\";\n"
 "\"Wi-Fi Configuration\" = \"Configuración Wi-Fi\";\n"
 "\"Wi-Fi Network\" = \"Wi-Fi Network\";\n"
-"\"Wi-Fi: %s\" = \"Wi-Fi: %s\";\n"
 "\"Width\" = \"Width\";\n"
 "\"Wrong old password.\" = \"contraseña vieja incorrecta.\";\n"
 "\"Yemen\" = \"Yemen\";\n"
@@ -574,6 +597,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"finishings.97\" = \"Pliegue paralelo\";\n"
 "\"finishings.98\" = \"Cargador\";\n"
 "\"finishings.99\" = \"Puerta derecha\";\n"
+"\"holding new jobs\" = \"holding new jobs\";\n"
 "\"job-state-reasons.aborted-by-system\" = \"Para.\";\n"
 "\"job-state-reasons.compression-error\" = \"Error de descompresión.\";\n"
 "\"job-state-reasons.document-format-error\" = \"Error de formato de documento.\";\n"
