@@ -159,6 +159,7 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Chad\" = \"Tchad\";\n"
 "\"Change Access Password\" = \"Change Access Password\";\n"
 "\"Change Log Level\" = \"Change Log Level\";\n"
+"\"Change Network Settings\" = \"Change Network Settings\";\n"
 "\"Change Wi-Fi Network\" = \"Change Wi-Fi Network\";\n"
 "\"Changes Saved.\" = \"Changements enregistrés.\";\n"
 "\"Changes saved.\" = \"Changements enregistrés.\";\n"
@@ -172,9 +173,7 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Configuration\" = \"Configuration\";\n"
 "\"Configure Media\" = \"Configurer Media\";\n"
 "\"Confirm Cancel All\" = \"Confirmer Annuler Tout\";\n"
-"\"Confirm Cancel Job\" = \"Confirmer Annuler Job\";\n"
 "\"Confirm Delete Printer\" = \"Confirmer Supprimer l'imprimante\";\n"
-"\"Confirm Reprint Job\" = \"Confirmer Reprint Job\";\n"
 "\"Congo\" = \"Congo\";\n"
 "\"Congo, Democratic Republic of the\" = \"Congo, République démocratique du Congo\";\n"
 "\"Contact\" = \"Contact\";\n"
@@ -192,6 +191,8 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Cyprus\" = \"Chypre\";\n"
 "\"Czechia\" = \"Czechia\";\n"
 "\"Côte d'Ivoire\" = \"Cte d'Ivoire\";\n"
+"\"DHCP\" = \"DHCP\";\n"
+"\"DHCP w/Manual Address\" = \"DHCP w/Manual Address\";\n"
 "\"DNS-SD Service Name\" = \"DNS-SD Nom de service\";\n"
 "\"Debugging\" = \"Debugging\";\n"
 "\"Default printer set.\" = \"Imprimante par défaut.\";\n"
@@ -199,6 +200,7 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Delete Printer\" = \"Supprimer Printer\";\n"
 "\"Denmark\" = \"Danemark\";\n"
 "\"Djibouti\" = \"Djibouti\";\n"
+"\"Domain Name\" = \"Domain Name\";\n"
 "\"Dominica\" = \"Dominique\";\n"
 "\"Dominican Republic\" = \"République dominicaine\";\n"
 "\"Download Certificate Request File\" = \"Télécharger le fichier Demande de certificat\";\n"
@@ -210,8 +212,6 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Errors\" = \"Erreurs\";\n"
 "\"Estonia\" = \"Estonie\";\n"
 "\"Eswatini\" = \"Eswatini\";\n"
-"\"Ethernet %d: %s\" = \"Ethernet %d: %s\";\n"
-"\"Ethernet: %s\" = \"Ethernet: %s\";\n"
 "\"Ethiopia\" = \"Éthiopie\";\n"
 "\"Falkland Islands (Malvinas)\" = \"Îles Falkland (Malvinas)\";\n"
 "\"Faroe Islands\" = \"Îles Féroé\";\n"
@@ -241,26 +241,39 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Guyana\" = \"Guyana\";\n"
 "\"Haiti\" = \"Haïti\";\n"
 "\"Heard Island and McDonald Islands\" = \"Heard Island et McDonald Islands\";\n"
+"\"Hold Job\" = \"Hold Job\";\n"
+"\"Hold New Jobs\" = \"Hold New Jobs\";\n"
+"\"Holding new jobs.\" = \"Holding new jobs.\";\n"
 "\"Holy See\" = \"Saint-Siège\";\n"
 "\"Honduras\" = \"Honduras\";\n"
 "\"Hong Kong\" = \"Hong Kong\";\n"
 "\"Hostname\" = \"Hostname\";\n"
 "\"Human-Readable Location\" = \"Emplacement lisible\";\n"
 "\"Hungary\" = \"Hongrie\";\n"
-"\"IPv4 Addresses\" = \"Adresses IPv4\";\n"
-"\"IPv6 Addresses\" = \"Adresses IPv6\";\n"
+"\"IPv4 Address\" = \"IPv4 Address\";\n"
+"\"IPv4 Configuration\" = \"IPv4 Configuration\";\n"
+"\"IPv4 Gateway\" = \"IPv4 Gateway\";\n"
+"\"IPv4 Netmask\" = \"IPv4 Netmask\";\n"
+"\"IPv6 Address\" = \"IPv6 Address\";\n"
+"\"IPv6 Configuration\" = \"IPv6 Configuration\";\n"
+"\"IPv6 Link-Local\" = \"IPv6 Link-Local\";\n"
+"\"IPv6 Prefix Length\" = \"IPv6 Prefix Length\";\n"
 "\"Iceland\" = \"Islande\";\n"
 "\"Identify Printer\" = \"Identifier l'imprimante\";\n"
 "\"India\" = \"Inde\";\n"
 "\"Indonesia\" = \"Indonésie\";\n"
 "\"Informational\" = \"Informational\";\n"
 "\"Install TLS Certificate\" = \"Installez TLS Certificat\";\n"
-"\"Invalid GET data.\" = \"Données de GET invalides.\";\n"
+"\"Invalid IPv4 address.\" = \"Invalid IPv4 address.\";\n"
+"\"Invalid IPv4 netmask.\" = \"Invalid IPv4 netmask.\";\n"
+"\"Invalid IPv6 address.\" = \"Invalid IPv6 address.\";\n"
+"\"Invalid IPv6 prefix length.\" = \"Invalid IPv6 prefix length.\";\n"
 "\"Invalid Job ID.\" = \"Invalid Job ID.\";\n"
 "\"Invalid certificate or private key.\" = \"Certificat invalide ou clé privée.\";\n"
 "\"Invalid form data.\" = \"Données de forme incorrectes.\";\n"
 "\"Invalid form submission.\" = \"Demande de formulaire non valide.\";\n"
 "\"Invalid job ID.\" = \"Invalide.\";\n"
+"\"Invalid network configuration.\" = \"Invalid network configuration.\";\n"
 "\"Iran (Islamic Republic of)\" = \"Iran (République islamique d ' )\";\n"
 "\"Iraq\" = \"Iraq\";\n"
 "\"Ireland\" = \"Irlande\";\n"
@@ -271,8 +284,10 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Japan\" = \"Japon\";\n"
 "\"Jersey\" = \"Jersey\";\n"
 "\"Job #\" = \"Job\";\n"
+"\"Job canceled.\" = \"Job canceled.\";\n"
+"\"Job held.\" = \"Job held.\";\n"
+"\"Job released.\" = \"Job released.\";\n"
 "\"Jobs\" = \"Jobs\";\n"
-"\"Joining Wi-Fi network.\" = \"Joignez-vous au réseau Wi-Fi.\";\n"
 "\"Jordan\" = \"Jordanie\";\n"
 "\"Kazakhstan\" = \"Kazakhstan\";\n"
 "\"Kenya\" = \"Kenya\";\n"
@@ -303,6 +318,7 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Maldives\" = \"Maldives\";\n"
 "\"Mali\" = \"Mali\";\n"
 "\"Malta\" = \"Malte\";\n"
+"\"Manual Configuration\" = \"Manual Configuration\";\n"
 "\"Marshall Islands\" = \"Îles Marshall\";\n"
 "\"Martinique\" = \"Martinique\";\n"
 "\"Mauritania\" = \"Mauritanie\";\n"
@@ -312,6 +328,7 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Mexico\" = \"Mexique\";\n"
 "\"Micronesia (Federated States of)\" = \"Micronésie (États fédérés de)\";\n"
 "\"Missing action.\" = \"Manque d'action.\";\n"
+"\"Missing job ID.\" = \"Missing job ID.\";\n"
 "\"Missing private key.\" = \"La clé privée manquante.\";\n"
 "\"Moldova, Republic of\" = \"République de Moldova\";\n"
 "\"Monaco\" = \"Monaco\";\n"
@@ -342,6 +359,7 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Northern Mariana Islands\" = \"Îles Mariannes du Nord\";\n"
 "\"Norway\" = \"Norvège\";\n"
 "\"Not set\" = \"Pas réglé\";\n"
+"\"Off\" = \"Off\";\n"
 "\"Offset\" = \"Offset\";\n"
 "\"Oman\" = \"Oman\";\n"
 "\"Options:\" = \"Options:\";\n"
@@ -370,6 +388,7 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Poland\" = \"Pologne\";\n"
 "\"Portrait\" = \"Portrait\";\n"
 "\"Portugal\" = \"Portugal\";\n"
+"\"Primary DNS\" = \"Primary DNS\";\n"
 "\"Print Group\" = \"Groupe d ' impression\";\n"
 "\"Print Test Page\" = \"Page d ' essai\";\n"
 "\"Print job options:\" = \"Imprimer les options d'emploi:\";\n"
@@ -385,7 +404,11 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Puerto Rico\" = \"Porto Rico\";\n"
 "\"Qatar\" = \"Qatar\";\n"
 "\"Queued at %s\" = \"Queued at %s\";\n"
+"\"Release Held New Jobs\" = \"Release Held New Jobs\";\n"
+"\"Release Job\" = \"Release Job\";\n"
+"\"Released held new jobs.\" = \"Released held new jobs.\";\n"
 "\"Reprint Job\" = \"Reprint Job\";\n"
+"\"Reprinted job.\" = \"Reprinted job.\";\n"
 "\"Rescan\" = \"Rescan\";\n"
 "\"Resume Printing\" = \"Récupérer l'impression\";\n"
 "\"Reverse Landscape\" = \"Paysage\";\n"
@@ -408,6 +431,7 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Sao Tome and Principe\" = \"Sao Tomé-et-Principe\";\n"
 "\"Saudi Arabia\" = \"Saudi Arabia\";\n"
 "\"Save Changes\" = \"Save Changes\";\n"
+"\"Secondary DNS\" = \"Secondary DNS\";\n"
 "\"Security\" = \"Sécurité\";\n"
 "\"Select Driver\" = \"Sélectionnez le pilote\";\n"
 "\"Senegal\" = \"Sénégal\";\n"
@@ -467,6 +491,7 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unable to identify printer.\" = \"Incapable d'identifier l'imprimante.\";\n"
 "\"Unable to join Wi-Fi network.\" = \"Impossible de rejoindre le réseau Wi-Fi.\";\n"
 "\"Unable to lookup address.\" = \"Incapable de regarder l'adresse.\";\n"
+"\"Unable to save network configuration.\" = \"Unable to save network configuration.\";\n"
 "\"Unable to use that driver.\" = \"Incapable d'utiliser ce conducteur.\";\n"
 "\"United Arab Emirates\" = \"Émirats arabes unis\";\n"
 "\"United Kingdom\" = \"Royaume-Uni\";\n"
@@ -492,10 +517,8 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Wallis and Futuna\" = \"Wallis et Futuna\";\n"
 "\"Warning\" = \"Avertissement\";\n"
 "\"Western Sahara\" = \"Sahara occidental\";\n"
-"\"Wi-Fi %d: %s\" = \"Wi-Fi %d: %s\";\n"
 "\"Wi-Fi Configuration\" = \"Configuration Wi-Fi\";\n"
 "\"Wi-Fi Network\" = \"Wi-Fi Network\";\n"
-"\"Wi-Fi: %s\" = \"Wi-Fi: %s\";\n"
 "\"Width\" = \"Largeur\";\n"
 "\"Wrong old password.\" = \"Un vieux mot de passe.\";\n"
 "\"Yemen\" = \"Yémen\";\n"
@@ -574,6 +597,7 @@ static const char *fr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"finishings.97\" = \"Repli parallèle\";\n"
 "\"finishings.98\" = \"Tableau pli\";\n"
 "\"finishings.99\" = \"Tour droit\";\n"
+"\"holding new jobs\" = \"holding new jobs\";\n"
 "\"job-state-reasons.aborted-by-system\" = \"Arrête.\";\n"
 "\"job-state-reasons.compression-error\" = \"Erreur de décompression.\";\n"
 "\"job-state-reasons.document-format-error\" = \"Erreur de format de document.\";\n"
