@@ -159,6 +159,7 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Chad\" = \"Ciad\";\n"
 "\"Change Access Password\" = \"Cambia password di accesso\";\n"
 "\"Change Log Level\" = \"Cambia il livello di registro\";\n"
+"\"Change Network Settings\" = \"Change Network Settings\";\n"
 "\"Change Wi-Fi Network\" = \"Modifica della rete Wi-Fi\";\n"
 "\"Changes Saved.\" = \"Cambiamenti salvati.\";\n"
 "\"Changes saved.\" = \"Cambiamenti salvati.\";\n"
@@ -172,9 +173,7 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Configuration\" = \"Configurazione\";\n"
 "\"Configure Media\" = \"Configurare i media\";\n"
 "\"Confirm Cancel All\" = \"Confermare Annulla Tutti\";\n"
-"\"Confirm Cancel Job\" = \"Confermare Annulla lavoro\";\n"
 "\"Confirm Delete Printer\" = \"Conferma Cancella stampante\";\n"
-"\"Confirm Reprint Job\" = \"Confermare il lavoro di ristampa\";\n"
 "\"Congo\" = \"Congo\";\n"
 "\"Congo, Democratic Republic of the\" = \"Congo, Repubblica Democratica del\";\n"
 "\"Contact\" = \"Contatto\";\n"
@@ -192,6 +191,8 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Cyprus\" = \"Cipro\";\n"
 "\"Czechia\" = \"Repubblica ceca\";\n"
 "\"Côte d'Ivoire\" = \"Cte d'Ivoire\";\n"
+"\"DHCP\" = \"DHCP\";\n"
+"\"DHCP w/Manual Address\" = \"DHCP w/Manual Address\";\n"
 "\"DNS-SD Service Name\" = \"Nome di servizio DNS-SD\";\n"
 "\"Debugging\" = \"Debug\";\n"
 "\"Default printer set.\" = \"Set stampante predefinito.\";\n"
@@ -199,6 +200,7 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Delete Printer\" = \"Eliminare la stampante\";\n"
 "\"Denmark\" = \"Danimarca\";\n"
 "\"Djibouti\" = \"Gibuti\";\n"
+"\"Domain Name\" = \"Domain Name\";\n"
 "\"Dominica\" = \"Dominica\";\n"
 "\"Dominican Republic\" = \"Repubblica Dominicana\";\n"
 "\"Download Certificate Request File\" = \"Scarica il file di richiesta del certificato\";\n"
@@ -210,8 +212,6 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Errors\" = \"Errori\";\n"
 "\"Estonia\" = \"Estonia\";\n"
 "\"Eswatini\" = \"Eswatini\";\n"
-"\"Ethernet %d: %s\" = \"Ethernet %d: %s\";\n"
-"\"Ethernet: %s\" = \"Ethernet: %s\";\n"
 "\"Ethiopia\" = \"Etiopia\";\n"
 "\"Falkland Islands (Malvinas)\" = \"Isole Falkland (Malvinas)\";\n"
 "\"Faroe Islands\" = \"Isole Faroe\";\n"
@@ -241,26 +241,39 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Guyana\" = \"Guyana\";\n"
 "\"Haiti\" = \"Haiti\";\n"
 "\"Heard Island and McDonald Islands\" = \"Heard Island e McDonald Islands\";\n"
+"\"Hold Job\" = \"Hold Job\";\n"
+"\"Hold New Jobs\" = \"Hold New Jobs\";\n"
+"\"Holding new jobs.\" = \"Holding new jobs.\";\n"
 "\"Holy See\" = \"Santa Sede\";\n"
 "\"Honduras\" = \"Honduras\";\n"
 "\"Hong Kong\" = \"Hong Kong\";\n"
 "\"Hostname\" = \"Nome host\";\n"
 "\"Human-Readable Location\" = \"Posizione leggibile umana\";\n"
 "\"Hungary\" = \"Ungheria\";\n"
-"\"IPv4 Addresses\" = \"Indirizzi IPv4\";\n"
-"\"IPv6 Addresses\" = \"Indirizzo IPv6\";\n"
+"\"IPv4 Address\" = \"IPv4 Address\";\n"
+"\"IPv4 Configuration\" = \"IPv4 Configuration\";\n"
+"\"IPv4 Gateway\" = \"IPv4 Gateway\";\n"
+"\"IPv4 Netmask\" = \"IPv4 Netmask\";\n"
+"\"IPv6 Address\" = \"IPv6 Address\";\n"
+"\"IPv6 Configuration\" = \"IPv6 Configuration\";\n"
+"\"IPv6 Link-Local\" = \"IPv6 Link-Local\";\n"
+"\"IPv6 Prefix Length\" = \"IPv6 Prefix Length\";\n"
 "\"Iceland\" = \"Islanda\";\n"
 "\"Identify Printer\" = \"Identificare la stampante\";\n"
 "\"India\" = \"India\";\n"
 "\"Indonesia\" = \"Indonesia\";\n"
 "\"Informational\" = \"Informazioni\";\n"
 "\"Install TLS Certificate\" = \"Installare TLS Certificato\";\n"
-"\"Invalid GET data.\" = \"Dati GET non validi.\";\n"
+"\"Invalid IPv4 address.\" = \"Invalid IPv4 address.\";\n"
+"\"Invalid IPv4 netmask.\" = \"Invalid IPv4 netmask.\";\n"
+"\"Invalid IPv6 address.\" = \"Invalid IPv6 address.\";\n"
+"\"Invalid IPv6 prefix length.\" = \"Invalid IPv6 prefix length.\";\n"
 "\"Invalid Job ID.\" = \"Invalid Job ID.\";\n"
 "\"Invalid certificate or private key.\" = \"Certificato non valido o chiave privata.\";\n"
 "\"Invalid form data.\" = \"Dati formali non validi.\";\n"
 "\"Invalid form submission.\" = \"Modulo non valido.\";\n"
 "\"Invalid job ID.\" = \"ID del lavoro non valido.\";\n"
+"\"Invalid network configuration.\" = \"Invalid network configuration.\";\n"
 "\"Iran (Islamic Republic of)\" = \"Iran (Repubblica Islamica di)\";\n"
 "\"Iraq\" = \"Iraq\";\n"
 "\"Ireland\" = \"Irlanda\";\n"
@@ -271,8 +284,10 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Japan\" = \"Giappone\";\n"
 "\"Jersey\" = \"Jersey\";\n"
 "\"Job #\" = \"Lavoro\";\n"
+"\"Job canceled.\" = \"Job canceled.\";\n"
+"\"Job held.\" = \"Job held.\";\n"
+"\"Job released.\" = \"Job released.\";\n"
 "\"Jobs\" = \"Lavoro\";\n"
-"\"Joining Wi-Fi network.\" = \"Unisciti alla rete Wi-Fi.\";\n"
 "\"Jordan\" = \"Giordania\";\n"
 "\"Kazakhstan\" = \"Kazakistan\";\n"
 "\"Kenya\" = \"Kenya\";\n"
@@ -303,6 +318,7 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Maldives\" = \"Maldive\";\n"
 "\"Mali\" = \"Mali\";\n"
 "\"Malta\" = \"Malta\";\n"
+"\"Manual Configuration\" = \"Manual Configuration\";\n"
 "\"Marshall Islands\" = \"Isole Marshall\";\n"
 "\"Martinique\" = \"Martinica\";\n"
 "\"Mauritania\" = \"Mauritania\";\n"
@@ -312,6 +328,7 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Mexico\" = \"Messico\";\n"
 "\"Micronesia (Federated States of)\" = \"Micronesia (Stati federali)\";\n"
 "\"Missing action.\" = \"Azione mancante.\";\n"
+"\"Missing job ID.\" = \"Missing job ID.\";\n"
 "\"Missing private key.\" = \"Manca la chiave privata.\";\n"
 "\"Moldova, Republic of\" = \"Moldavia, Repubblica\";\n"
 "\"Monaco\" = \"Monaco\";\n"
@@ -342,6 +359,7 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Northern Mariana Islands\" = \"Isole Marianne Settentrionali\";\n"
 "\"Norway\" = \"Norvegia\";\n"
 "\"Not set\" = \"Non impostato\";\n"
+"\"Off\" = \"Off\";\n"
 "\"Offset\" = \"Offset\";\n"
 "\"Oman\" = \"Oman\";\n"
 "\"Options:\" = \"Opzioni:\";\n"
@@ -370,6 +388,7 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Poland\" = \"Polonia\";\n"
 "\"Portrait\" = \"Ritratto\";\n"
 "\"Portugal\" = \"Portogallo\";\n"
+"\"Primary DNS\" = \"Primary DNS\";\n"
 "\"Print Group\" = \"Gruppo di stampa\";\n"
 "\"Print Test Page\" = \"Pagina di test di stampa\";\n"
 "\"Print job options:\" = \"Stampa opzioni di lavoro:\";\n"
@@ -385,7 +404,11 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Puerto Rico\" = \"Porto Rico\";\n"
 "\"Qatar\" = \"Qatar\";\n"
 "\"Queued at %s\" = \"Condividi su %s\";\n"
+"\"Release Held New Jobs\" = \"Release Held New Jobs\";\n"
+"\"Release Job\" = \"Release Job\";\n"
+"\"Released held new jobs.\" = \"Released held new jobs.\";\n"
 "\"Reprint Job\" = \"Ristampa lavoro\";\n"
+"\"Reprinted job.\" = \"Reprinted job.\";\n"
 "\"Rescan\" = \"Rescan\";\n"
 "\"Resume Printing\" = \"Riprendi la stampa\";\n"
 "\"Reverse Landscape\" = \"Paesaggio inverso\";\n"
@@ -408,6 +431,7 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Sao Tome and Principe\" = \"Sao Tome e Principe\";\n"
 "\"Saudi Arabia\" = \"Arabia Saudita\";\n"
 "\"Save Changes\" = \"Salvare le modifiche\";\n"
+"\"Secondary DNS\" = \"Secondary DNS\";\n"
 "\"Security\" = \"Sicurezza\";\n"
 "\"Select Driver\" = \"Selezionare il driver\";\n"
 "\"Senegal\" = \"Senegal\";\n"
@@ -467,6 +491,7 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unable to identify printer.\" = \"Incapace di identificare la stampante.\";\n"
 "\"Unable to join Wi-Fi network.\" = \"Non è possibile accedere alla rete Wi-Fi.\";\n"
 "\"Unable to lookup address.\" = \"Non riesco a cercare l'indirizzo.\";\n"
+"\"Unable to save network configuration.\" = \"Unable to save network configuration.\";\n"
 "\"Unable to use that driver.\" = \"Non riesco a usare quel conducente.\";\n"
 "\"United Arab Emirates\" = \"Emirati Arabi Uniti\";\n"
 "\"United Kingdom\" = \"Regno Unito\";\n"
@@ -492,10 +517,8 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Wallis and Futuna\" = \"Wallis e Futuna\";\n"
 "\"Warning\" = \"Avvertenza\";\n"
 "\"Western Sahara\" = \"Sahara occidentale\";\n"
-"\"Wi-Fi %d: %s\" = \"Wi-Fi %d: %s\";\n"
 "\"Wi-Fi Configuration\" = \"Configurazione Wi-Fi\";\n"
 "\"Wi-Fi Network\" = \"Rete Wi-Fi\";\n"
-"\"Wi-Fi: %s\" = \"Wi-Fi: %s\";\n"
 "\"Width\" = \"Larghezza\";\n"
 "\"Wrong old password.\" = \"Una vecchia password sbagliata.\";\n"
 "\"Yemen\" = \"Yemen\";\n"
@@ -574,6 +597,7 @@ static const char *it_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"finishings.97\" = \"Piega parallela\";\n"
 "\"finishings.98\" = \"Poster fold\";\n"
 "\"finishings.99\" = \"Cancello destro\";\n"
+"\"holding new jobs\" = \"holding new jobs\";\n"
 "\"job-state-reasons.aborted-by-system\" = \"Stoccata.\";\n"
 "\"job-state-reasons.compression-error\" = \"Errore di decompressione.\";\n"
 "\"job-state-reasons.document-format-error\" = \"Errore formato documento.\";\n"
