@@ -159,6 +159,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Chad\" = \"チャド\";\n"
 "\"Change Access Password\" = \"アクセスパスワードの変更\";\n"
 "\"Change Log Level\" = \"ログレベルを変更する\";\n"
+"\"Change Network Settings\" = \"Change Network Settings\";\n"
 "\"Change Wi-Fi Network\" = \"Wi-Fiネットワークの変更\";\n"
 "\"Changes Saved.\" = \"保存された変更。\";\n"
 "\"Changes saved.\" = \"保存された変更。\";\n"
@@ -172,9 +173,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Configuration\" = \"コンテンツ\";\n"
 "\"Configure Media\" = \"メディアの設定\";\n"
 "\"Confirm Cancel All\" = \"キャンセル確認 すべて\";\n"
-"\"Confirm Cancel Job\" = \"キャンセルジョブの確認\";\n"
 "\"Confirm Delete Printer\" = \"プリンターの削除を確認する\";\n"
-"\"Confirm Reprint Job\" = \"再印刷ジョブの確認\";\n"
 "\"Congo\" = \"コンゴ\";\n"
 "\"Congo, Democratic Republic of the\" = \"コンゴ、民主共和国\";\n"
 "\"Contact\" = \"お問い合わせ\";\n"
@@ -192,6 +191,8 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Cyprus\" = \"キプロス\";\n"
 "\"Czechia\" = \"チェコ語\";\n"
 "\"Côte d'Ivoire\" = \"Cte d'イボワール\";\n"
+"\"DHCP\" = \"DHCP\";\n"
+"\"DHCP w/Manual Address\" = \"DHCP w/Manual Address\";\n"
 "\"DNS-SD Service Name\" = \"DNS-SDサービス名\";\n"
 "\"Debugging\" = \"デバッギング\";\n"
 "\"Default printer set.\" = \"デフォルトプリンターセット。\";\n"
@@ -199,6 +200,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Delete Printer\" = \"プリンターの削除\";\n"
 "\"Denmark\" = \"デンマーク\";\n"
 "\"Djibouti\" = \"ジブチ\";\n"
+"\"Domain Name\" = \"Domain Name\";\n"
 "\"Dominica\" = \"ドミニカ\";\n"
 "\"Dominican Republic\" = \"ドミニカ共和国\";\n"
 "\"Download Certificate Request File\" = \"証明書の要求ファイルのダウンロード\";\n"
@@ -210,8 +212,6 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Errors\" = \"エラー\";\n"
 "\"Estonia\" = \"エストニア\";\n"
 "\"Eswatini\" = \"エスカティーニ\";\n"
-"\"Ethernet %d: %s\" = \"イーサネット %d : %s\";\n"
-"\"Ethernet: %s\" = \"イーサネット: %s\";\n"
 "\"Ethiopia\" = \"エチオピア\";\n"
 "\"Falkland Islands (Malvinas)\" = \"フォークランド諸島 (Malvinas)\";\n"
 "\"Faroe Islands\" = \"フェロー諸島\";\n"
@@ -241,26 +241,39 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Guyana\" = \"ガイアナ\";\n"
 "\"Haiti\" = \"ハイチ\";\n"
 "\"Heard Island and McDonald Islands\" = \"ヘアード島とマクドナルド諸島\";\n"
+"\"Hold Job\" = \"Hold Job\";\n"
+"\"Hold New Jobs\" = \"Hold New Jobs\";\n"
+"\"Holding new jobs.\" = \"Holding new jobs.\";\n"
 "\"Holy See\" = \"聖なる参照\";\n"
 "\"Honduras\" = \"ホンジュラス\";\n"
 "\"Hong Kong\" = \"香港\";\n"
 "\"Hostname\" = \"ホスト名\";\n"
 "\"Human-Readable Location\" = \"人間可読の場所\";\n"
 "\"Hungary\" = \"ハンガリー\";\n"
-"\"IPv4 Addresses\" = \"IPv4 アドレス\";\n"
-"\"IPv6 Addresses\" = \"IPv6 アドレス\";\n"
+"\"IPv4 Address\" = \"IPv4 Address\";\n"
+"\"IPv4 Configuration\" = \"IPv4 Configuration\";\n"
+"\"IPv4 Gateway\" = \"IPv4 Gateway\";\n"
+"\"IPv4 Netmask\" = \"IPv4 Netmask\";\n"
+"\"IPv6 Address\" = \"IPv6 Address\";\n"
+"\"IPv6 Configuration\" = \"IPv6 Configuration\";\n"
+"\"IPv6 Link-Local\" = \"IPv6 Link-Local\";\n"
+"\"IPv6 Prefix Length\" = \"IPv6 Prefix Length\";\n"
 "\"Iceland\" = \"アイスランド\";\n"
 "\"Identify Printer\" = \"プリンターの識別\";\n"
 "\"India\" = \"インド\";\n"
 "\"Indonesia\" = \"インドネシア\";\n"
 "\"Informational\" = \"インフォメーション\";\n"
 "\"Install TLS Certificate\" = \"TLSをインストールする 証明書\";\n"
-"\"Invalid GET data.\" = \"無効なGETデータ。\";\n"
+"\"Invalid IPv4 address.\" = \"Invalid IPv4 address.\";\n"
+"\"Invalid IPv4 netmask.\" = \"Invalid IPv4 netmask.\";\n"
+"\"Invalid IPv6 address.\" = \"Invalid IPv6 address.\";\n"
+"\"Invalid IPv6 prefix length.\" = \"Invalid IPv6 prefix length.\";\n"
 "\"Invalid Job ID.\" = \"無効なジョブID。\";\n"
 "\"Invalid certificate or private key.\" = \"無効な証明書か秘密鍵。\";\n"
 "\"Invalid form data.\" = \"無効なフォームデータ。\";\n"
 "\"Invalid form submission.\" = \"無効なフォーム送信。\";\n"
 "\"Invalid job ID.\" = \"無効なジョブID。\";\n"
+"\"Invalid network configuration.\" = \"Invalid network configuration.\";\n"
 "\"Iran (Islamic Republic of)\" = \"イラン(イスラム共和国)\";\n"
 "\"Iraq\" = \"イラク\";\n"
 "\"Ireland\" = \"アイルランド\";\n"
@@ -271,8 +284,10 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Japan\" = \"ジャパンジャパン\";\n"
 "\"Jersey\" = \"ジャージー\";\n"
 "\"Job #\" = \"求人番号\";\n"
+"\"Job canceled.\" = \"Job canceled.\";\n"
+"\"Job held.\" = \"Job held.\";\n"
+"\"Job released.\" = \"Job released.\";\n"
 "\"Jobs\" = \"求人情報\";\n"
-"\"Joining Wi-Fi network.\" = \"Wi-Fiネットワークに参加\";\n"
 "\"Jordan\" = \"ヨルダン\";\n"
 "\"Kazakhstan\" = \"カザフスタン\";\n"
 "\"Kenya\" = \"ケニア\";\n"
@@ -303,6 +318,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Maldives\" = \"モルディブ\";\n"
 "\"Mali\" = \"マリ\";\n"
 "\"Malta\" = \"マルタ\";\n"
+"\"Manual Configuration\" = \"Manual Configuration\";\n"
 "\"Marshall Islands\" = \"マーシャル諸島\";\n"
 "\"Martinique\" = \"マルティニーク\";\n"
 "\"Mauritania\" = \"モーリタニア\";\n"
@@ -312,6 +328,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Mexico\" = \"メキシコ\";\n"
 "\"Micronesia (Federated States of)\" = \"マイクロネシア(連邦)\";\n"
 "\"Missing action.\" = \"行動を欠く。\";\n"
+"\"Missing job ID.\" = \"Missing job ID.\";\n"
 "\"Missing private key.\" = \"プライベートキーを欠く。\";\n"
 "\"Moldova, Republic of\" = \"モルドバ共和国\";\n"
 "\"Monaco\" = \"メニュー\";\n"
@@ -342,6 +359,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Northern Mariana Islands\" = \"北マリアナ諸島\";\n"
 "\"Norway\" = \"ノルウェー\";\n"
 "\"Not set\" = \"セットなし\";\n"
+"\"Off\" = \"Off\";\n"
 "\"Offset\" = \"オフセット\";\n"
 "\"Oman\" = \"オマーン\";\n"
 "\"Options:\" = \"オプション:\";\n"
@@ -370,6 +388,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Poland\" = \"ポーランド\";\n"
 "\"Portrait\" = \"ポートレート\";\n"
 "\"Portugal\" = \"ポルトガル\";\n"
+"\"Primary DNS\" = \"Primary DNS\";\n"
 "\"Print Group\" = \"プリントグループ\";\n"
 "\"Print Test Page\" = \"プリントテストページ\";\n"
 "\"Print job options:\" = \"ジョブオプションを印刷:\";\n"
@@ -385,7 +404,11 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Puerto Rico\" = \"プエルトリコ\";\n"
 "\"Qatar\" = \"カタール\";\n"
 "\"Queued at %s\" = \"%sでキューイング\";\n"
+"\"Release Held New Jobs\" = \"Release Held New Jobs\";\n"
+"\"Release Job\" = \"Release Job\";\n"
+"\"Released held new jobs.\" = \"Released held new jobs.\";\n"
 "\"Reprint Job\" = \"再印刷ジョブ\";\n"
+"\"Reprinted job.\" = \"Reprinted job.\";\n"
 "\"Rescan\" = \"リスカ\";\n"
 "\"Resume Printing\" = \"再開の印刷\";\n"
 "\"Reverse Landscape\" = \"逆の風景\";\n"
@@ -408,6 +431,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Sao Tome and Principe\" = \"トームとプリンシペ\";\n"
 "\"Saudi Arabia\" = \"サウジアラビア\";\n"
 "\"Save Changes\" = \"変更を保存\";\n"
+"\"Secondary DNS\" = \"Secondary DNS\";\n"
 "\"Security\" = \"セキュリティ\";\n"
 "\"Select Driver\" = \"ドライバーの選択\";\n"
 "\"Senegal\" = \"セネガル\";\n"
@@ -467,6 +491,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unable to identify printer.\" = \"プリンターを識別できません。\";\n"
 "\"Unable to join Wi-Fi network.\" = \"Wi-Fiネットワークへの参加はできません。\";\n"
 "\"Unable to lookup address.\" = \"アドレスを調べることができません。\";\n"
+"\"Unable to save network configuration.\" = \"Unable to save network configuration.\";\n"
 "\"Unable to use that driver.\" = \"そのドライバを使用できません。\";\n"
 "\"United Arab Emirates\" = \"アラブ首長国連邦\";\n"
 "\"United Kingdom\" = \"イギリス\";\n"
@@ -492,10 +517,8 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Wallis and Futuna\" = \"ウォリスとフツナ\";\n"
 "\"Warning\" = \"インフォメーション\";\n"
 "\"Western Sahara\" = \"西サハラ\";\n"
-"\"Wi-Fi %d: %s\" = \"無線LAN %d : %s\";\n"
 "\"Wi-Fi Configuration\" = \"Wi-Fiの設定\";\n"
 "\"Wi-Fi Network\" = \"Wi-Fiネットワーク\";\n"
-"\"Wi-Fi: %s\" = \"無線LAN: %s\";\n"
 "\"Width\" = \"アクセス\";\n"
 "\"Wrong old password.\" = \"古いパスワードが間違っています。\";\n"
 "\"Yemen\" = \"イエメン\";\n"
@@ -574,6 +597,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"finishings.97\" = \"平行折目\";\n"
 "\"finishings.98\" = \"ポスター折り目\";\n"
 "\"finishings.99\" = \"右ゲートの折目\";\n"
+"\"holding new jobs\" = \"holding new jobs\";\n"
 "\"job-state-reasons.aborted-by-system\" = \"停止。\";\n"
 "\"job-state-reasons.compression-error\" = \"圧縮エラー。\";\n"
 "\"job-state-reasons.document-format-error\" = \"ドキュメントフォーマットエラー。\";\n"
