@@ -126,9 +126,9 @@ _papplSubscriptionIPPCreate(
     username = "anonymous";
 
   // Skip past the initial attributes to the first subscription group.
-  attr = ippFirstAttribute(client->request);
+  attr = ippGetFirstAttribute(client->request);
   while (attr && ippGetGroupTag(attr) != IPP_TAG_SUBSCRIPTION)
-    attr = ippNextAttribute(client->request);
+    attr = ippGetNextAttribute(client->request);
 
   while (attr)
   {
@@ -230,7 +230,7 @@ _papplSubscriptionIPPCreate(
 	}
       }
 
-      attr = ippNextAttribute(client->request);
+      attr = ippGetNextAttribute(client->request);
     }
 
     if (!pull_method || events == PAPPL_EVENT_NONE)
@@ -306,9 +306,9 @@ _papplSubscriptionIPPGetAttributes(
 
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
 
-  pthread_rwlock_rdlock(&sub->rwlock);
+  _papplRWLockRead(sub);
   _papplCopyAttributes(client->response, sub->attrs, ra, IPP_TAG_SUBSCRIPTION, 0);
-  pthread_rwlock_unlock(&sub->rwlock);
+  _papplRWUnlock(sub);
 
   cupsArrayDelete(ra);
 }
@@ -379,7 +379,7 @@ _papplSubscriptionIPPGetNotifications(
 	break;
       }
 
-      pthread_rwlock_rdlock(&sub->rwlock);
+      _papplRWLockRead(sub);
 
       seq_num = ippGetInteger(seq_nums, i);
       if (seq_num < sub->first_sequence)
@@ -388,7 +388,7 @@ _papplSubscriptionIPPGetNotifications(
       if (seq_num > sub->last_sequence)
       {
         // No more events...
-        pthread_rwlock_unlock(&sub->rwlock);
+        _papplRWUnlock(sub);
 	continue;
       }
 
@@ -415,7 +415,7 @@ _papplSubscriptionIPPGetNotifications(
 	num_events ++;
       }
 
-      pthread_rwlock_unlock(&sub->rwlock);
+      _papplRWUnlock(sub);
     }
 
     if (i < count || !notify_wait)
@@ -496,7 +496,7 @@ _papplSubscriptionIPPList(
     username = "anonymous";
 
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
-  pthread_rwlock_rdlock(&client->system->rwlock);
+  _papplRWLockRead(client->system);
 
   for (sub = (pappl_subscription_t *)cupsArrayGetFirst(client->system->subscriptions); sub; sub = (pappl_subscription_t *)cupsArrayGetNext(client->system->subscriptions))
   {
@@ -509,15 +509,15 @@ _papplSubscriptionIPPList(
     if (count > 0)
       ippAddSeparator(client->response);
 
-    pthread_rwlock_rdlock(&sub->rwlock);
+    _papplRWLockRead(sub);
     _papplCopyAttributes(client->response, sub->attrs, ra, IPP_TAG_SUBSCRIPTION, 0);
-    pthread_rwlock_unlock(&sub->rwlock);
+    _papplRWUnlock(sub);
 
     count ++;
     if (limit > 0 && count >= limit)
       break;
   }
-  pthread_rwlock_unlock(&client->system->rwlock);
+  _papplRWUnlock(client->system);
 
   cupsArrayDelete(ra);
 }
