@@ -33,6 +33,7 @@
 
 static bool		add_listeners(pappl_system_t *system, const char *name, int port, int family);
 static int		compare_filters(_pappl_mime_filter_t *a, _pappl_mime_filter_t *b);
+static int		compare_timers(_pappl_timer_t *a, _pappl_timer_t *b);
 static _pappl_mime_filter_t *copy_filter(_pappl_mime_filter_t *f);
 
 
@@ -213,6 +214,64 @@ papplSystemAddMIMEFilter(
 }
 
 
+//
+// 'papplSystemAddTimerCallback()' - Add a timer callback to a system.
+//
+// This function schedules a function that will be called on the main run loop
+// thread at the specified time and optionally every "interval" seconds
+// thereafter.  The timimg accuracy is typically within a few milliseconds but
+// is not guaranteed.  Since the callback is run on the main run loop thread,
+// functions should create a new thread for any long-running operations.
+//
+// The callback function receives the "system" and "cb_data" pointers and
+// returns `true` to repeat the timer or `false` to remove it:
+//
+// ```
+// bool my_timer_cb(pappl_system_t *system, void *cb_data)
+// {
+//   ... do periodic task ...
+//   return (true); // repeat the timer
+// }
+// ```
+//
+
+bool					// O - `true` on success, `false` on error
+papplSystemAddTimerCallback(
+    pappl_system_t   *system,		// I - System
+    time_t           start,		// I - Start time in seconds or `0` for the current time
+    int              interval,		// I - Repeat interval in seconds or `0` for no repeat
+    pappl_timer_cb_t cb,		// I - Callback function
+    void             *cb_data)		// I - Callback data
+{
+  _pappl_timer_t	*newt;		// New timer
+
+
+  // Range check input...
+  if (!system || !cb || interval < 0)
+    return (false);
+
+  // Allocate the new timer...
+  if ((newt = calloc(1, sizeof(_pappl_timer_t))) == NULL)
+    return (false);
+
+  _papplRWLockWrite(system);
+
+  if (!system->timers)
+    system->timers = cupsArrayNew((cups_array_cb_t)compare_timers, NULL, NULL, 0, NULL, NULL);
+
+  newt->cb       = cb;
+  newt->cb_data  = cb_data;
+  newt->next     = start ? start : time(NULL) + interval;
+  newt->interval = interval;
+
+  cupsArrayAdd(system->timers, newt);
+
+  _papplRWUnlock(system);
+
+  return (true);
+}
+
+
 //
 // '_papplSystemExportVersions()' - Export the firmware versions to IPP attributes...
 //
@@ -361,14 +420,14 @@ _papplSystemFindMIMEFilter(
   if (!system || !srctype || !dsttype)
     return (NULL);
 
-  pthread_rwlock_rdlock(&system->rwlock);
+  _papplRWLockRead(system);
 
   key.src = srctype;
   key.dst = dsttype;
 
   match = (_pappl_mime_filter_t *)cupsArrayFind(system->filters, &key);
 
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 
   return (match);
 }
@@ -392,7 +451,7 @@ papplSystemGetAdminGroup(
 
   if (system && buffer && bufsize > 0)
   {
-    pthread_rwlock_rdlock(&system->rwlock);
+    _papplRWLockRead(system);
 
     if (system->admin_group)
     {
@@ -402,7 +461,7 @@ papplSystemGetAdminGroup(
     else
       *buffer = '\0';
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
   else if (buffer)
     *buffer = '\0';
@@ -446,11 +505,11 @@ papplSystemGetContact(
     return (contact);
   }
 
-  pthread_rwlock_rdlock(&system->rwlock);
+  _papplRWLockRead(system);
 
   *contact = system->contact;
 
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 
   return (contact);
 }
@@ -489,7 +548,7 @@ papplSystemGetDefaultPrintGroup(
 
   if (system && buffer && bufsize > 0)
   {
-    pthread_rwlock_rdlock(&system->rwlock);
+    _papplRWLockRead(system);
 
     if (system->default_print_group)
     {
@@ -499,7 +558,7 @@ papplSystemGetDefaultPrintGroup(
     else
       *buffer = '\0';
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
   else if (buffer)
     *buffer = '\0';
@@ -526,7 +585,7 @@ papplSystemGetDNSSDName(
 
   if (system && buffer && bufsize > 0)
   {
-    pthread_rwlock_rdlock(&system->rwlock);
+    _papplRWLockRead(system);
 
     if (system->dns_sd_name)
     {
@@ -536,7 +595,7 @@ papplSystemGetDNSSDName(
     else
       *buffer = '\0';
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
   else if (buffer)
     *buffer = '\0';
@@ -578,7 +637,7 @@ papplSystemGetGeoLocation(
 
   if (system && buffer && bufsize > 0)
   {
-    pthread_rwlock_rdlock(&system->rwlock);
+    _papplRWLockRead(system);
 
     if (system->geo_location)
     {
@@ -588,7 +647,7 @@ papplSystemGetGeoLocation(
     else
       *buffer = '\0';
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
   else if (buffer)
     *buffer = '\0';
@@ -633,7 +692,7 @@ papplSystemGetHostName(
 
   if (system && buffer && bufsize > 0)
   {
-    pthread_rwlock_rdlock(&system->rwlock);
+    _papplRWLockRead(system);
 
     if (system->hostname)
     {
@@ -643,7 +702,7 @@ papplSystemGetHostName(
     else
       *buffer = '\0';
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
   else if (buffer)
     *buffer = '\0';
@@ -686,7 +745,7 @@ papplSystemGetLocation(
 
   if (system && buffer && bufsize > 0)
   {
-    pthread_rwlock_rdlock(&system->rwlock);
+    _papplRWLockRead(system);
 
     if (system->location)
     {
@@ -696,7 +755,7 @@ papplSystemGetLocation(
     else
       *buffer = '\0';
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
   else if (buffer)
     *buffer = '\0';
@@ -734,6 +793,50 @@ papplSystemGetMaxClients(
 }
 
 
+//
+// 'papplSystemGetMaxImageSize()' - Get the maximum supported size for images.
+//
+// This function retrieves the image size limits in bytes (uncompressed),
+// columns, and lines.
+//
+
+size_t					// O - Maximum image size (uncompressed)
+papplSystemGetMaxImageSize(
+    pappl_system_t *system,		// I - System
+    int            *max_width,		// O - Maximum width in columns
+    int            *max_height)		// O - Maximum height in lines
+{
+  size_t	max_size;		// Maximum image file size
+
+
+  // Range check input...
+  if (!system)
+  {
+    if (max_width)
+      *max_width = 0;
+    if (max_height)
+      *max_height = 0;
+
+    return (0);
+  }
+
+  // Grab a snapshot of the limits...
+  _papplRWLockRead(system);
+
+  max_size = system->max_image_size;
+
+  if (max_width)
+    *max_width = system->max_image_width;
+
+  if (max_height)
+    *max_height = system->max_image_height;
+
+  _papplRWUnlock(system);
+
+  return (max_size);
+}
+
+
 //
 // 'papplSystemGetMaxLogSize()' - Get the maximum log file size.
 //
@@ -787,7 +890,7 @@ papplSystemGetName(
 
   if (system && buffer && bufsize > 0)
   {
-    pthread_rwlock_rdlock(&system->rwlock);
+    _papplRWLockRead(system);
 
     if (system->name)
     {
@@ -797,7 +900,7 @@ papplSystemGetName(
     else
       *buffer = '\0';
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
   else if (buffer)
     *buffer = '\0';
@@ -853,7 +956,7 @@ papplSystemGetOrganization(
 
   if (system && buffer && bufsize > 0)
   {
-    pthread_rwlock_rdlock(&system->rwlock);
+    _papplRWLockRead(system);
 
     if (system->organization)
     {
@@ -863,7 +966,7 @@ papplSystemGetOrganization(
     else
       *buffer = '\0';
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
   else if (buffer)
     *buffer = '\0';
@@ -890,7 +993,7 @@ papplSystemGetOrganizationalUnit(
 
   if (system && buffer && bufsize > 0)
   {
-    pthread_rwlock_rdlock(&system->rwlock);
+    _papplRWLockRead(system);
 
     if (system->org_unit)
     {
@@ -900,7 +1003,7 @@ papplSystemGetOrganizationalUnit(
     else
       *buffer = '\0';
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
   else if (buffer)
     *buffer = '\0';
@@ -927,11 +1030,11 @@ papplSystemGetPassword(
 {
   if (system && buffer && bufsize > 0)
   {
-    pthread_rwlock_rdlock(&system->rwlock);
+    _papplRWLockRead(system);
 
     papplCopyString(buffer, system->password_hash, bufsize);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
   else if (buffer)
     *buffer = '\0';
@@ -1066,14 +1169,14 @@ papplSystemGetVersions(
 
   if (system && versions && system->num_versions > 0)
   {
-    pthread_rwlock_rdlock(&system->rwlock);
+    _papplRWLockRead(system);
 
     if (max_versions > (int)system->num_versions)
-      memcpy(versions, system->versions, system->num_versions * sizeof(pappl_version_t));
+      memcpy(versions, system->versions, (size_t)system->num_versions * sizeof(pappl_version_t));
     else
       memcpy(versions, system->versions, (size_t)max_versions * sizeof(pappl_version_t));
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 
   return (system ? (int)system->num_versions : 0);
@@ -1143,7 +1246,17 @@ bool					// O - `true` if the system is running, `false` otherwise
 papplSystemIsRunning(
     pappl_system_t *system)		// I - System
 {
-  return (system ? system->is_running : false);
+  bool	is_running;			// Return value
+
+  // Range check input
+  if (!system)
+    return (false);
+
+  _papplRWLockRead(system);
+  is_running = system->is_running;
+  _papplRWUnlock(system);
+
+  return (is_running);
 }
 
 
@@ -1175,7 +1288,7 @@ papplSystemIteratePrinters(
     pappl_printer_cb_t cb,		// I - Callback function
     void               *data)		// I - Callback data
 {
-  size_t		i,		// Looping var
+  cups_len_t		i,		// Looping var
 			count;		// Number of printers
 
 
@@ -1187,10 +1300,10 @@ papplSystemIteratePrinters(
   // Note: Cannot use cupsArrayGetFirst/Last since other threads might be
   // enumerating the printers array.
 
-  pthread_rwlock_rdlock(&system->rwlock);
+  _papplRWLockRead(system);
   for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
     (cb)((pappl_printer_t *)cupsArrayGetElement(system->printers, i), data);
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 }
 
 
@@ -1203,7 +1316,7 @@ papplSystemMatchDriver(
     pappl_system_t *system,		// I - System
     const char     *device_id)		// I - IEEE-1284 device ID string
 {
-  size_t	i;			// Looping var
+  cups_len_t	i;			// Looping var
   pappl_pr_driver_t *driver;		// Current driver
   const char	*drvstart,		// Start of key/value pair
 		*drvend,		// End of key/value pair
@@ -1275,6 +1388,57 @@ papplSystemMatchDriver(
 }
 
 
+//
+// '_papplSystemNeedClean()' - Mark the system needing cleaning.
+//
+
+void
+_papplSystemNeedClean(
+    pappl_system_t *system)		// I - System
+{
+  _papplRWLockWrite(system);
+  if (!system->clean_time)
+    system->clean_time = time(NULL) + 60;
+  _papplRWUnlock(system);
+}
+
+
+//
+// 'papplSystemRemoveTimerCallback()' - Remove a timer callback.
+//
+// This function removes all matching timer callbacks from the specified system.
+// Both the callback function and data must match to remove a timer.
+//
+
+void
+papplSystemRemoveTimerCallback(
+    pappl_system_t   *system,		// I - System
+    pappl_timer_cb_t cb,		// I - Callback function
+    void             *cb_data)		// I - Callback data
+{
+  _pappl_timer_t	*t;		// Current timer
+
+
+  // Range check input...
+  if (!system || !cb)
+    return;
+
+  // Loop through the timers and remove any matches...
+  _papplRWLockWrite(system);
+
+  for (t = (_pappl_timer_t *)cupsArrayGetFirst(system->timers); t; t = (_pappl_timer_t *)cupsArrayGetNext(system->timers))
+  {
+    if (t->cb == cb && t->cb_data == cb_data)
+    {
+      cupsArrayRemove(system->timers, t);
+      free(t);
+    }
+  }
+
+  _papplRWUnlock(system);
+}
+
+
 //
 // 'papplSystemSetAdminGroup()' - Set the administrative group.
 //
@@ -1292,7 +1456,7 @@ papplSystemSetAdminGroup(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     free(system->admin_group);
     system->admin_group = value ? strdup(value) : NULL;
@@ -1315,7 +1479,7 @@ papplSystemSetAdminGroup(
 
     _papplSystemConfigChanged(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1343,14 +1507,14 @@ papplSystemSetAuthCallback(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     free(system->auth_scheme);
     system->auth_scheme = auth_scheme ? strdup(auth_scheme) : NULL;
     system->auth_cb     = auth_cb;
     system->auth_cbdata = auth_cbdata;
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1368,13 +1532,13 @@ papplSystemSetContact(
   if (!system || !contact)
     return;
 
-  pthread_rwlock_wrlock(&system->rwlock);
+  _papplRWLockWrite(system);
 
   system->contact = *contact;
 
   _papplSystemConfigChanged(system);
 
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 }
 
 
@@ -1392,13 +1556,13 @@ papplSystemSetDefaultPrinterID(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     system->default_printer_id = default_printer_id;
 
     _papplSystemConfigChanged(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1419,14 +1583,14 @@ papplSystemSetDefaultPrintGroup(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     free(system->default_print_group);
     system->default_print_group = value ? strdup(value) : NULL;
 
     _papplSystemConfigChanged(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1445,7 +1609,7 @@ papplSystemSetDNSSDName(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     free(system->dns_sd_name);
     system->dns_sd_name      = value ? strdup(value) : NULL;
@@ -1459,7 +1623,7 @@ papplSystemSetDNSSDName(
     else
       _papplSystemRegisterDNSSDNoLock(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1479,12 +1643,12 @@ papplSystemSetEventCallback(
 {
   if (system && event_cb)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     system->event_cb   = event_cb;
     system->event_data = event_data;
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1505,12 +1669,12 @@ papplSystemSetFooterHTML(
 {
   if (system && html && !system->is_running)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     free(system->footer_html);
     system->footer_html = strdup(html);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1536,7 +1700,7 @@ papplSystemSetGeoLocation(
 
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     free(system->geo_location);
     system->geo_location = value ? strdup(value) : NULL;
@@ -1545,7 +1709,7 @@ papplSystemSetGeoLocation(
 
     _papplSystemRegisterDNSSDNoLock(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1568,6 +1732,90 @@ papplSystemSetHostname(
 }
 
 
+//
+// '_papplSystemSetHostNameNoLock()' - Set the system hostname without locking.
+//
+// This function sets the system hostname.  If `NULL`, the default hostname
+// is used.
+//
+
+void
+_papplSystemSetHostNameNoLock(
+    pappl_system_t *system,		// I - System
+    const char     *value)		// I - Hostname or `NULL` for default
+{
+  char	temp[1024],			// Temporary hostname string
+	*ptr;				// Pointer in temporary hostname
+
+
+  if (value)
+  {
+#if !defined(__APPLE__) && !_WIN32
+    cups_file_t	*fp;			// Hostname file
+
+    if ((fp = cupsFileOpen("/etc/hostname", "w")) != NULL)
+    {
+      cupsFilePrintf(fp, "%s\n", value);
+      cupsFileClose(fp);
+    }
+#endif // !__APPLE__ && !_WIN32
+
+#ifdef HAVE_AVAHI
+    _pappl_dns_sd_t	master = _papplDNSSDInit(system);
+					// DNS-SD master reference
+
+    if (master)
+      avahi_client_set_host_name(master, value);
+#endif // HAVE_AVAHI
+
+#if !_WIN32
+    sethostname(value, (int)strlen(value));
+#endif // !_WIN32
+  }
+  else
+  {
+#ifdef HAVE_AVAHI
+    _pappl_dns_sd_t	master = _papplDNSSDInit(system);
+					// DNS-SD master reference
+    const char *avahi_name = master ? avahi_client_get_host_name_fqdn(master) : NULL;
+					// mDNS hostname
+
+    if (avahi_name)
+      papplCopyString(temp, avahi_name, sizeof(temp));
+    else
+#endif /* HAVE_AVAHI */
+    httpGetHostname(NULL, temp, sizeof(temp));
+
+    if ((ptr = strstr(temp, ".lan")) != NULL && !ptr[4])
+    {
+      // Replace hostname.lan with hostname.local
+      papplCopyString(ptr, ".local", sizeof(temp) - (size_t)(ptr - temp));
+    }
+    else if (!strrchr(temp, '.'))
+    {
+      // No domain information, so append .local to hostname...
+      ptr = temp + strlen(temp);
+      papplCopyString(ptr, ".local", sizeof(temp) - (size_t)(ptr - temp));
+    }
+
+    value = temp;
+  }
+
+  if (system->hostname && strcasecmp(system->hostname, value) && system->is_running)
+  {
+    // Force an update of all DNS-SD registrations...
+    system->dns_sd_host_changes = -1;
+  }
+
+  // Save the new hostname value
+  free(system->hostname);
+  system->hostname = strdup(value);
+
+  // Set the system TLS credentials...
+  cupsSetServerCredentials(NULL, system->hostname, 1);
+}
+
+
 //
 // 'papplSystemSetHostName()' - Set the system hostname.
 //
@@ -1582,78 +1830,9 @@ papplSystemSetHostName(
 {
   if (system)
   {
-    char	temp[1024],		// Temporary hostname string
-		*ptr;			// Pointer in temporary hostname
-
-    pthread_rwlock_wrlock(&system->rwlock);
-
-    if (value)
-    {
-#if !defined(__APPLE__) && !_WIN32
-      cups_file_t	*fp;		// Hostname file
-
-      if ((fp = cupsFileOpen("/etc/hostname", "w")) != NULL)
-      {
-        cupsFilePrintf(fp, "%s\n", value);
-        cupsFileClose(fp);
-      }
-#endif // !__APPLE__ && !_WIN32
-
-#ifdef HAVE_AVAHI
-      _pappl_dns_sd_t	master = _papplDNSSDInit(system);
-					  // DNS-SD master reference
-
-      if (master)
-        avahi_client_set_host_name(master, value);
-#endif // HAVE_AVAHI
-
-#if !_WIN32
-      sethostname(value, (int)strlen(value));
-#endif // !_WIN32
-    }
-    else
-    {
-#ifdef HAVE_AVAHI
-      _pappl_dns_sd_t	master = _papplDNSSDInit(system);
-					  // DNS-SD master reference
-      const char *avahi_name = master ? avahi_client_get_host_name_fqdn(master) : NULL;
-					  // mDNS hostname
-
-      if (avahi_name)
-	papplCopyString(temp, avahi_name, sizeof(temp));
-      else
-#endif /* HAVE_AVAHI */
-      httpGetHostname(NULL, temp, sizeof(temp));
-
-      if ((ptr = strstr(temp, ".lan")) != NULL && !ptr[4])
-      {
-        // Replace hostname.lan with hostname.local
-        papplCopyString(ptr, ".local", sizeof(temp) - (size_t)(ptr - temp));
-      }
-      else if (!strrchr(temp, '.'))
-      {
-        // No domain information, so append .local to hostname...
-        ptr = temp + strlen(temp);
-        papplCopyString(ptr, ".local", sizeof(temp) - (size_t)(ptr - temp));
-      }
-
-      value = temp;
-    }
-
-    if (system->hostname && strcasecmp(system->hostname, value) && system->is_running)
-    {
-      // Force an update of all DNS-SD registrations...
-      system->dns_sd_host_changes = -1;
-    }
-
-    // Save the new hostname value
-    free(system->hostname);
-    system->hostname = strdup(value);
-
-    // Set the system TLS credentials...
-    cupsSetServerCredentials(NULL, system->hostname, 1);
-
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWLockWrite(system);
+    _papplSystemSetHostNameNoLock(system, value);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1672,7 +1851,7 @@ papplSystemSetLocation(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     free(system->location);
     system->location    = value ? strdup(value) : NULL;
@@ -1681,10 +1860,11 @@ papplSystemSetLocation(
 
     _papplSystemRegisterDNSSDNoLock(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
+
 //
 // 'papplSystemSetLogLevel()' - Set the system log level
 //
@@ -1698,13 +1878,13 @@ papplSystemSetLogLevel(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     system->loglevel = loglevel;
 
     _papplSystemConfigChanged(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1782,13 +1962,74 @@ papplSystemSetMaxClients(
     max_clients = 32768;
 
   // Set the new value...
-  pthread_rwlock_wrlock(&system->rwlock);
+  _papplRWLockWrite(system);
 
   system->max_clients = max_clients;
 
   _papplSystemConfigChanged(system);
 
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
+}
+
+
+//
+// 'papplSystemSetMaxImageSize()' - Set the maximum allowed JPEG/PNG image sizes.
+//
+// This function sets the maximum size allowed for JPEG and PNG images.  The
+// default limits are 16384x16384 and 1/10th the maximum memory the current
+// process can use or 1GiB, whichever is less.
+//
+
+void
+papplSystemSetMaxImageSize(
+    pappl_system_t *system,		// I - System
+    size_t         max_size,		// I - Maximum image size (uncompressed) or `0` for default
+    int            max_width,		// I - Maximum image width in columns or `0` for default
+    int            max_height)		// I - Maximum image height in lines or `0` for default
+{
+  // Range check input...
+  if (!system || max_width < 0 || max_height < 0)
+    return;
+
+  if (max_size == 0)
+  {
+    // By default, limit images to 1/10th available memory...
+#if _WIN32
+    MEMORYSTATUSEX	statex;		// Memory status
+
+    if (GlobalMemoryStatusEx(&statex))
+      max_size = (size_t)statex.ullTotalPhys / 10;
+    else
+      max_size = 16 * 1024 * 1024;
+
+#else
+    struct rlimit	limit;		// Memmory limits
+
+    if (getrlimit(RLIMIT_DATA, &limit))
+      max_size = 16 * 1024 * 1024;
+    else
+      max_size = limit.rlim_cur / 10;
+#endif // _WIN32
+  }
+
+  // Don't allow overlarge limits...
+  if (max_size > (1024 * 1024 * 1024))	// Max 1GB total size
+    max_size = 1024 * 1024 * 1024;
+  if (max_width > 65535)		// Max 65535 wide
+    max_width = 65535;
+  if (max_height > 65535)		// Max 65535 high
+    max_height = 65535;
+
+  // Update values
+  _papplRWLockWrite(system);
+
+  system->max_image_size   = max_size;
+  system->max_image_width  = max_width == 0 ? 16384 : max_width;
+  system->max_image_height = max_height == 0 ? 16384 : max_height;
+
+  _papplSystemConfigChanged(system);
+
+  _papplRWUnlock(system);
 }
 
 
@@ -1810,13 +2051,13 @@ papplSystemSetMaxLogSize(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     system->logmaxsize = maxsize;
 
     _papplSystemConfigChanged(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1837,13 +2078,13 @@ papplSystemSetMaxSubscriptions(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     system->max_subscriptions = max_subscriptions;
 
     _papplSystemConfigChanged(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1870,12 +2111,44 @@ papplSystemSetMIMECallback(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     system->mime_cb     = cb;
     system->mime_cbdata = data;
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
+  }
+}
+
+
+//
+// 'papplSystemSetNetworkCallbacks()' - Set the network configuration callbacks.
+//
+// This function sets the network configuration callbacks for a system.  The
+// "get" callback reads the configuration of all network interfaces and stores
+// them in an array of @link pappl_network_t@ structures that is passed to the
+// callback.  The "set" callback writes the configuration of all network
+// interfaces and returns a boolean value indicating whether the configuration
+// has been written successfully.
+//
+
+void
+papplSystemSetNetworkCallbacks(
+    pappl_system_t         *system,	// I - System
+    pappl_network_get_cb_t get_cb,	// I - "Get networks" callback
+    pappl_network_set_cb_t set_cb,	// I - "Set networks" callback
+    void                   *cb_data)	// I - Callback data
+{
+  // Range check input...
+  if (system && (get_cb != NULL) == (set_cb != NULL))
+  {
+    _papplRWLockWrite(system);
+
+    system->network_get_cb = get_cb;
+    system->network_set_cb = set_cb;
+    system->network_cbdata = cb_data;
+
+    _papplRWUnlock(system);
   }
 }
 
@@ -1898,13 +2171,13 @@ papplSystemSetNextPrinterID(
 {
   if (system && !system->is_running)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     system->next_printer_id = next_printer_id;
 
     _papplSystemConfigChanged(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1928,10 +2201,10 @@ papplSystemSetOperationCallback(
 {
   if (system && !system->is_running)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
     system->op_cb     = cb;
     system->op_cbdata = data;
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1950,14 +2223,14 @@ papplSystemSetOrganization(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     free(system->organization);
     system->organization = value ? strdup(value) : NULL;
 
     _papplSystemConfigChanged(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -1977,14 +2250,14 @@ papplSystemSetOrganizationalUnit(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     free(system->org_unit);
     system->org_unit = value ? strdup(value) : NULL;
 
     _papplSystemConfigChanged(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -2006,13 +2279,13 @@ papplSystemSetPassword(
 {
   if (system && hash)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     papplCopyString(system->password_hash, hash, sizeof(system->password_hash));
 
     _papplSystemConfigChanged(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -2051,16 +2324,16 @@ papplSystemSetPrinterDrivers(
 {
   if (system)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
-    system->num_drivers   = (size_t)num_drivers;
+    system->num_drivers   = (cups_len_t)num_drivers;
     system->drivers       = drivers;
     system->autoadd_cb    = autoadd_cb;
     system->create_cb     = create_cb;
     system->driver_cb     = driver_cb;
     system->driver_cbdata = data;
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -2090,10 +2363,10 @@ papplSystemSetSaveCallback(
 {
   if (system && !system->is_running)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
     system->save_cb     = cb;
     system->save_cbdata = data;
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -2115,7 +2388,7 @@ papplSystemSetUUID(
 {
   if (system && !system->is_running)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     free(system->uuid);
 
@@ -2133,7 +2406,7 @@ papplSystemSetUUID(
 
     _papplSystemRegisterDNSSDNoLock(system);
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -2153,16 +2426,16 @@ papplSystemSetVersions(
 {
   if (system && num_versions && versions)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
 
     if (num_versions > (int)(sizeof(system->versions) / sizeof(system->versions[0])))
       system->num_versions = sizeof(system->versions) / sizeof(system->versions[0]);
     else
-      system->num_versions = (size_t)num_versions;
+      system->num_versions = (cups_len_t)num_versions;
 
-    memcpy(system->versions, versions, system->num_versions * sizeof(pappl_version_t));
+    memcpy(system->versions, versions, (size_t)system->num_versions * sizeof(pappl_version_t));
 
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -2193,12 +2466,12 @@ papplSystemSetWiFiCallbacks(
 {
   if (system && !system->is_running && join_cb && status_cb)
   {
-    pthread_rwlock_wrlock(&system->rwlock);
+    _papplRWLockWrite(system);
     system->wifi_join_cb   = join_cb;
     system->wifi_list_cb   = list_cb;
     system->wifi_status_cb = status_cb;
     system->wifi_cbdata    = data;
-    pthread_rwlock_unlock(&system->rwlock);
+    _papplRWUnlock(system);
   }
 }
 
@@ -2245,7 +2518,7 @@ add_listeners(
 	  if (name && *name == '/')
 	    papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to create listener socket for '%s': %s", name, cupsLastErrorString());
 	  else
-	    papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to create listener socket for '%s:%d': %s", httpAddrString(&addr->addr, temp, (int)sizeof(temp)), system->port, cupsLastErrorString());
+	    papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to create listener socket for '%s:%d': %s", httpAddrGetString(&addr->addr, temp, (cups_len_t)sizeof(temp)), system->port, cupsLastErrorString());
 	}
       }
       else
@@ -2286,6 +2559,27 @@ compare_filters(_pappl_mime_filter_t *a,// I - First filter
 }
 
 
+//
+// 'compare_timers()' - Compare two timers.
+//
+
+static int				// O - Result of comparison
+compare_timers(_pappl_timer_t *a,	// I - First timer
+               _pappl_timer_t *b)	// I - Second timer
+{
+  if (a->next < b->next)
+    return (-1);
+  else if (a->next > b->next)
+    return (1);
+  else if (a < b)
+    return (-1);
+  else if (a > b)
+    return (1);
+  else
+    return (0);
+}
+
+
 //
 // 'copy_filter()' - Copy a filter definition.
 //
