@@ -26,15 +26,23 @@ typedef struct _pappl_attr_s		// Input attribute structure
   cups_len_t	max_count;		// Max number of values
 } _pappl_attr_t;
 
+typedef struct _pappl_device_s		// Find devices callback data
+{
+  pappl_client_t *client;		// Client connection
+  ipp_attribute_t *device_col;		// "smi55357-device-col" attribute
+} _pappl_device_t;
 
 //
 // Local functions...
 //
 
+static bool	find_devices_cb(const char *device_info, const char *device_uri, const char *device_id, _pappl_device_t *data);
 static void	ipp_create_printer(pappl_client_t *client);
 static void	ipp_delete_printer(pappl_client_t *client);
 static void	ipp_disable_all_printers(pappl_client_t *client);
 static void	ipp_enable_all_printers(pappl_client_t *client);
+static void	ipp_find_devices(pappl_client_t *client);
+static void	ipp_find_drivers(pappl_client_t *client);
 static void	ipp_get_printers(pappl_client_t *client);
 static void	ipp_get_system_attributes(pappl_client_t *client);
 static void	ipp_pause_all_printers(pappl_client_t *client);
@@ -42,6 +50,7 @@ static void	ipp_resume_all_printers(pappl_client_t *client);
 static void	ipp_set_system_attributes(pappl_client_t *client);
 static void	ipp_shutdown_all_printers(pappl_client_t *client);
 
+
 //
 // '_papplSystemProcessIPP()' - Process an IPP System request.
 //
@@ -50,7 +59,7 @@ void
 _papplSystemProcessIPP(
     pappl_client_t *client)		// I - Client
 {
-  switch (ippGetOperation(client->request))
+  switch ((int)ippGetOperation(client->request))
   {
     case IPP_OP_CREATE_PRINTER :
 	ipp_create_printer(client);
@@ -124,6 +133,14 @@ _papplSystemProcessIPP(
         _papplSubscriptionIPPGetNotifications(client);
         break;
 
+    case IPP_OP_PAPPL_FIND_DEVICES :
+        ipp_find_devices(client);
+        break;
+
+    case IPP_OP_PAPPL_FIND_DRIVERS :
+        ipp_find_drivers(client);
+        break;
+
     default :
         if (client->system->op_cb && (client->system->op_cb)(client, client->system->op_cbdata))
           break;
@@ -134,6 +151,38 @@ _papplSystemProcessIPP(
 }
 
 
+//
+// 'find_devices_cb()' - Add a device to the response.
+//
+
+static bool				// O - `true` to continue, `false` to stop
+find_devices_cb(
+    const char      *device_info,	// I - Device info
+    const char      *device_uri,	// I - Device URI
+    const char      *device_id,		// I - Device ID
+    _pappl_device_t *data)		// I - Callback data
+{
+  ipp_t	*col;				// Collection value
+
+
+  // Create a new collection value for "smi55357-device-col"...
+  col = ippNew();
+  ippAddString(col, IPP_TAG_ZERO, IPP_TAG_TEXT, "smi55357-device-id", NULL, device_id);
+  ippAddString(col, IPP_TAG_ZERO, IPP_TAG_TEXT, "smi55357-device-info", NULL, device_info);
+  ippAddString(col, IPP_TAG_ZERO, IPP_TAG_URI, "smi55357-device-uri", NULL, device_uri);
+
+  // Add or update the attribute...
+  if (data->device_col)
+    ippSetCollection(data->client->response, &data->device_col, ippGetCount(data->device_col), col);
+  else
+    data->device_col = ippAddCollection(data->client->response, IPP_TAG_SYSTEM, "smi55357-device-col", col);
+
+  ippDelete(col);
+
+  return (true);
+}
+
+
 //
 // 'ipp_create_printer()' - Create a printer.
 //
@@ -143,7 +192,7 @@ ipp_create_printer(
     pappl_client_t *client)		// I - Client
 {
   const char	*printer_name,		// Printer name
-		*device_id,		// Device URI
+		*device_id,		// Device ID
 		*device_uri,		// Device URI
 		*driver_name;		// Name of driver
   ipp_attribute_t *attr;		// Current attribute
@@ -199,12 +248,12 @@ ipp_create_printer(
   else
     device_id = ippGetString(attr, 0, NULL);
 
-  if ((attr = ippFindAttribute(client->request, "smi2699-device-uri", IPP_TAG_ZERO)) == NULL)
+  if ((attr = ippFindAttribute(client->request, "smi55357-device-uri", IPP_TAG_ZERO)) == NULL)
   {
-    papplClientRespondIPP(client, IPP_STATUS_ERROR_BAD_REQUEST, "Missing 'smi2699-device-uri' attribute in request.");
+    papplClientRespondIPP(client, IPP_STATUS_ERROR_BAD_REQUEST, "Missing 'smi55357-device-uri' attribute in request.");
     return;
   }
-  else if (ippGetGroupTag(attr) != IPP_TAG_PRINTER || ippGetValueTag(attr) != IPP_TAG_URI || ippGetCount(attr) != 1)
+  else if (ippGetGroupTag(attr) != IPP_TAG_OPERATION || ippGetValueTag(attr) != IPP_TAG_URI || ippGetCount(attr) != 1)
   {
     papplClientRespondIPPUnsupported(client, attr);
     return;
@@ -220,12 +269,12 @@ ipp_create_printer(
     }
   }
 
-  if ((attr = ippFindAttribute(client->request, "smi2699-device-command", IPP_TAG_ZERO)) == NULL)
+  if ((attr = ippFindAttribute(client->request, "smi55357-driver", IPP_TAG_ZERO)) == NULL)
   {
-    papplClientRespondIPP(client, IPP_STATUS_ERROR_BAD_REQUEST, "Missing 'smi2699-device-command' attribute in request.");
+    papplClientRespondIPP(client, IPP_STATUS_ERROR_BAD_REQUEST, "Missing 'smi55357-driver' attribute in request.");
     return;
   }
-  else if (ippGetGroupTag(attr) != IPP_TAG_PRINTER || ippGetValueTag(attr) != IPP_TAG_KEYWORD || ippGetCount(attr) != 1)
+  else if (ippGetGroupTag(attr) != IPP_TAG_OPERATION || ippGetValueTag(attr) != IPP_TAG_KEYWORD || ippGetCount(attr) != 1)
   {
     papplClientRespondIPPUnsupported(client, attr);
     return;
@@ -253,7 +302,7 @@ ipp_create_printer(
     else if (errno == EIO)
     {
       papplClientRespondIPP(client, IPP_STATUS_ERROR_ATTRIBUTES_OR_VALUES, "Driver '%s' cannot be used with this printer.", driver_name);
-      ippAddString(client->response, IPP_TAG_UNSUPPORTED_GROUP, IPP_TAG_KEYWORD, "smi2699-device-command", NULL, driver_name);
+      ippAddString(client->response, IPP_TAG_UNSUPPORTED_GROUP, IPP_TAG_KEYWORD, "smi55357-driver", NULL, driver_name);
     }
     else if (errno == EINVAL)
     {
@@ -282,7 +331,12 @@ ipp_create_printer(
   cupsArrayAdd(ra, "printer-uuid");
   cupsArrayAdd(ra, "printer-xri-supported");
 
-  _papplPrinterCopyAttributes(printer, client, ra, NULL);
+  _papplRWLockRead(printer->system);
+  _papplRWLockRead(printer);
+  _papplPrinterCopyAttributesNoLock(printer, client, ra, NULL);
+  _papplRWUnlock(printer);
+  _papplRWUnlock(printer->system);
+
   cupsArrayDelete(ra);
 }
 
@@ -311,10 +365,19 @@ ipp_delete_printer(
     return;
   }
 
+  _papplRWLockWrite(client->printer);
   if (!client->printer->processing_job)
+  {
+    // Not busy, delete immediately...
+    _papplRWUnlock(client->printer);
     papplPrinterDelete(client->printer);
+  }
   else
+  {
+    // Busy, delete when current job is completed...
     client->printer->is_deleted = true;
+    _papplRWUnlock(client->printer);
+  }
 
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
 }
@@ -342,12 +405,12 @@ ipp_disable_all_printers(
   }
 
   // Loop through the printers...
-  pthread_rwlock_rdlock(&system->rwlock);
+  _papplRWLockRead(system);
   for (printer = (pappl_printer_t *)cupsArrayGetFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayGetNext(system->printers))
   {
     papplPrinterDisable(printer);
   }
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 }
 
 
@@ -373,12 +436,230 @@ ipp_enable_all_printers(
   }
 
   // Loop through the printers...
-  pthread_rwlock_rdlock(&system->rwlock);
+  _papplRWLockRead(system);
   for (printer = (pappl_printer_t *)cupsArrayGetFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayGetNext(system->printers))
   {
     papplPrinterEnable(printer);
   }
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
+}
+
+
+//
+// 'ipp_find_devices()' - Find devices.
+//
+
+static void
+ipp_find_devices(
+    pappl_client_t *client)		// I - Client
+{
+  http_status_t		auth_status;	// Authorization status
+  ipp_attribute_t	*type_attr;	// "smi55357-device-type" attribute
+  cups_len_t		i,		// Looping var
+			count;		// Number of values
+  pappl_devtype_t	types;		// Device types
+  _pappl_device_t	data;		// Callback data
+
+
+  // Verify the connection is authorized...
+  if ((auth_status = papplClientIsAuthorized(client)) != HTTP_STATUS_CONTINUE)
+  {
+    papplClientRespond(client, auth_status, NULL, NULL, 0, 0);
+    return;
+  }
+
+  // Get the device type bits...
+  if ((type_attr = ippFindAttribute(client->request, "smi55357-device-type", IPP_TAG_KEYWORD)) != NULL)
+  {
+    for (types = (pappl_devtype_t)0, i = 0, count = ippGetCount(type_attr); i < count; i ++)
+    {
+      const char *type = ippGetString(type_attr, i, NULL);
+					// Current type keyword
+
+      if (!strcmp(type, "all"))
+      {
+        types = PAPPL_DEVTYPE_ALL;
+        break;
+      }
+      else if (!strcmp(type, "dns-sd"))
+      {
+        types |= PAPPL_DEVTYPE_DNS_SD;
+      }
+      else if (!strcmp(type, "local"))
+      {
+        types |= PAPPL_DEVTYPE_LOCAL;
+      }
+      else if (!strcmp(type, "network"))
+      {
+        types |= PAPPL_DEVTYPE_NETWORK;
+      }
+      else if (!strcmp(type, "other-local"))
+      {
+        types |= PAPPL_DEVTYPE_CUSTOM_LOCAL;
+      }
+      else if (!strcmp(type, "other-network"))
+      {
+        types |= PAPPL_DEVTYPE_CUSTOM_NETWORK;
+      }
+      else if (!strcmp(type, "snmp"))
+      {
+        types |= PAPPL_DEVTYPE_SNMP;
+      }
+      else if (!strcmp(type, "usb"))
+      {
+        types |= PAPPL_DEVTYPE_USB;
+      }
+      else
+      {
+        papplClientRespondIPPUnsupported(client, type_attr);
+        return;
+      }
+    }
+  }
+  else
+  {
+    // Report all devices...
+    types = PAPPL_DEVTYPE_ALL;
+  }
+
+  // List all devices
+  papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
+
+  memset(&data, 0, sizeof(data));
+  data.client = client;
+
+  papplDeviceList(types, (pappl_device_cb_t)find_devices_cb, &data, papplLogDevice, client->system);
+
+  if (!data.device_col)
+    papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_FOUND, "No devices found.");
+}
+
+
+//
+// 'ipp_find_drivers()' - Find drivers.
+//
+
+static void
+ipp_find_drivers(
+    pappl_client_t *client)		// I - Client
+{
+  http_status_t	auth_status;		// Authorization status
+  const char	*device_id;		// Device ID
+  int		num_dids = 0;		// Number of key/value pairs
+  cups_option_t	*dids = NULL;		// Device ID key/value pairs
+  const char	*driver_name = NULL,	// Matching driver name, if any
+		*cmd = NULL,		// Command set from device ID
+		*make = NULL,		// Make from device ID
+		*model = NULL;		// Model from device ID
+  cups_len_t	i;			// Looping var
+  pappl_pr_driver_t *driver;		// Current driver
+  ipp_attribute_t *driver_col = NULL;	// Collection for drivers
+  ipp_t		*col;			// Collection value
+
+
+  // Verify the connection is authorized...
+  if ((auth_status = papplClientIsAuthorized(client)) != HTTP_STATUS_CONTINUE)
+  {
+    papplClientRespond(client, auth_status, NULL, NULL, 0, 0);
+    return;
+  }
+
+  // Assemble a list of drivers...
+  papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
+
+  // See if the Client provided a device ID to match...
+  if ((device_id = ippGetString(ippFindAttribute(client->request, "smi55357-device-id", IPP_TAG_TEXT), 0, NULL)) != NULL)
+  {
+    // Yes, filter based on device ID...
+    if (client->system->autoadd_cb)
+    {
+      // Filter using the auto-add callback...
+      if ((driver_name = (client->system->autoadd_cb)(NULL, NULL, device_id, client->system->driver_cbdata)) == NULL)
+      {
+        // No matching driver...
+	papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_FOUND, "No matching driver found.");
+        return;
+      }
+    }
+    else
+    {
+      // Filter using device ID matching...
+      num_dids = papplDeviceParseID(device_id, &dids);
+
+      if ((cmd = cupsGetOption("COMMAND SET", num_dids, dids)) == NULL)
+        cmd = cupsGetOption("CMD", num_dids, dids);
+
+      if ((make = cupsGetOption("MANUFACTURER", num_dids, dids)) == NULL)
+        make = cupsGetOption("MFG", num_dids, dids);
+
+      if ((model = cupsGetOption("MODEL", num_dids, dids)) == NULL)
+        model = cupsGetOption("MDL", num_dids, dids);
+    }
+  }
+
+  for (i = client->system->num_drivers, driver = client->system->drivers; i > 0; i --, driver ++)
+  {
+    if (driver_name)
+    {
+      // Compare driver name...
+      if (strcmp(driver_name, driver->name))
+        continue;
+    }
+    else if (num_dids > 0)
+    {
+      // Compare device ID values...
+      int		num_dids2;	// Number of device ID key/value pairs
+      cups_option_t	*dids2;		// Device ID key/value pairs
+      const char	*cmd2,		// Command set from device ID
+			*make2,		// Make from device ID
+			*model2;	// Model from device ID
+      bool		match = true;	// Do we have a match?
+
+      num_dids2 = papplDeviceParseID(driver->device_id, &dids2);
+
+      if ((cmd2 = cupsGetOption("COMMAND SET", num_dids2, dids2)) == NULL)
+        cmd2 = cupsGetOption("CMD", num_dids2, dids2);
+
+      if (cmd && cmd2 && !strstr(cmd, cmd2))
+        match = false;
+
+      if ((make2 = cupsGetOption("MANUFACTURER", num_dids2, dids2)) == NULL)
+        make2 = cupsGetOption("MFG", num_dids2, dids2);
+
+      if (make && make2 && strcmp(make, make2))
+        match = false;
+
+      if ((model2 = cupsGetOption("MODEL", num_dids2, dids2)) == NULL)
+        model2 = cupsGetOption("MDL", num_dids2, dids2);
+
+      if (model && model2 && strcmp(model, model2))
+        match = false;
+
+      cupsFreeOptions(num_dids2, dids2);
+
+      if (!match)
+        continue;
+    }
+
+    // Create a collection for the driver values...
+    col = ippNew();
+    if (driver->device_id)
+      ippAddString(col, IPP_TAG_ZERO, IPP_TAG_TEXT, "smi55357-device-id", NULL, driver->device_id);
+    ippAddString(col, IPP_TAG_ZERO, IPP_TAG_KEYWORD, "smi55357-driver", NULL, driver->name);
+    ippAddString(col, IPP_TAG_ZERO, IPP_TAG_TEXT, "smi55357-driver-info", NULL, driver->description);
+
+    if (driver_col)
+      ippSetCollection(client->response, &driver_col, ippGetCount(driver_col), col);
+    else
+      driver_col = ippAddCollection(client->response, IPP_TAG_SYSTEM, "smi55357-driver-col", col);
+
+    ippDelete(col);
+  }
+
+  cupsFreeOptions(num_dids, dids);
+
+  if (!driver_col)
+    papplClientRespondIPP(client, IPP_STATUS_ERROR_NOT_FOUND, "No matching drivers found.");
 }
 
 
@@ -393,7 +674,7 @@ ipp_get_printers(
   pappl_system_t	*system = client->system;
 					// System
   cups_array_t		*ra;		// Requested attributes array
-  size_t		i,		// Looping var
+  cups_len_t		i,		// Looping var
 			count,		// Number of printers
 			limit;		// Maximum number to return
   pappl_printer_t	*printer;	// Current printer
@@ -401,13 +682,13 @@ ipp_get_printers(
 
 
   // Get request attributes...
-  limit  = (size_t)ippGetInteger(ippFindAttribute(client->request, "limit", IPP_TAG_INTEGER), 0);
+  limit  = (cups_len_t)ippGetInteger(ippFindAttribute(client->request, "limit", IPP_TAG_INTEGER), 0);
   ra     = ippCreateRequestedArray(client->request);
   format = ippGetString(ippFindAttribute(client->request, "document-format", IPP_TAG_MIMETYPE), 0, NULL);
 
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
 
-  pthread_rwlock_rdlock(&system->rwlock);
+  _papplRWLockRead(system);
 
   // Enumerate the printers for the client...
   count = cupsArrayGetCount(system->printers);
@@ -425,12 +706,12 @@ ipp_get_printers(
     if (i)
       ippAddSeparator(client->response);
 
-    pthread_rwlock_rdlock(&printer->rwlock);
-    _papplPrinterCopyAttributes(printer, client, ra, format);
-    pthread_rwlock_unlock(&printer->rwlock);
+    _papplRWLockRead(printer);
+    _papplPrinterCopyAttributesNoLock(printer, client, ra, format);
+    _papplRWUnlock(printer);
   }
 
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 
   cupsArrayDelete(ra);
 }
@@ -447,7 +728,7 @@ ipp_get_system_attributes(
   pappl_system_t	*system = client->system;
 					// System
   cups_array_t		*ra;		// Requested attributes array
-  size_t		i,		// Looping var
+  cups_len_t		i,		// Looping var
 			count;		// Count of values
   pappl_printer_t	*printer;	// Current printer
   ipp_attribute_t	*attr;		// Current attribute
@@ -461,7 +742,7 @@ ipp_get_system_attributes(
 
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
 
-  pthread_rwlock_rdlock(&system->rwlock);
+  _papplRWLockRead(system);
 
   _papplCopyAttributes(client->response, system->attrs, ra, IPP_TAG_ZERO, IPP_TAG_CUPS_CONST);
 
@@ -492,16 +773,16 @@ ipp_get_system_attributes(
 
       col = ippNew();
 
-      pthread_rwlock_rdlock(&printer->rwlock);
+      _papplRWLockRead(printer);
 
       ippAddInteger(col, IPP_TAG_SYSTEM, IPP_TAG_INTEGER, "printer-id", printer->printer_id);
       ippAddString(col, IPP_TAG_SYSTEM, IPP_TAG_TEXT, "printer-info", NULL, printer->name);
       ippAddString(col, IPP_TAG_SYSTEM, IPP_TAG_TEXT, "printer-name", NULL, printer->name);
       ippAddString(col, IPP_TAG_SYSTEM, IPP_TAG_KEYWORD, "printer-service-type", NULL, "print");
-      _papplPrinterCopyState(printer, IPP_TAG_PRINTER, col, client, NULL);
-      _papplPrinterCopyXRI(printer, col, client);
+      _papplPrinterCopyStateNoLock(printer, IPP_TAG_PRINTER, col, client, NULL);
+      _papplPrinterCopyXRINoLock(printer, col, client);
 
-      pthread_rwlock_unlock(&printer->rwlock);
+      _papplRWUnlock(printer);
 
       ippSetCollection(client->response, &attr, IPP_NUM_CAST i, col);
       ippDelete(col);
@@ -630,7 +911,7 @@ ipp_get_system_attributes(
     ippDelete(col);
   }
 
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 
   cupsArrayDelete(ra);
 }
@@ -658,12 +939,12 @@ ipp_pause_all_printers(
   }
 
   // Loop through the printers...
-  pthread_rwlock_rdlock(&system->rwlock);
+  _papplRWLockRead(system);
   for (printer = (pappl_printer_t *)cupsArrayGetFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayGetNext(system->printers))
   {
     papplPrinterPause(client->printer);
   }
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 }
 
 
@@ -689,12 +970,12 @@ ipp_resume_all_printers(
   }
 
   // Loop through the printers...
-  pthread_rwlock_rdlock(&system->rwlock);
+  _papplRWLockRead(system);
   for (printer = (pappl_printer_t *)cupsArrayGetFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayGetNext(system->printers))
   {
     papplPrinterResume(client->printer);
   }
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 }
 
 
@@ -733,7 +1014,7 @@ ipp_set_system_attributes(
   }
 
   // Preflight request attributes...
-  for (rattr = ippFirstAttribute(client->request); rattr; rattr = ippNextAttribute(client->request))
+  for (rattr = ippGetFirstAttribute(client->request); rattr; rattr = ippGetNextAttribute(client->request))
   {
     papplLogClient(client, PAPPL_LOGLEVEL_DEBUG, "%s %s %s%s ...", ippTagString(ippGetGroupTag(rattr)), ippGetName(rattr), ippGetCount(rattr) > 1 ? "1setOf " : "", ippTagString(ippGetValueTag(rattr)));
 
@@ -774,9 +1055,9 @@ ipp_set_system_attributes(
     return;
 
   // Now apply changes...
-  pthread_rwlock_wrlock(&system->rwlock);
+  _papplRWLockWrite(system);
 
-  for (rattr = ippFirstAttribute(client->request); rattr; rattr = ippNextAttribute(client->request))
+  for (rattr = ippGetFirstAttribute(client->request); rattr; rattr = ippGetNextAttribute(client->request))
   {
     if (ippGetGroupTag(rattr) == IPP_TAG_OPERATION)
       continue;
@@ -816,7 +1097,7 @@ ipp_set_system_attributes(
 
   system->config_changes ++;
 
-  pthread_rwlock_unlock(&system->rwlock);
+  _papplRWUnlock(system);
 
   papplSystemAddEvent(system, NULL, NULL, PAPPL_EVENT_SYSTEM_CONFIG_CHANGED, NULL);
 
