@@ -197,6 +197,7 @@ papplSystemCreate(
   system->max_subscriptions = 100;
 
   papplSystemSetMaxClients(system, 0);
+  papplSystemSetMaxImageSize(system, 0, 0, 0);
 
   if (!system->name || !system->dns_sd_name || (spooldir && !system->directory) || (logfile && !system->logfile) || (subtypes && !system->subtypes) || (auth_service && !system->auth_service))
     goto fatal;
@@ -277,7 +278,8 @@ void
 papplSystemDelete(
     pappl_system_t *system)		// I - System object
 {
-  size_t	i;			// Looping var
+  cups_len_t	i;			// Looping var
+  _pappl_timer_t *t;			// Current timer
 
 
   if (!system || system->is_running)
@@ -321,6 +323,13 @@ papplSystemDelete(
   pthread_cond_destroy(&system->subscription_cond);
   pthread_mutex_destroy(&system->subscription_mutex);
 
+  for (t = (_pappl_timer_t *)cupsArrayGetFirst(system->timers); t; t = (_pappl_timer_t *)cupsArrayGetNext(system->timers))
+  {
+    cupsArrayRemove(system->timers, t);
+    free(t);
+  }
+  cupsArrayDelete(system->timers);
+
   pthread_rwlock_destroy(&system->rwlock);
   pthread_rwlock_destroy(&system->session_rwlock);
   pthread_mutex_destroy(&system->config_mutex);
@@ -381,9 +390,10 @@ _papplSystemMakeUUID(
 void
 papplSystemRun(pappl_system_t *system)	// I - System
 {
-  size_t		i,		// Looping var
+  cups_len_t		i,		// Looping var
 			count;		// Number of listeners that fired
-  int			pcount;		// Poll count
+  int			pcount,		// Poll count
+			ptimeout;	// Poll timeout
   pappl_client_t	*client;	// New client
   char			header[HTTP_MAX_VALUE];
 					// Server: header value
@@ -391,15 +401,18 @@ papplSystemRun(pappl_system_t *system)	// I - System
 					// Current number of host name changes
   pappl_printer_t	*printer;	// Current printer
   pthread_attr_t	tattr;		// Thread creation attributes
-  time_t		curtime,	// Current time
+  struct timeval	curtime;	// Current time
+  time_t		next,		// Next time for scheduling...
 			subtime = 0;	// Subscription checking time
+  _pappl_timer_t	*timer;		// Current timer
+  bool			save_changes;	// Save changes?
 
 
   // Range check...
   if (!system)
     return;
 
-  if (system->is_running)
+  if (papplSystemIsRunning(system))
   {
     papplLog(system, PAPPL_LOGLEVEL_FATAL, "Tried to run system when already running.");
     return;
@@ -411,7 +424,9 @@ papplSystemRun(pappl_system_t *system)	// I - System
     return;
   }
 
+  _papplRWLockWrite(system);
   system->is_running = true;
+  _papplRWUnlock(system);
 
   // Add fallback resources...
   papplSystemAddResourceData(system, "/favicon.png", "image/png", icon_md_png, sizeof(icon_md_png));
@@ -556,7 +571,32 @@ papplSystemRun(pappl_system_t *system)	// I - System
       _papplLogOpen(system);
     }
 
-    if ((pcount = poll(system->listeners, (nfds_t)system->num_listeners, 1000)) < 0 && errno != EINTR && errno != EAGAIN)
+    gettimeofday(&curtime, NULL);
+
+    _papplRWLockRead(system);
+
+    if (system->shutdown_time || sigterm_time)
+      next = curtime.tv_sec + 1;
+    else
+      next = curtime.tv_sec + 30;
+
+    if ((timer = (_pappl_timer_t *)cupsArrayGetFirst(system->timers)) != NULL && timer->next < next)
+      next = timer->next;
+
+    if (system->clean_time && system->clean_time < next)
+      next = system->clean_time;
+
+    if (subtime < next && cupsArrayGetCount(system->subscriptions) > 0)
+      next = subtime;
+
+    _papplRWUnlock(system);
+
+    if (next <= curtime.tv_sec)
+      ptimeout = 0;
+    else
+      ptimeout = 1000 * (int)(next - curtime.tv_sec) - (int)curtime.tv_usec / 1000;
+
+    if ((pcount = poll(system->listeners, (nfds_t)system->num_listeners, ptimeout)) < 0 && errno != EINTR && errno != EAGAIN)
     {
       papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to accept new connections: %s", strerror(errno));
       break;
@@ -565,15 +605,15 @@ papplSystemRun(pappl_system_t *system)	// I - System
     if (pcount > 0)
     {
       // Accept client connections as needed...
-      for (i = 0; i < (size_t)system->num_listeners; i ++)
+      for (i = 0; i < (cups_len_t)system->num_listeners; i ++)
       {
 	if (system->listeners[i].revents & POLLIN)
 	{
 	  if ((client = _papplClientCreate(system, (int)system->listeners[i].fd)) != NULL)
 	  {
-	    pthread_rwlock_wrlock(&system->rwlock);
+	    _papplRWLockWrite(system);
 	    system->num_clients ++;
-	    pthread_rwlock_unlock(&system->rwlock);
+	    _papplRWUnlock(system);
 
 	    if (pthread_create(&client->thread_id, &tattr, (void *(*)(void *))_papplClientRun, client))
 	    {
@@ -585,16 +625,23 @@ papplSystemRun(pappl_system_t *system)	// I - System
 	}
       }
 
+      _papplRWLockRead(system);
+
       if (system->num_clients >= system->max_clients)
       {
-	for (i = 0; i < (size_t)system->num_listeners; i ++)
+	for (i = 0; i < system->num_listeners; i ++)
 	  system->listeners[i].events = 0;
       }
     }
-    else if (system->num_clients < system->max_clients)
+    else
     {
-      for (i = 0; i < (size_t)system->num_listeners; i ++)
-	system->listeners[i].events = POLLIN;
+      _papplRWLockRead(system);
+
+      if (system->num_clients < system->max_clients)
+      {
+	for (i = 0; i < system->num_listeners; i ++)
+	  system->listeners[i].events = POLLIN;
+      }
     }
 
     dns_sd_host_changes = _papplDNSSDGetHostChanges();
@@ -606,9 +653,7 @@ papplSystemRun(pappl_system_t *system)	// I - System
 					// Force re-registration?
 
       if (force_dns_sd)
-        papplSystemSetHostName(system, NULL);
-
-      pthread_rwlock_rdlock(&system->rwlock);
+        _papplSystemSetHostNameNoLock(system, NULL);
 
       if (system->dns_sd_collision || force_dns_sd)
         _papplSystemRegisterDNSSDNoLock(system);
@@ -623,18 +668,18 @@ papplSystemRun(pappl_system_t *system)	// I - System
 
       system->dns_sd_any_collision = false;
       system->dns_sd_host_changes  = dns_sd_host_changes;
-
-      pthread_rwlock_unlock(&system->rwlock);
     }
 
-    if (system->config_changes > system->save_changes)
+    _papplRWUnlock(system);
+
+    pthread_mutex_lock(&system->config_mutex);
+    save_changes = system->config_changes > system->save_changes;
+    pthread_mutex_unlock(&system->config_mutex);
+
+    if (save_changes)
     {
-      pthread_mutex_lock(&system->config_mutex);
-
       system->save_changes = system->config_changes;
 
-      pthread_mutex_unlock(&system->config_mutex);
-
       if (system->save_cb)
       {
         // Save the configuration...
@@ -642,49 +687,78 @@ papplSystemRun(pappl_system_t *system)	// I - System
       }
     }
 
+    _papplRWLockRead(system);
     if (system->shutdown_time || sigterm_time)
     {
       // Shutdown requested, see if we can do so safely...
-      size_t		jcount = 0;	// Number of active jobs
+      cups_len_t	jcount = 0;	// Number of active jobs
 
       // Force shutdown after 60 seconds
       if (system->shutdown_time && (time(NULL) - system->shutdown_time) > 60)
+      {
+        _papplRWUnlock(system);
         break;				// Shutdown-System request
+      }
 
       if (sigterm_time && (time(NULL) - sigterm_time) > 60)
+      {
+        _papplRWUnlock(system);
         break;				// SIGTERM received
+      }
 
       // Otherwise shutdown immediately if there are no more active jobs...
-      pthread_rwlock_rdlock(&system->rwlock);
       for (i = 0, count = cupsArrayGetCount(system->printers); i < count; i ++)
       {
 	printer = (pappl_printer_t *)cupsArrayGetElement(system->printers, i);
 
-        pthread_rwlock_rdlock(&printer->rwlock);
+        _papplRWLockRead(printer);
         jcount += cupsArrayGetCount(printer->active_jobs);
-        pthread_rwlock_unlock(&printer->rwlock);
+        _papplRWUnlock(printer);
       }
-      pthread_rwlock_unlock(&system->rwlock);
+      _papplRWUnlock(system);
 
       if (jcount == 0)
         break;
     }
+    else
+    {
+      _papplRWUnlock(system);
+    }
+
+    // Run any timers...
+    gettimeofday(&curtime, NULL);
+
+    _papplRWLockRead(system);
+    for (timer = (_pappl_timer_t *)cupsArrayGetFirst(system->timers); timer; timer = (_pappl_timer_t *)cupsArrayGetNext(system->timers))
+    {
+      if (timer->next > curtime.tv_sec)
+        break;
+
+      cupsArrayRemove(system->timers, timer);
+
+      if ((timer->cb)(system, timer->cb_data) && timer->interval)
+      {
+	timer->next += timer->interval;
+	cupsArrayAdd(system->timers, timer);
+      }
+    }
+    _papplRWUnlock(system);
 
     // Clean out old jobs and subscriptions...
-    curtime = time(NULL);
-
-    if (system->clean_time && curtime >= system->clean_time)
+    if (system->clean_time && curtime.tv_sec >= system->clean_time)
       papplSystemCleanJobs(system);
 
-    if (curtime >= subtime)
+    if (curtime.tv_sec >= subtime)
     {
       _papplSystemCleanSubscriptions(system, false);
-      subtime = curtime + 10;
+      subtime = curtime.tv_sec + 10;
     }
   }
 
   papplLog(system, PAPPL_LOGLEVEL_INFO, "Shutting down system.");
 
+  _papplRWLockWrite(system);
+
   ippDelete(system->attrs);
   system->attrs = NULL;
 
@@ -702,19 +776,36 @@ papplSystemRun(pappl_system_t *system)	// I - System
       _papplPrinterUnregisterDNSSDNoLock(printer);
   }
 
-  if (system->save_changes < system->config_changes && system->save_cb)
-  {
-    // Save the configuration...
-    (system->save_cb)(system, system->save_cbdata);
-  }
-
   system->is_running = false;
 
+  _papplRWUnlock(system);
+
+  pthread_mutex_lock(&system->config_mutex);
+  save_changes = system->config_changes > system->save_changes;
+  pthread_mutex_unlock(&system->config_mutex);
+
+  if (save_changes)
+  {
+    system->save_changes = system->config_changes;
+
+    if (system->save_cb)
+    {
+      // Save the configuration...
+      (system->save_cb)(system, system->save_cbdata);
+    }
+  }
+
   if ((system->options & PAPPL_SOPTIONS_USB_PRINTER) && (printer = papplSystemFindPrinter(system, NULL, system->default_printer_id, NULL)) != NULL)
   {
     // Wait for the USB gadget thread(s) to complete...
+    _papplRWLockRead(printer);
     while (printer->usb_active)
+    {
+      _papplRWUnlock(printer);
       usleep(100000);
+      _papplRWLockRead(printer);
+    }
+    _papplRWUnlock(printer);
   }
 }
 
@@ -742,8 +833,6 @@ papplSystemShutdown(
 static void
 make_attributes(pappl_system_t *system)	// I - System
 {
-  cups_len_t		i;		// Looping var
-  ipp_attribute_t	*attr;		// Attribute
   static const char * const charset[] =	// charset-supported values
   {
     "us-ascii",
@@ -776,7 +865,9 @@ make_attributes(pappl_system_t *system)	// I - System
     IPP_OP_SET_SYSTEM_ATTRIBUTES,
     IPP_OP_SHUTDOWN_ALL_PRINTERS,
     IPP_OP_CUPS_GET_DEFAULT,
-    IPP_OP_CUPS_GET_PRINTERS
+    IPP_OP_CUPS_GET_PRINTERS,
+    IPP_OP_PAPPL_FIND_DEVICES,
+    IPP_OP_PAPPL_FIND_DRIVERS
   };
   static const char * const printer_creation_attributes_supported[] =
   {					// "printer-creation-attributes-supported" Values
@@ -796,14 +887,14 @@ make_attributes(pappl_system_t *system)	// I - System
     "printer-location",
     "printer-name",
     "printer-resolution-default",
-    "smi2699-device-command",
-    "smi2699-device-uri"
+    "smi55357-device-uri",
+    "smi55357-driver"
   };
   static const char * const system_mandatory_printer_attributes[] =
   {					// "system-mandatory-printer-attributes" values
     "printer-name",
-    "smi2699-device-command",
-    "smi2699-device-uri"
+    "smi55357-device-uri",
+    "smi55357-driver"
   };
   static const char * const system_settable_attributes_supported[] =
   {					// "system-settable-attributes-supported" values
@@ -859,24 +950,7 @@ make_attributes(pappl_system_t *system)	// I - System
   // printer-creation-attributes-supported
   ippAddStrings(system->attrs, IPP_TAG_SYSTEM, IPP_CONST_TAG(IPP_TAG_KEYWORD), "printer-creation-attributes-supported", (int)(sizeof(printer_creation_attributes_supported) / sizeof(printer_creation_attributes_supported[0])), NULL, printer_creation_attributes_supported);
 
-  // smi2699-device-command-supported
-  if (system->num_drivers > 0)
-  {
-    size_t num_drivers = system->num_drivers;
-					// Number of drivers to report
-    if (system->autoadd_cb)
-      num_drivers ++;
-
-    attr = ippAddStrings(system->attrs, IPP_TAG_SYSTEM, IPP_CONST_TAG(IPP_TAG_NAME), "smi2699-device-command-supported", (cups_len_t)num_drivers, NULL, NULL);
-
-    for (i = 0; i < (cups_len_t)system->num_drivers; i ++)
-      ippSetString(system->attrs, &attr, i, system->drivers[i].name);
-
-    if (system->autoadd_cb)
-      ippSetString(system->attrs, &attr, (cups_len_t)system->num_drivers, "auto");
-  }
-
-  // smi2699-device-uri-schemes-supported
+  // smi55357-device-uri-schemes-supported
   _papplDeviceAddSupportedSchemes(system->attrs);
 
   // system-mandatory-printer-attributes
